package com.ruoyi.web.controller.IndexController;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.system.domain.EtOperatingArea;
import com.ruoyi.system.domain.vo.IndexAdminVo;
import com.ruoyi.system.domain.vo.IndexVo;
import com.ruoyi.system.domain.vo.LeaderboardVo;
import com.ruoyi.system.service.IEtOperatingAreaService;
import com.ruoyi.system.service.IEtOrderService;
import com.ruoyi.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

/**
 * index首页统计
 *  校验
 * @author ruoyi
 */
@RestController
@RequestMapping("/index")
public class IndexController extends BaseController
{

    @Autowired
    private IEtOrderService etOrderService;

    @Autowired
    private IEtOperatingAreaService etOperatingAreaService;

    @Resource
    private ISysUserService sysUserService;

    /**
     * 根据token获取运营区列表（后台）
     */
    @GetMapping("/getAreaList")
    public AjaxResult getAreaList()
    {
        List<EtOperatingArea> longs;
        SysUser sysUser = sysUserService.selectUserById(getUserId());
        if(getUserId() == 1){
            longs = etOperatingAreaService.selectAreaListByDeptId2(null,null);
        }else{
            if(ObjectUtil.isNotNull(sysUser.getAreaId())){
                longs = etOperatingAreaService.selectAreaListByDeptId2(null,sysUser.getAreaId());
            }else{
                if(ObjectUtil.isNotNull(sysUser.getDeptId())){
                    longs = etOperatingAreaService.selectAreaListByDeptId2(sysUser.getDeptId(),null);
                }else{
                    throw new ServiceException("该用户都没有绑定运营区和运营商（至少绑定一个）");
                }
            }
        }
        logger.info("根据token获取运营区列表：【{}】", JSON.toJSON(longs));
        return success(longs);
    }

    /**
     * 首页统计-运营商
     */
    @GetMapping(value = "/statistics")
    public AjaxResult statistics(Long areaId)
    {
        logger.info("【首页统计】请求参数: areaId={}", areaId);
        IndexVo indexVo = etOrderService.statistics(areaId);
        return success(indexVo);
    }


    /**
     * 首页统计-总管理
     */
    @GetMapping(value = "/admim/statistics")
    public AjaxResult admimStatistics()
    {
        logger.info("【首页统计】请求参数");
        IndexAdminVo indexAdminVo = etOrderService.admimStatistics();
        return success(indexAdminVo);
    }


    /**
     * 首页统计-排行榜
     * type：1-根据订单数排序，2-根据订单金额排序; 3-进行中的订单
     * timeLimit：1-今天，2-本周
     */
    @GetMapping(value = "/admim/leaderboard")
    public AjaxResult leaderboard(String type,String timeLimit)
    {
        logger.info("【首页统计】排行榜");
        List<LeaderboardVo> leaderboard = etOrderService.leaderboard(type, timeLimit);
        return success(leaderboard);
    }

}
