package com.ruoyi.web.controller.app;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysDictData;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.CommonUtil;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.verify.vo.IDResponse;
import com.ruoyi.framework.web.service.TokenService;
import com.ruoyi.system.domain.*;
import com.ruoyi.system.domain.response.FaultResponse;
import com.ruoyi.system.domain.response.OrderResponse;
import com.ruoyi.system.domain.vo.*;
import com.ruoyi.system.mapper.*;
import com.ruoyi.system.service.*;
import com.wechat.pay.java.service.payments.jsapi.model.PrepayWithRequestPaymentResponse;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * app接口（需要登录校验的）
 *  校验
 * @author ruoyi
 */
@RestController
@RequestMapping("/appVerify")
public class AppVerifyController extends BaseController
{
    @Autowired
    private IEtFaultService etFaultService;

    @Autowired
    private IAsDeviceService asDeviceService;

    @Autowired
    private IEtOrderService etOrderService;

    @Autowired
    private IAsUserService asUserService;

    @Autowired
    private ISysDictDataService dictDataService;

    @Autowired
    private IEtAdminOrderService etAdminOrderService;

    @Autowired
    private IEtAdminOrderHistoryService etAdminOrderHistoryService;

    @Autowired
    private IEtParkingAreaService etParkingAreaService;

    @Autowired
    private TokenService tokenService;

    @Autowired
    private IEtOperatingAreaService etOperatingAreaService;

    @Resource
    private AsDeviceMapper asDeviceMapper;

    @Resource
    private ISysUserService sysUserService;

    @Resource
    private AsUserMapper asUserMapper;

    @Resource
    private EtOrderMapper etOrderMapper;

    @Resource
    private ISysUserService userService;

    @Autowired
    private ISysDeptService deptService;

    @Autowired
    private IEtCapitalFlowService etCapitalFlowService;

    @Autowired
    private IWxPayService wxPayService;

    @Resource
    private EtCapitalFlowMapper etCapitalFlowMapper;

    @Resource
    private EtOperatingAreaMapper etOperatingAreaMapper;

    @Autowired
    private IEtCouponService etCouponService;

    @Autowired
    private IEtCouponClaimLogService etCouponClaimLogService;


    /**
     * 故障上报
     */
    @Log(title = "故障上报", businessType = BusinessType.INSERT)
    @PostMapping("/fault")
    public AjaxResult add(@RequestBody EtFault etFault)
    {
        return toAjax(etFaultService.insertEtFault(etFault));
    }

    /**
     * 获取故障上报详细信息
     */
    @GetMapping(value = "/fault/{faultId}")
    public AjaxResult getFaultInfo(@PathVariable("faultId") Long faultId)
    {
        return success(etFaultService.selectEtFaultByFaultId(faultId));
    }

    /**
     * 查询故障上报列表
     */
    @GetMapping("/fault/list")
    public TableDataInfo faultList(EtFault etFault)
    {
        startPage();
        List<EtFault> list = etFaultService.selectEtFaultList(etFault);
        return getDataTable(list);
    }

    /**
     * 所有的故障类型（分车头、车身）
     */
    @GetMapping("/fault/allList")
    public AjaxResult list(SysDictData dictData)
    {
        FaultResponse faultResponse = new FaultResponse();
        List<SysDictData> list = dictDataService.selectDictDataList(dictData);
        List<SysDictData> headstock = new ArrayList<>();
        List<SysDictData> body = new ArrayList<>();
        List<SysDictData> special = new ArrayList<>();
        //根据list中的dictValue值分为三个list,dictValue值0-10，11-20，21-30
        list.forEach(item->{
            if(item.getDictValue().startsWith("0")){
                item.setListClass("车头故障");
                headstock.add(item);
            }else if(item.getDictValue().startsWith("1")){
                item.setListClass("车身故障");
                body.add(item);
            }else if(item.getDictValue().startsWith("2")){
                item.setListClass("特殊故障");
                special.add(item);
            }
        });
        faultResponse.setHeadstock(headstock);
        faultResponse.setBody(body);
        faultResponse.setSpecial(special);
        return success(faultResponse);
    }


    /**
     * 扫码/编号开锁骑行
     */
    @Log(title = "扫码/编号开锁骑行", businessType = BusinessType.UNLOCKING)
    @PostMapping("/device/snSwitch")
    public AjaxResult snSwitch(@RequestBody EtOrderVo order)
    {
        logger.info("【扫码/编号开锁骑行】请求：{}", JSON.toJSON(order));
        if(order.getRuleId()==null){
            return error("=============================================ruleId未传！！！=============================================");
        }
        if(order.getUserId()==null){
            return error("=============================================userId未传！！！=============================================");
        }
        // 判断该订单是否已结束
        EtOrder etOrder = etOrderService.selectEtOrderByOrderNo(order.getOrderNo());
        if(ObjectUtil.isNotNull(etOrder)){
            if(ServiceConstants.ORDER_STATUS_ORDER_END.equals(etOrder.getStatus())){
                return error("订单已结束，请重新扫码");
            }
        }
        AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(order.getSn());
        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(asDevice.getAreaId());
        //实名判断
        if(area.getAuthentication().equals(ServiceConstants.IS_AUTHENTICATION_YES) && !asUserService.checkIsAuthentication(order.getUserId())){
            return error("您还未实名，请先实名");
        }
        //运营时间判断
        if(!asDeviceService.isOperatingTime(order.getSn())){
            return error("不在营业时间内，不得骑行");
        }
        //非正常状态不得骑行
        String status = asDevice.getStatus();
        if(!ServiceConstants.VEHICLE_STATUS_NORMAL.equals(status)){
            EtOrder order1 = etOrderService.selectEtOrderByOrderNo(order.getOrderNo());
            if(ObjectUtil.isNotNull(order1)){
                if(!order1.getSn().equals(order.getSn())){
                    //根据状态值返回不同的提示
                    return error(CommonUtil.format(status));
                }
            }else{
                return error(CommonUtil.format(status));
            }
        }
        // 当前有骑行中的订单
        EtOrder currentOrder = etOrderService.getCurrentOrder(asDevice.getSn());
        if(ObjectUtil.isNotNull(currentOrder)){
            return error("当前车辆有骑行中的订单，请换车");
        }
        //根据余额和充值记录判断是否有充值过押金，没有充值过押金，提示充值押金
        if(!asUserService.checkIsDeposit(order.getUserId())){
            return error("您还未充值押金，请先充值押金");
        }
        logger.info("【扫码/编号开锁骑行】请求：{}", JSON.toJSON(order));
        if(etOrderService.isInOrder(order.getUserId(),order.getOrderNo()).size()>0){
            return error("您有未完成的订单，请先完成订单");
        }
        //低电量不得骑行判断
        if(asDeviceService.isLowBattery(order.getSn())){
            return error("低电量不得骑行");
        }
        OrderResponse orderResponse =asDeviceService.snSwitch(order);
        return success(orderResponse);
    }


    /**
     * 我的订单列表
     */
    @GetMapping("/order/list")
    public TableDataInfo orderList(EtOrder etOrder)
    {
        logger.info("【我的订单列表】接收到的参数：{}", JSON.toJSON(etOrder));
        startPage();
        List<EtOrder> list = etOrderService.selectEtOrderList(etOrder);
        return getDataTable(list);
    }

    /**
     * 订单详情
     */
    @GetMapping("/order/{orderNo}")
    public AjaxResult orderInfo(@PathVariable("orderNo") String orderNo)
    {
        logger.info("【订单详情】：{}", orderNo);
        return success(etOrderService.getOrderInfo(orderNo));
    }

    /**
     * 改价
     */
    @Log(title = "订单改价", businessType = BusinessType.EDITPRICE)
    @PutMapping("/order/editPrice")
    public AjaxResult orderEdit(@RequestBody EtOrder etOrder)
    {
        logger.info("【订单改价信息】：{}", JSON.toJSON(etOrder));
        //根据订单号查询订单信息
        EtOrder etOrder1 = etOrderService.selectEtOrderByOrderNo(etOrder.getOrderNo());
        if(ObjectUtil.isNull(etOrder1)){
            throw new ServiceException("订单不存在");
        }
        if(!ServiceConstants.ORDER_TYPE_RIDING.equals(etOrder1.getType())){
            throw new ServiceException("改价失败，类型必须是骑行订单");
        }
        if(!ServiceConstants.ORDER_PAY_STATUS_NON_PAYMENT.equals(etOrder1.getPaid())){
            throw new ServiceException("订单已支付，不能改价");
        }
        if("1".equals(etOrder1.getLocking())){
            throw new ServiceException("用户正在支付中，请勿操作");
        }
        if(!ServiceConstants.ORDER_STATUS_RIDING_END.equals(etOrder1.getStatus()) && !ServiceConstants.ORDER_STATUS_CANCEL_APPOINTMENT.equals(etOrder1.getStatus())){
            throw new ServiceException("改价失败，订单未结束，订单状态："+etOrder1.getStatus());
        }
        BigDecimal payFee = BigDecimal.ZERO;
        if(ObjectUtil.isNotNull(etOrder.getDispatchFee())){
            payFee = payFee.add(etOrder.getDispatchFee());
        }
        if(ObjectUtil.isNotNull(etOrder.getRidingFee())){
            payFee = payFee.add(etOrder.getRidingFee());
        }
        if(ObjectUtil.isNotNull(etOrder.getManageFee())){
            payFee = payFee.add(etOrder.getManageFee());
        }
        if(ObjectUtil.isNotNull(etOrder.getAppointmentFee())){
            payFee = payFee.add(etOrder.getAppointmentFee());
        }
        etOrder.setPayFee(payFee);
        etOrder.setTotalFee(payFee);
        return toAjax(etOrderService.updateEtOrderByOrderNo(etOrder));
    }

    /**
     * 退款
     */
    @Log(title = "订单退款", businessType = BusinessType.REFUND)
    @PutMapping("/order/refund")
    public AjaxResult orderRefund(@RequestBody EtOrder etOrder)
    {
        logger.info("【订单退款请求】：{}", JSON.toJSON(etOrder));
        return toAjax(etOrderService.refund(etOrder));
    }

    /**
     * 押金抵扣
     */
    @Log(title = "押金抵扣", businessType = BusinessType.DEDUCTION)
    @PostMapping("/order/deduction")
    public AjaxResult deduction(@RequestBody EtOrder etOrder)
    {
        logger.info("【押金抵扣请求】：{}", JSON.toJSON(etOrder));
        return toAjax(etOrderService.deduction(etOrder));
    }

    /**
     * 提现
     */
    @Transactional
    @Log(title = "提现", businessType = BusinessType.WITHDRAW)
    @PostMapping("/order/withdraw")
    public AjaxResult withdraw()
    {

        //根据订单号查询订单信息
        EtOrder etOrder = new EtOrder();
        etOrder.setUserId(getUserId());
        etOrder.setType(ServiceConstants.ORDER_TYPE_DEPOSIT);
        etOrder.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
        etOrder.setPaid(ServiceConstants.ORDER_PAY_STATUS_PAID);
        List<EtOrder> etOrders = etOrderMapper.selectEtOrderList(etOrder);
        if(etOrders.size()==0){
            throw new ServiceException("提现失败，未找到押金订单");
        }
        List<EtOrder> inOrder = etOrderService.isInOrder(etOrder.getUserId(), null);
        if(inOrder.size()>0){
            throw new ServiceException("提现失败，用户还有未完成订单");
        }
        List<EtOrder> inAuditOrder = etOrderService.isInAuditOrder(etOrder.getUserId(), null);
        if(inAuditOrder.size()>0){
            throw new ServiceException("提现失败，用户还有未审核完成的订单");
        }
        if(etOrders.size()>1){
            logger.info("有多条status="+ServiceConstants.ORDER_STATUS_ORDER_END+"（已完成）,paid="+ServiceConstants.ORDER_PAY_STATUS_PAID+"（已支付）的押金订单");
            logger.info("获取最后一条押金充值记录");
            Optional<EtOrder> latestOrder = etOrders.stream()
                    .max(Comparator.comparing(EtOrder::getPayTime));
            if (latestOrder.isPresent()) {
                EtOrder newestOrder = latestOrder.get();
                return toAjax(depositWithdraw(newestOrder));
            }
        }
        EtOrder etOrder1 = etOrders.get(0);
        Integer withdraw = depositWithdraw(etOrder1);
        return toAjax(withdraw);
    }

    /**
     * 押金提现
     */
    @NotNull
    private Integer depositWithdraw(EtOrder etOrder1) {
        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(etOrder1.getAreaId());
        BigDecimal deposit = new BigDecimal(area.getDeposit());
//        if(deposit.compareTo(etOrder1.getTotalFee())!=0){
//            throw new ServiceException("订单充值金额与系统押金不相等");
//        }
        Long userId = etOrder1.getUserId();
        if(userId!=null){
            AsUser asUser = asUserService.selectUserById(userId);
            if(asUser!=null){
                int comparisonResult = deposit.compareTo(BigDecimal.ZERO);
                if (comparisonResult < 0) {
                    throw new ServiceException("设置的押金不能小于0");
                }
                // 更新用户并更新缓存
                LoginUser loginUser = SecurityUtils.getLoginUser();
                AsUser currentUser = loginUser.getAsUser();
                currentUser.setBalance(BigDecimal.ZERO);
                if (asUserService.updateUserProfile(currentUser) > 0)
                {
                    logger.info("【提现金额】更新用户缓存:"+ JSON.toJSON(currentUser));
                    // 更新缓存用户信息
                    tokenService.setLoginUser(loginUser);
                }
            }
        }
        logger.info("提现金额：【{}】", etOrder1.getTotalFee());
        return etOrderService.withdraw(etOrder1,etOrder1.getTotalFee());
    }

    /**
     * 车辆预约
     */
    @Log(title = "车辆预约", businessType = BusinessType.APPOINTMENT)
    @PostMapping("/device/appointment")
    public AjaxResult deviceAppointment(EtOrderVo appointmentVo)
    {
        logger.info("【车辆预约信息】：{}", JSON.toJSON(appointmentVo));
        AsDevice asDevice = asDeviceService.selectAsDeviceBySn(appointmentVo.getSn());
        //设备是否在线
        if(!asDeviceService.isOnline(appointmentVo.getSn())){
            return error("设备不在线");
        }
        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(asDevice.getAreaId());
        //实名判断
        if(area.getAuthentication().equals(ServiceConstants.IS_AUTHENTICATION_YES) && !asUserService.checkIsAuthentication(appointmentVo.getUserId())){
            return error("您还未实名，请先实名");
        }
        //运营时间判断
        if(!asDeviceService.isOperatingTime(appointmentVo.getSn())){
            return error("不在营业时间内，不得骑行");
        }
        if(etOrderService.isInOrder(appointmentVo.getUserId(),appointmentVo.getOrderNo()).size()>0){
            throw new ServiceException("您有未完成的订单，请先完成订单");
        }
        //根据余额和充值记录判断是否有充值过押金，没有充值过押金，提示充值押金
        if(!asUserService.checkIsDeposit(appointmentVo.getUserId())){
            throw new ServiceException("您还未充值押金，请先充值押金");
        }
        //检验预约订单只能有一个
        if(etOrderService.checkIsUnique(appointmentVo.getUserId())){
            throw new ServiceException("您有预约中的订单，请务重复操作");
        }
        appointmentVo.setType("2");
        String status = asDevice.getStatus();
        if(!ServiceConstants.VEHICLE_STATUS_NORMAL.equals(status)){
            //根据状态值返回不同的提示
            return error(CommonUtil.format(status));
        }
        OrderResponse orderResponse = asDeviceService.deviceAppointment(appointmentVo);
        return success(orderResponse);
    }


    /**
     * 取消预约接口  用户，订单号，sn, ruleId有则是套餐，没有则是普通预约
     */
    @Log(title = "取消预约接口", businessType = BusinessType.CANCEL_APPOINTMENT)
    @PostMapping("/device/cancelAppointment")
    public AjaxResult cancelAppointment(EtOrderVo appointmentVo)
    {
        logger.info("【取消预约接口】：{}", JSON.toJSON(appointmentVo));
        EtOrder order = etOrderService.selectEtOrderByOrderNo(appointmentVo.getOrderNo());
        if(ObjectUtil.isNull(order)){
            return error("没有查询到订单信息："+appointmentVo.getOrderNo());
        }
        AsDevice asDevice = asDeviceService.selectAsDeviceBySn(order.getSn());
        String status = asDevice.getStatus();
        if(!ServiceConstants.VEHICLE_STATUS_IN_APPOINTMENT.equals(status)){
            return error("车辆状态不是预约中");
        }
        OrderResponse orderResponse = asDeviceService.cancelAppointment(appointmentVo,asDevice);
        return success(orderResponse);
    }


    /**
     * 还车
     * 1. 记录还车时间
     * 2. 计算订单各种费用，保存订单总金额
     * 3. 车辆远程关锁
     */
    @Log(title = "还车", businessType = BusinessType.RETURN_VEHICLE)
    @PostMapping("/device/return")
    public AjaxResult returnVehicle(String orderNo,String returnType,String isBluetooth,String lon,String lat,String voltage,String isInParkingArea)
    {
        logger.info("【接收还车请求参数】：orderNo={},是否辅助还车：{},是否蓝牙控制：{},经度：{},纬度：{},电压：{},是否在停车点内：{}", orderNo,returnType,isBluetooth,lon,lat,voltage,isInParkingArea);
        Boolean aBoolean = asDeviceService.returnVehicle(orderNo,returnType,isBluetooth,lon,lat,voltage,isInParkingArea);
        return success(aBoolean);
    }

    /**
     * 预下单
     * 类型 1骑行 2预约 3套餐 4押金
     */
    @PostMapping("/pre/order")
    public AjaxResult preOrder(@RequestBody EtOrderVo order)
    {
        logger.info("【预下单订单信息】：{}", JSON.toJSON(order));
        PrepayWithRequestPaymentResponse paymentResponse = etOrderService.preOrder(order);
        return success(paymentResponse);
    }

    /**
     * 临时锁车
     */
    @Log(title = "临时锁车", businessType = BusinessType.LOCK)
    @PostMapping("/device/lock")
    public AjaxResult deviceLock(String sn,String orderNo,String isBluetooth,String lon,String lat)
    {
        logger.info("【临时锁车请求参数】：{},订单号：{},是否蓝牙控制：{}", sn,orderNo,isBluetooth);
        Boolean i =asDeviceService.lock(sn,orderNo,isBluetooth,lon,lat);
        return success(i);
    }

    /**
     * 临时解锁
     */
    @Log(title = "临时解锁", businessType = BusinessType.UNLOCK)
    @PostMapping("/device/unlock")
    public AjaxResult deviceUnlock(String sn,String orderNo,String isBluetooth,String lon,String lat)
    {
        logger.info("【临时解锁请求参数】：{},订单号：{},是否蓝牙控制：{}", sn,orderNo,isBluetooth);
        Boolean i =asDeviceService.unlock(sn,orderNo,isBluetooth,lon,lat);
        return success(i);
    }

    /**
     * 更新车牌号
     */
    @Log(title = "更新车牌号", businessType = BusinessType.UPDATE)
    @PutMapping("/device/edit")
    public AjaxResult deviceEdit(String sn, String vehicleNum, String status)
    {
        AsDevice asDevice = new AsDevice();
        asDevice.setSn(sn);
        asDevice.setVehicleNum(vehicleNum);
        asDevice.setStatus(status);
        return toAjax(asDeviceService.updateAsDeviceBySn(asDevice));
    }


    /**
     * 我的工单列表
     */
    @GetMapping("/adminOrder/list")
    public TableDataInfo adminAsDevice(EtAdminOrder etAdminOrder)
    {
        logger.info("【我的工单列表请求参数】：{}", JSON.toJSON(etAdminOrder));
        startPage();
        List<EtAdminOrder> list = etAdminOrderService.selectEtAdminOrderList(etAdminOrder);
        return getDataTable(list);
    }

    /**
     * 我的工单数
     */
    @GetMapping("/adminOrder/num")
    public AjaxResult adminOrderNum(EtAdminOrder etAdminOrder)
    {
        logger.info("【我的工单数请求参数】：{}", JSON.toJSON(etAdminOrder));
        AdminOrderVo adminOrderVo =etAdminOrderService.selectEtAdminOrderNum(etAdminOrder);
        return success(adminOrderVo);
    }

    /**
     * 报修单详情
     */
    @GetMapping("/adminOrder/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        logger.info("【报修单详情请求参数】：{}", id);
        return success(etAdminOrderService.selectEtAdminOrderById(id));
    }

    /**
     * 管理员开锁
     */
    @Log(title = "管理员开锁", businessType = BusinessType.UNLOCKING)
    @PostMapping("/admin/unlocking")
    public AjaxResult unlocking(String sn)
    {
        logger.info("【管理员开锁请求】：{}",sn);
        AsUser asUser = getLoginUser().getAsUser();
        Boolean aBoolean =asDeviceService.unlocking(sn,asUser.getUserName());
        return success(aBoolean);
    }

    /**
     * 管理员关锁
     */
    @Log(title = "管理员关锁", businessType = BusinessType.LOCK)
    @PostMapping("/admin/lock")
    public AjaxResult lock(String sn)
    {
        logger.info("【管理员关锁请求】：{}",sn);
        AsUser asUser = getLoginUser().getAsUser();
        Boolean aBoolean =asDeviceService.adminLock(sn,asUser.getUserName());
        return success(aBoolean);
    }


    /**
     * 车辆上线
     */
    @Log(title = "车辆上线", businessType = BusinessType.ONLINE)
    @PostMapping("/device/online")
    public AjaxResult online(String sn)
    {
        logger.info("【车辆上线请求】：{}",sn);
        Boolean aBoolean =asDeviceService.online(sn);
        return success(aBoolean);
    }

    /**
     * 车辆下线
     */
    @Log(title = "车辆下线", businessType = BusinessType.OFFLINE)
    @PostMapping("/device/offline")
    public AjaxResult offline(String sn)
    {
        logger.info("【车辆下线请求】：{}",sn);
        Boolean aBoolean =asDeviceService.offline(sn,ServiceConstants.VEHICLE_STATUS_IN_OFFLINE);
        return success(aBoolean);
    }

    /**
     * 处理进度
     */
    @GetMapping("/admiOrder/historyList")
    public TableDataInfo list(EtAdminOrderHistory etAdminOrderHistory)
    {
        startPage();
        List<EtAdminOrderHistory> list = etAdminOrderHistoryService.selectEtAdminOrderHistoryList(etAdminOrderHistory);
        return getDataTable(list);
    }


    /**
     * 维修处理
     */
    @Log(title = "维修处理", businessType = BusinessType.REPAIR)
    @PutMapping("/admiOrder")
    public AjaxResult repairHandle(EtAdminOrder etAdminOrder)
    {
        return toAjax(etAdminOrderService.repairHandle(etAdminOrder));
    }

    /**----------------------------管理员相关----------------------------*/
    /**
     * 所有车辆定位
     */
    @GetMapping(value = "/allVehicleInfo")
    public AjaxResult allVehicleInfo(String powerStart,String powerEnd, String status,String sort,String areaId,String onlineStatus)
    {
        List<AsDevice> asDevices = asDeviceService.allVehicleInfo(powerStart,powerEnd,status,sort,areaId,onlineStatus);
        return success(asDevices);
    }

    /**
     * 批量禁用/解禁、出仓/回仓
     * disable-禁用 enable-解禁  inStash-回仓  outStash-出仓
     */
    @Transactional
    @Log(title = "批量修改操作", businessType = BusinessType.DISABLE)
    @PostMapping("/device/disable")
    public AjaxResult batchDisable(String sns, String disableType)
    {
        logger.info("【批量禁用/解禁请求参数】：{},【{}】", sns, disableType);
        if(StrUtil.isBlank(sns)){
            throw new ServiceException("车辆sn不能为空");
        }
        String[] smList = sns.split(",");
        if(ServiceConstants.BATCH_OPERATION_TYPE_DISABLE.equals(disableType)){//禁用
            for(String sn:smList){
                asDeviceService.offline(sn,ServiceConstants.VEHICLE_STATUS_IN_OFFLINE);
            }
        }else if(ServiceConstants.BATCH_OPERATION_TYPE_ENABLE.equals(disableType) || ServiceConstants.BATCH_OPERATION_TYPE_OUTSTASH.equals(disableType)){
            for(String sn:smList){
                asDeviceService.online(sn);
            }
        }else if(ServiceConstants.BATCH_OPERATION_TYPE_INSTASH.equals(disableType)) {//回仓
            for (String sn : smList) {
                asDeviceService.offline(sn,ServiceConstants.VEHICLE_STATUS_NOT_LISTING);
            }
        }else{
            throw new ServiceException("操作类型错误");
        }
        return toAjax(1);
    }

    /**
     * 查询车辆数量
     */
    @GetMapping(value = "/allVehicleNum")
    public AjaxResult allVehicleNum(String powerStart,String powerEnd,String areaId)
    {
        logger.info("【查询车辆数量】请求参数：powerStart:【{}】,powerEnd:【{}】,areaId:【{}】", powerStart,powerEnd,areaId);
        DeviceNumVo deviceNumVo = asDeviceService.allVehicleNum(powerStart,powerEnd,areaId);
        return success(deviceNumVo);
    }

    /**
     * 所有停车点
     */
    @GetMapping(value = "/allParkingArea")
    public AjaxResult allParkingArea()
    {
        List<EtParkingArea> etParkingAreas = etParkingAreaService.allParkingArea();
        return success(etParkingAreas);
    }

    /**
     * 运营数据
     */
    @GetMapping(value = "/operatingData")
    public AjaxResult operatingData(String timeStart,String timeEnd,String areaId)
    {
        logger.info("【运营数据】请求参数：timeStart={},timeEnd={},areaId={}", timeStart,timeEnd,areaId);
        OperatingDataVo operatingDataVo = etOrderService.getOperatingData(timeStart,timeEnd,areaId);
        return success(operatingDataVo);
    }

    /**
     * 运营数据2
     */
    @GetMapping(value = "/operatingData2")
    public AjaxResult operatingData2(String timeStart,String timeEnd,String areaId)
    {
        logger.info("【运营数据】请求参数：timeStart={},timeEnd={},areaId={}", timeStart,timeEnd,areaId);
        OperatingDataVo2 operatingDataVo = etOrderService.getOperatingData2(timeStart,timeEnd,areaId);
        return success(operatingDataVo);
    }


    /**
     * 收入对账
     * type:1-按日期 2-按车辆
     */
    @GetMapping(value = "/reconciliation")
    public AjaxResult reconciliation(String timeStart,String timeEnd, String type,String sn,String areaId)
    {
        logger.info("【收入对账】请求参数：timeStart={},timeEnd={},type={},sn={},areaId={}", timeStart,timeEnd,type,sn,areaId);
        ReconciliationVo operatingDataVo = etOrderService.reconciliation(timeStart,timeEnd,type,sn,areaId);
        return success(operatingDataVo);
    }

    /**
     * 最近一笔订单
     */
    @GetMapping(value = "/latestOrder")
    public AjaxResult latestOrder(EtOrder etOrder)
    {
        logger.info("【最近一笔订单】请求参数：etOrder={}", JSON.toJSON(etOrder));
        EtOrder order =etOrderService.latestOrder(etOrder);
        return success(order);
    }


    /**
     * 获取用户列表
     */
    @GetMapping("/user/list")
    public TableDataInfo list(AsUser asUser)
    {
        startPage();
        List<AsUser> list = asUserService.selectUserList(asUser);
        return getDataTable(list);
    }

    /**
     * 分账账单
     *
     * 合伙人账单
     * 合伙人1  分账金额
     */
    @GetMapping("/partnerBill")
    public AjaxResult partnerBill()
    {
//        etOrderService.partnerBill();
        return success();
    }

    /**
     * 实名认证
     */
    @Log(title = "实名认证", businessType = BusinessType.AUTHENTICATION)
    @GetMapping("/user/authentication")
    public AjaxResult partnerBillList(AuthenticationVo authenticationVo)
    {
        logger.info("【实名认证】请求参数：authenticationVo={}", JSON.toJSON(authenticationVo));
        Object authentication = asUserService.authentication(authenticationVo);
        //判断o是否是字符串
        if(authentication instanceof String){
            return AjaxResult.error((String) authentication);
        }else{
            IDResponse.Result result = (IDResponse.Result)authentication;
            if(result.getIsok()){
               //保存身份信息
                // 更新用户并更新缓存
                LoginUser loginUser = SecurityUtils.getLoginUser();
                AsUser asUser = loginUser.getAsUser();
                asUser.setUserId(authenticationVo.getUserId());
                asUser.setRealName(authenticationVo.getRealName());
                asUser.setIdCard(authenticationVo.getIdCard());
                asUser.setIsAuthentication(ServiceConstants.IS_AUTHENTICATION_YES);
                int updateUser = asUserService.updateUser(asUser);
                if (updateUser>0)
                {
                    logger.info("【实名认证】保存身份信息成功");
                    // 更新缓存用户信息
                    tokenService.setLoginUser(loginUser);
                }
            }
            return AjaxResult.success(authentication);
        }
    }

    /**
     * 是否缴纳过押金
     */
    @GetMapping(value = "/checkIsDeposit")
    public AjaxResult checkIsDeposit(Long userId)
    {
        Boolean aBoolean = asUserService.checkIsDeposit(userId);
        return AjaxResult.success("操作成功",aBoolean);
    }

    /**
     * sn和mac号绑定
     */
    @Log(title = "设备绑定", businessType = BusinessType.BAND)
    @PostMapping("/band")
    public AjaxResult bandSn(String sn,String mac,Long hardwareVersionId)
    {
        logger.info("sn和mac号绑定：【sn="+sn+"】，【mac="+mac+"】,【hardwareVersionId="+hardwareVersionId+"】");
        AsDevice asDevice = new AsDevice();
        asDevice.setSn(sn);
        asDevice.setMac(mac);
        asDevice.setHardwareVersionId(hardwareVersionId);
        asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_NOT_BAND);
        asDevice.setCreateTime(DateUtils.getNowDate());
        return toAjax(asDeviceService.bandSn(asDevice));
    }

    /**
     * 根据mac修改sn
     */
    @Log(title = "根据mac修改sn", businessType = BusinessType.UPDATESN)
    @PostMapping("/updateSn")
    public AjaxResult updateSn(String sn,String mac,Long hardwareVersionId)
    {
        logger.info("根据mac修改sn：【sn="+sn+"】，【mac="+mac+"】,【hardwareVersionId="+hardwareVersionId+"】");
        AsDevice asDevice = new AsDevice();
        asDevice.setSn(sn);
        asDevice.setMac(mac);
        asDevice.setHardwareVersionId(hardwareVersionId);
        asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_NOT_BAND);
        asDevice.setCreateTime(DateUtils.getNowDate());
        return toAjax(asDeviceService.updateSn(asDevice));
    }

    /**
     * 根据userId获取areaId(管理员属于哪个运营区)
     */
    @PostMapping("/getAreaId")
    public AjaxResult getAreaId()
    {
        Long userId = SecurityUtils.getLoginUser().getAsUser().getUserId();
        logger.info("根据userId获取areaId(管理员属于哪个运营区)：【userId="+userId+"】");
        AsUser asUser = asUserMapper.selectUserById(userId);
        logger.info("根据userId获取areaId(管理员属于哪个运营区)：【"+JSON.toJSON(asUser)+"】");
        if(ObjectUtil.isNull(asUser)){
            throw new ServiceException("用户【"+userId+"】不存在");
        }
        SysUser sysUser = sysUserService.selectUserById(asUser.getSysUserId());
        if(ObjectUtil.isNull(sysUser)){
            throw new ServiceException("用户【"+userId+"】未绑定系统用户");
        }
        return success(sysUser.getAreaId());
    }

    /**
     * 根据条件模糊查询车辆信息
     * type:1-sn,2-mac,3-车牌号
     */
    @GetMapping("/device/search")
    public AjaxResult deviceSearch(String key,String type,String areaId)
    {
        logger.info("根据条件模糊查询车辆信息：【key="+key+"】,【type="+type+"】,【areaId="+areaId+"】");
        List<AsDevice> asDevices = asDeviceService.deviceSearch(key,type,areaId);
        return success(asDevices);
    }

    /**
     * 根据经纬度判断是否在停车区
     */
    @Log(title = "判断是否在停车区", businessType = BusinessType.ISINPARKING)
    @GetMapping("/isInParkingArea")
    public AjaxResult isInParkingArea(String longitude,String latitude,String areaId,String sn)
    {
        if(StrUtil.isBlank(longitude) || StrUtil.isBlank(latitude)){
            logger.info("没有经纬度参数：【longitude={}】，【latitude={}】，【areaId={}】,【sn={}】",longitude,latitude,areaId,sn);
            return error("请传经纬度参数"+"【longitude="+longitude+"】，【latitude="+latitude+"】");
        }
        IsInParkingAreaVo isInParkingAreaVo = asDeviceService.isInParkingArea(longitude,latitude,areaId,sn);
        return success(isInParkingAreaVo);
    }

    /**
     * 根据token获取运营区列表
     */
    @GetMapping("/getAreaList")
    public AjaxResult getAreaList()
    {
        AsUser asUser = getLoginUser().getAsUser();
        AsUser asUser1 = asUserMapper.selectUserById(asUser.getUserId());
        logger.info("获取到当前app登录用户：【{}】", JSON.toJSON(asUser1));
        if(ObjectUtil.isNull(asUser1.getSysUserId())){
            throw new RuntimeException("用户【"+asUser1.getUserName()+"】未绑定系统用户");
        }
        SysUser sysUser = userService.selectUserById(asUser1.getSysUserId());
        List<EtOperatingArea> longs;
        if(sysUser.isAdmin()){
            longs = etOperatingAreaService.selectAreaListByDeptId2(null,null);
        }else{
            if(ObjectUtil.isNotNull(sysUser.getAreaId())){
                longs = etOperatingAreaService.selectAreaListByDeptId2(null,sysUser.getAreaId());
            }else{
                if(ObjectUtil.isNotNull(sysUser.getDeptId())){
                    longs = etOperatingAreaService.selectAreaListByDeptId2(sysUser.getDeptId(),null);
                }else{
                    throw new ServiceException("该用户都没有绑定运营区和运营商（至少绑定一个）");
                }
            }
        }
        logger.info("根据token获取运营区列表：【{}】", JSON.toJSON(longs));
        return success(longs);
    }

    /**
     * 获取运营商信息
     */
    @GetMapping("/getDept")
    public AjaxResult getDept(Long areaId)
    {
        logger.info("【获取运营商信息】获取到areaId：【{}】", areaId);
        SysDept sysDept = wxPayService.getDeptObjByAreaId(areaId);

        if(ObjectUtil.isNotNull(sysDept)){
            BigDecimal todayOrderAmount = getTodayOrderAmount(sysDept);
            // 更新 sysDept 的余额字段
            if (sysDept.getBalance() != null) {
                sysDept.setSettlementAmount(todayOrderAmount);
                sysDept.setWithdrawableAmount(sysDept.getBalance().subtract(todayOrderAmount));
            }
        }
        return success(sysDept);
    }

    /** 获取今日订单金额 */
    @NotNull
    private BigDecimal getTodayOrderAmount(SysDept sysDept) {
        // 获取今天的日期字符串
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        String startDateStr = dateFormat.format(new Date()) + " 00:00:00";
        String endDateStr = dateFormat.format(new Date()) + " 23:59:59";

        //今日订单金额
        BigDecimal todayOrderAmount = BigDecimal.ZERO;
        List<Long> longs = etOperatingAreaMapper.selectAreaListByDeptId(sysDept.getDeptId());
        for (Long id:longs) {
            BigDecimal payFee = defaultIfNull(etOrderMapper.getPayFee(startDateStr, endDateStr, null, id), BigDecimal.ZERO);//新增
            BigDecimal refundFee = defaultIfNull(etOrderMapper.getRefundFee(startDateStr, endDateStr, null, id), BigDecimal.ZERO);//退款
            BigDecimal serviceFee = etCapitalFlowMapper.getHandlingFee(startDateStr, endDateStr, null, id);//手续费,扣除掉退款部分的
            BigDecimal platformServiceFee = etCapitalFlowMapper.getServiceFee(startDateStr, endDateStr, null,id);//平台服务费 ,扣除掉退款部分的
            BigDecimal areaOrderAmount = defaultIfNull(payFee.subtract(serviceFee).subtract(refundFee).subtract(platformServiceFee), BigDecimal.ZERO);//营收 = 新增 - 手续费 - 退款 - 平台服务费
            if (areaOrderAmount != null) {
                todayOrderAmount = todayOrderAmount.add(areaOrderAmount);
            }
        }
        if(todayOrderAmount.compareTo(sysDept.getBalance()) > 0){
            todayOrderAmount = sysDept.getBalance();
        }
        return todayOrderAmount;
    }

    private BigDecimal defaultIfNull(BigDecimal value, BigDecimal defaultValue) {
        return value != null ? value : defaultValue;
    }

    /**
     * 绑定APP用户
     */
    @Log(title = "绑定APP用户", businessType = BusinessType.UPDATE)
    @PutMapping("/bandAppUser")
    public AjaxResult bandAppUser(@RequestBody SysUser user)
    {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        user.setUserId(loginUser.getAsUser().getSysUserId());
        return toAjax(userService.bandAppUser(user));
    }

    /**
     * 重启设备
     */
    @Log(title = "重启设备", businessType = BusinessType.REBOOT)
    @PostMapping("/device/reboot")
    public AjaxResult reboot(String sn)
    {
        if(StrUtil.isBlank(sn)){
            logger.info("没有sn号参数：【sn={}】",sn);
            return error("请传sn号参数"+"【sn="+sn+"】");
        }
        Boolean i =asDeviceService.reboot(sn);
        return success(i);
    }

    /**
     * 保存视频地址
     */
    @Log(title = "保存视频", businessType = BusinessType.VIDEO)
    @PutMapping("/order/saveVideoUrl")
    public AjaxResult saveVideoUrl(@RequestBody EtOrder etOrder)
    {
        logger.info("【订单保存视频请求参数】：{}", JSON.toJSON(etOrder));
        //根据订单号查询订单信息
        EtOrder etOrder1 = etOrderService.selectEtOrderByOrderNo(etOrder.getOrderNo());
        if(ObjectUtil.isNull(etOrder1)){
            throw new ServiceException("订单不存在");
        }
        if(!ServiceConstants.ORDER_TYPE_RIDING.equals(etOrder1.getType())){
            throw new ServiceException("保存视频失败，类型必须是骑行订单");
        }
        etOrder.setUploadTime(DateUtils.getNowDate());
        return toAjax(etOrderService.updateEtOrderByOrderNo(etOrder));
    }

    /**
     * 取消锁单
     */
    @Log(title = "取消锁单", businessType = BusinessType.UNLOCKORDER)
    @PostMapping("/order/unlockOrder")
    public AjaxResult unlockOrder(String orderNo)
    {
//        logger.info("【取消锁单】：{}", orderNo);
//        EtOrder etOrder = new EtOrder();
//        etOrder.setLocking("0");
//        etOrder.setOrderNo(orderNo);
//        return toAjax(etOrderService.updateEtOrderByOrderNo(etOrder));
        return success();
    }

    /**
     * 还车审核通过
     *  改状态，退押金
     */
    @Transactional
    @Log(title = "还车审核通过", businessType = BusinessType.PASSAUDIT)
    @PostMapping("/passAudit/{orderNo}")
    public AjaxResult passAudit(@PathVariable("orderNo") String orderNo)
    {
        logger.info("还车审核通过请求：【{}】", orderNo);
        EtOrder order = etOrderService.selectEtOrderByOrderNo(orderNo);
        if(!order.getStatus().equals(ServiceConstants.ORDER_STATUS_TO_BE_AUDIT)){
            throw new ServiceException("该订单用户未支付！");
        }
        if(ObjectUtil.isNotNull(order)){
            order.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
            return toAjax(etOrderService.passAudit(order));
        }else{
            throw new ServiceException("订单不存在");
        }
    }

    /**
     * 车辆有损坏
     */
    @Log(title = "车辆有损坏", businessType = BusinessType.DAMAGED)
    @PutMapping("/order/damaged")
    public AjaxResult edit(@RequestBody EtOrder etOrder)
    {
        return toAjax(etOrderService.updateEtOrderByOrderNo(etOrder));
    }

    /**
     * 官方审核通过
     */
    @Log(title = "官方审核通过", businessType = BusinessType.AUTHORITYPASS)
    @PutMapping("/authoritypass/{orderNo}")
    public AjaxResult authoritypass(@PathVariable("orderNo") String orderNo)
    {
        logger.info("官方审核通过请求：【{}】", orderNo);
        EtOrder order = etOrderService.selectEtOrderByOrderNo(orderNo);
        if(ObjectUtil.isNotNull(order)){
            order.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
            return toAjax(etOrderService.authoritypass(order));
        }else{
            throw new ServiceException("订单不存在");
        }
    }

    /**
     * 查询押金订单
     */
    @GetMapping("/getDepositOrder")
    public AjaxResult getDepositOrder(Long userId)
    {
        logger.info("查询押金订单请求：【{}】", userId);
        EtOrder depositOrder = etOrderService.getDepositOrder(userId);
        return AjaxResult.success(depositOrder);
    }

    /**
     * 提交审核
     */
    @Log(title = "提交审核", businessType = BusinessType.SUBMITAUDIT)
    @PutMapping("/order/submitAudit")
    public AjaxResult edit2(@RequestBody EtOrder etOrder)
    {
        return toAjax(etOrderService.updateEtOrderByOrderNo(etOrder));
    }

    /**
     * 查询资金流水列表
     */
    @GetMapping("/flowList")
    public TableDataInfo list(EtCapitalFlow etCapitalFlow)
    {
        if(ObjectUtil.isNotNull(etCapitalFlow.getAreaId())){
            SysDept deptObjByAreaId = wxPayService.getDeptObjByAreaId(etCapitalFlow.getAreaId());
            if(ObjectUtil.isNotNull(deptObjByAreaId)){
                etCapitalFlow.setOwnerId(deptObjByAreaId.getDeptId());
                etCapitalFlow.setOwnerType("1");
                etCapitalFlow.setAreaId(null);
            }
        }
        startPage();
        List<EtCapitalFlow> list = etCapitalFlowService.selectEtCapitalFlowList(etCapitalFlow);
        return getDataTable(list);
    }

    /**
     * 管理员提现
     */
    @Log(title = "管理员提现", businessType = BusinessType.ADMINWITHDRAW)
    @PostMapping("/admin/withdraw")
    public AjaxResult add(@RequestBody EtWithdraw etWithdraw)
    {
        logger.info("管理员提现请求：【{}】", JSON.toJSON(etWithdraw));
        SysDept sysDept = wxPayService.getDeptObjByAreaId(etWithdraw.getAreaId());
        int i = deptService.adminWithdraw(etWithdraw.getAmount(),sysDept,"1");
        return toAjax(i);
    }

    /**
     * 换车关锁
     */
    @Log(title = "换车关锁", businessType = BusinessType.CHANGELOCK)
    @PostMapping("/changeVehicle/lock")
    public AjaxResult changeVehicleLock(String orderNo,String changeReason,String isBluetooth,String lon,String lat,String voltage,String newSn)
    {
        if (StrUtil.isBlank(orderNo)){
            logger.info("没有orderNo参数：【orderNo={}】",orderNo);
            return error("请传orderNo号参数"+"【orderNo="+orderNo+"】");
        }
        logger.info("【换车关锁请求】：orderNo={}",orderNo);
        Boolean aBoolean = etOrderService.changeVehicleLock(orderNo,changeReason,isBluetooth,lon,lat,voltage,newSn);
        return success(aBoolean);
    }

    /**
     * 换车开锁
     */
    @Log(title = "换车开锁", businessType = BusinessType.CHANGEUNLOCK)
    @PostMapping("/changeVehicle/unlocking")
    public AjaxResult changeVehicleLockUnlocking(String orderNo,String newSn,String isBluetooth,String lon,String lat,String voltage)
    {
        if (StrUtil.isBlank(newSn)){
            logger.info("没有sn号参数：【newSn={}】",newSn);
            return error("请传sn号参数"+"【newSn="+newSn+"】");
        }
        if (StrUtil.isBlank(orderNo)){
            logger.info("没有orderNo参数：【orderNo={}】",orderNo);
            return error("请传orderNo号参数"+"【orderNo="+orderNo+"】");
        }
        //运营时间判断
        if(!asDeviceService.isOperatingTime(newSn)){
            return error("不在营业时间内，不得骑行");
        }
        //非正常状态不得骑行
        AsDevice newDevice = asDeviceMapper.selectAsDeviceBySn(newSn);
        String status = newDevice.getStatus();
        if(!ServiceConstants.VEHICLE_STATUS_NORMAL.equals(status)){
            return error(CommonUtil.format(status));
        }
        // 当前有骑行中的订单
        EtOrder currentOrder = etOrderService.getCurrentOrder(newDevice.getSn());
        if(ObjectUtil.isNotNull(currentOrder) && !currentOrder.getOrderNo().equals(orderNo)){
            return error("当前车辆有骑行中的订单，请换车");
        }
        //低电量不得骑行判断
        if(asDeviceService.isLowBattery(newDevice.getSn())){
            return error("低电量不得骑行");
        }
        logger.info("【换车开锁请求】：orderNo={},newSn ={}",orderNo,newSn);
        Boolean aBoolean =etOrderService.changeVehicleLockUnlocking(orderNo,newSn,isBluetooth,lon,lat,voltage);
        return success(aBoolean);
    }

    /**
     * 开坐垫锁
     */
    @PostMapping("/device/seatCushionLock")
    public AjaxResult seatCushionLock(String sn)
    {
        if(StrUtil.isBlank(sn)){
            logger.info("没有sn号参数：【sn={}】",sn);
            return error("请传sn号参数"+"【sn="+sn+"】");
        }
        Boolean i =asDeviceService.seatCushionLock(sn);
        return success(i);
    }


    /**
     * 坐垫锁用mac
     */
    @Log(title = "坐垫锁", businessType = BusinessType.UPDATE)
    @PostMapping("/device/seatCushionLockByMac")
    public AjaxResult seatCushionLockByMac(String mac)
    {
        if(StrUtil.isBlank(mac)){
            logger.info("没有mac号参数：【mac={}】",mac);
            return error("请传mac号参数"+"【mac="+mac+"】");
        }
        Boolean i =asDeviceService.seatCushionLockByMac(mac);
        return success(i);
    }


    /**
     * app重置密码
     */
    @Log(title = "app重置密码", businessType = BusinessType.UPDATE)
    @PutMapping("/resetAppPwd")
    public AjaxResult resetAppPwd(String oldPassword, String newPassword)
    {
        LoginUser loginUser = getLoginUser();
        AsUser asUser = loginUser.getAsUser();
        Long userId = asUser.getUserId();
        String password = asUser.getPassword();
        if (!SecurityUtils.matchesPassword(oldPassword, password))
        {
            return error("修改密码失败，旧密码错误");
        }
        if (SecurityUtils.matchesPassword(newPassword, password))
        {
            return error("新密码不能与旧密码相同");
        }
        newPassword = SecurityUtils.encryptPassword(newPassword);
        if (asUserMapper.updateUserPwd(userId, newPassword) > 0)
        {
            // 更新缓存用户密码
            loginUser.getAsUser().setPassword(newPassword);
            tokenService.setLoginUser(loginUser);
            return success();
        }
        return error("修改密码异常，请联系管理员");
    }

    /**
     * 根据token判断是运营商还是合伙人
     * 判断类型
     * 运营商才有资金
     * 合伙人才有资金
     */
    @GetMapping("/ownerType")
    public AjaxResult ownerType()
    {
        AsUser asUser = getLoginUser().getAsUser();
        AsUser asUser1 = asUserMapper.selectUserById(asUser.getUserId());
        logger.info("根据token判断是运营商还是合伙人：【{}】", JSON.toJSON(asUser1));
        //
        if(ObjectUtil.isNotNull(asUser1.getSysUserId())){
            SysUser sysUser = userService.selectUserById(asUser1.getSysUserId());
            if(sysUser.isAdmin()){
                return success(OwnerVo.builder().ownerId(sysUser.getUserId()).ownerType("1").build());
            }else{
                if(ObjectUtil.isNotNull(sysUser)){
                    if("03".equals(sysUser.getUserType())){//合伙人
                        return success(OwnerVo.builder().ownerId(sysUser.getUserId()).ownerType("2").build());
                    }else if(asUser1.getRole().equals("9")){
                        return success(OwnerVo.builder().ownerId(sysUser.getDeptId()).ownerType("1").build());
                    }else {
                        return error("用户【"+asUser.getUserName()+"】无账户权限");
                    }
                }
            }
        }
        return error("用户"+asUser.getUserName()+"未绑定系统用户");
    }

    /**
     * 根据订单号查询订单支付结果
     */
    @GetMapping("/queryResultByOrderNo")
    public AjaxResult queryResultByOrderNo(String orderNo)
    {
        logger.info("根据订单号查询订单支付结果：【orderNo="+orderNo+"】");
        Boolean aBoolean = wxPayService.queryResultByOrderNo(orderNo);
        return AjaxResult.success(aBoolean);
    }

    /**
     * 优惠券购买（预支付）
     */
    @GetMapping("/coupon/buy")
    public AjaxResult couponBuy(Long userId, Long couponId)
    {
        logger.info("优惠券购买：【userId="+userId+"】,【couponId="+couponId+"】");
        PrepayWithRequestPaymentResponse paymentResponse = etCouponService.couponBuy(userId, couponId);
        return success(paymentResponse);
    }

    /**
     * 我的骑行卡(未使用和已过期)
     */
    @GetMapping("/getCouponListByUserId")
    public AjaxResult getCouponListByUserId(Long userId)
    {
        logger.info("根据用户搜索优惠券：【userId="+userId+"】");
        List<EtCouponUserLog> list = etCouponClaimLogService.selectCouponListByUserId(userId);
//        // 判空处理，确保list不为null
//        if (list != null && !list.isEmpty()) {
//            // 将 couponType 为 1 和 2 的记录按类型分组，并各保留一条记录
//            Map<String, List<EtCouponUserLog>> type12Map = list.stream()
//                    .filter(log -> "1".equals(log.getCouponType()) || "2".equals(log.getCouponType()))
//                    .collect(Collectors.groupingBy(EtCouponUserLog::getCouponType));
//
//            // 创建结果列表，并添加每个类型的第一条记录
//            List<EtCouponUserLog> resultList = new ArrayList<>();
//            type12Map.values().forEach(logs -> {
//                if (logs != null && !logs.isEmpty()) {
//                    resultList.add(logs.get(0));
//                }
//            });
//
//            // 添加其他类型的记录
//            list.stream()
//                    .filter(log -> !"1".equals(log.getCouponType()) && !"2".equals(log.getCouponType()))
//                    .forEach(resultList::add);
//
//            // 将结果列表赋值回原列表
//            list = resultList;
//        }
        return success(list);
    }

    /**
     * 优惠券使用记录（已使用）
     */
    @GetMapping("/getCouponUsageRecord")
    public AjaxResult getCouponUsageRecord(Long userId)
    {
        logger.info("【优惠券使用记录（已使用）】：【userId="+userId+"】");
        EtCouponUserLog etCouponUserLog = new EtCouponUserLog();
        etCouponUserLog.setUserId(userId);
        etCouponUserLog.setStatus(ServiceConstants.COUPON_STATUS_USED);
        List<EtCouponUserLog> list = etCouponClaimLogService.selectEtCouponClaimLogList(etCouponUserLog);
        return success(list);
    }

    /**
     * 优惠券购买记录
     */
    @GetMapping("/couponBuyRecord")
    public AjaxResult couponBuyRecord(Long userId, String[] typeList)
    {
        logger.info("【优惠券购买记录】：【userId="+userId+"】");
        EtCouponUserLog etCouponUserLog = new EtCouponUserLog();
        etCouponUserLog.setUserId(userId);
        etCouponUserLog.setGainMethod(ServiceConstants.COUPON_GAIN_METHOD_BUY);
        etCouponUserLog.setTypeList(typeList);
        List<EtCouponUserLog> list = etCouponClaimLogService.selectEtCouponClaimLogList(etCouponUserLog);
        return success(list);
    }

    /**
     * 撤销审核
     */
    @Log(title = "撤销审核", businessType = BusinessType.UPDATE)
    @PostMapping("/cancelAudit")
    public AjaxResult cancelAudit(@RequestBody EtOrder etOrder)
    {
        logger.info("撤销审核：【etOrder="+etOrder+"】");
        EtOrder order = etOrderService.selectEtOrderByOrderNo(etOrder.getOrderNo());
        if (Integer.parseInt(order.getStatus())<5){
            throw new ServiceException("订单状态不对，不能撤销审核");
        }
        return toAjax(etOrderService.updateEtOrderByOrderNo(etOrder));
    }

    /**
     * 更新设备最新信息
     */
    @Log(title = "更新设备", businessType = BusinessType.REFRESH)
    @PostMapping("/refreshDevice")
    public AjaxResult refreshDevice(String sn)
    {
        if (StrUtil.isBlank(sn)){
            throw new ServiceException("未传sn");
        }
        return toAjax(asDeviceService.refreshDeviceBySn(sn));
    }
}
