package com.ruoyi.web.controller.system;

import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.AsDevice;
import com.ruoyi.system.service.IAsDeviceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 设备Controller
 *
 * @author 邱贞招
 * @date 2024-04-05
 */
@RestController
@RequestMapping("/system/device")
public class AsDeviceController extends BaseController
{
    @Autowired
    private IAsDeviceService asDeviceService;

    /**
     * 查询设备列表
     */
    @PreAuthorize("@ss.hasPermi('system:device:list')")
    @GetMapping("/list")
    public TableDataInfo list(AsDevice asDevice)
    {
        startPage();
        List<AsDevice> list = asDeviceService.selectAsDeviceListWithIsolate(asDevice);
        return getDataTable(list);
    }

    /**
     * 导出设备列表
     */
    @PreAuthorize("@ss.hasPermi('system:device:export')")
    @Log(title = "设备", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, AsDevice asDevice)
    {
        List<AsDevice> list = asDeviceService.selectAsDeviceList(asDevice);
        ExcelUtil<AsDevice> util = new ExcelUtil<AsDevice>(AsDevice.class);
        util.exportExcel(response, list, "设备数据");
    }

    /**
     * 获取设备详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:device:query')")
    @GetMapping(value = "/{deviceId}")
    public AjaxResult getInfo(@PathVariable("deviceId") Long deviceId)
    {
        return success(asDeviceService.selectAsDeviceByDeviceId(deviceId));
    }

    /**
     * 新增设备
     */
    @PreAuthorize("@ss.hasPermi('system:device:add')")
    @Log(title = "设备", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody AsDevice asDevice)
    {
        if(!asDeviceService.checkSNUnique(asDevice)){
            return error("新增车辆'" + asDevice.getDeviceName() + "'失败，SN已存在");
        }
        if(!asDeviceService.checkMACUnique(asDevice)){
            return error("新增车辆'" + asDevice.getDeviceName() + "'失败，MAC已存在");
        }
        return toAjax(asDeviceService.insertAsDevice(asDevice));
    }

    /**
     * 修改设备
     */
    @PreAuthorize("@ss.hasPermi('system:device:edit')")
    @Log(title = "设备", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody AsDevice asDevice)
    {
        if(!asDeviceService.checkSNUnique(asDevice)){
            return error("修改车辆'" + asDevice.getDeviceName() + "'失败，SN已存在");
        }
        if(!asDeviceService.checkMACUnique(asDevice)){
            return error("修改车辆'" + asDevice.getDeviceName() + "'失败，MAC已存在");
        }
        return toAjax(asDeviceService.updateAsDevice2(asDevice));
    }

    /**
     * 删除设备
     */
    @PreAuthorize("@ss.hasPermi('system:device:remove')")
    @Log(title = "设备", businessType = BusinessType.DELETE)
	@DeleteMapping("/{deviceIds}")
    public AjaxResult remove(@PathVariable Long[] deviceIds)
    {
        return toAjax(asDeviceService.deleteAsDeviceByDeviceIds(deviceIds));
    }

    /**
     * 一键上线
     */
    @PreAuthorize("@ss.hasPermi('system:device:oneClickOnline')")
    @Log(title = "设备", businessType = BusinessType.ONLINE)
    @PostMapping("/oneClickOnline/{deviceIds}")
    public AjaxResult oneClickOnline(@PathVariable Long[] deviceIds)
    {
        return toAjax(asDeviceService.oneClickOnline(deviceIds));
    }

    /**
     * 一键禁用
     */
    @PreAuthorize("@ss.hasPermi('system:device:oneClickOffline')")
    @Log(title = "设备", businessType = BusinessType.OFFLINE)
    @PostMapping("/oneClickOffline/{deviceIds}")
    public AjaxResult oneClickOffline(@PathVariable Long[] deviceIds)
    {
        return toAjax(asDeviceService.oneClickOffline(deviceIds));
    }


    /**
     * 一键出仓
     */
    @PreAuthorize("@ss.hasPermi('system:device:oneClickOnline')")
    @Log(title = "设备", businessType = BusinessType.ONLINE)
    @PostMapping("/oneClickListing/{deviceIds}")
    public AjaxResult oneClickListing(@PathVariable Long[] deviceIds)
    {
        return toAjax(asDeviceService.oneClickListing(deviceIds));
    }


    /**
     * 一键入仓
     */
    @PreAuthorize("@ss.hasPermi('system:device:oneClickOnline')")
    @Log(title = "设备", businessType = BusinessType.ONLINE)
    @PostMapping("/oneClickWarehousing/{deviceIds}")
    public AjaxResult oneClickWarehousing(@PathVariable Long[] deviceIds)
    {
        return toAjax(asDeviceService.oneClickWarehousing(deviceIds));
    }

    /**
     * 更新设备
     */
    @PreAuthorize("@ss.hasPermi('system:device:refresh')")
    @Log(title = "更新设备", businessType = BusinessType.REFRESH)
    @PostMapping("/refreshDevice/{deviceIds}")
    public AjaxResult refreshDevice(@PathVariable Long[] deviceIds)
    {
        return toAjax(asDeviceService.refreshDevice(deviceIds));
    }

    /**
     * 响铃寻车
     */
    @PreAuthorize("@ss.hasPermi('system:device:ring')")
    @PostMapping("/device/ring")
    public AjaxResult ring(String sn)
    {
        if(StrUtil.isBlank(sn)){
            logger.info("没有sn号参数：【sn={}】",sn);
            return error("请传sn号参数"+"【sn="+sn+"】");
        }
        Boolean i =asDeviceService.ring(sn);
        return success(i);
    }

    /**
     * 重启 reboot
     */
    @PreAuthorize("@ss.hasPermi('system:device:reboot')")
    @PostMapping("/device/reboot")
    public AjaxResult reboot(String mac)
    {
        if(StrUtil.isBlank(mac)){
            logger.info("没有sn号参数：【mac={}】",mac);
            return error("请传sn号参数"+"【mac="+mac+"】");
        }
        Boolean i =asDeviceService.rebootByMac(mac);
        return success(i);
    }

    /**
     * 管理员开锁
     */
    @PreAuthorize("@ss.hasPermi('system:device:unlocking')")
    @Log(title = "管理员开锁", businessType = BusinessType.UNLOCKING)
    @PostMapping("/admin/unlocking")
    public AjaxResult unlocking(String sn)
    {
        logger.info("【管理员开锁请求】：{}",sn);
        Boolean aBoolean =asDeviceService.unlocking(sn,getUsername());
        return success(aBoolean);
    }


    /**
     * 管理员开锁
     */
    @PreAuthorize("@ss.hasPermi('system:device:unlocking')")
    @Log(title = "管理员开锁", businessType = BusinessType.UNLOCKING)
    @PostMapping("/admin/unlockingByMac")
    public AjaxResult unlockingByMac(String mac)
    {
        logger.info("【管理员开锁请求】：{}",mac);
        Boolean aBoolean =asDeviceService.unlockingByMac(mac,getUsername());
        return success(aBoolean);
    }

    /**
     * 管理员关锁
     */
    @PreAuthorize("@ss.hasPermi('system:device:unlocking')")
    @Log(title = "管理员关锁", businessType = BusinessType.LOCK)
    @PostMapping("/admin/lock")
    public AjaxResult lock(String sn)
    {
        logger.info("【管理员关锁请求】：{}",sn);
        Boolean aBoolean =asDeviceService.adminLock(sn,getUsername());
        return success(aBoolean);
    }

    /**
     * 管理员关锁
     */
    @PreAuthorize("@ss.hasPermi('system:device:unlocking')")
    @Log(title = "管理员关锁", businessType = BusinessType.LOCK)
    @PostMapping("/admin/lockByMac")
    public AjaxResult lockByMac(String mac)
    {
        logger.info("【管理员关锁请求】：{}",mac);
        Boolean aBoolean =asDeviceService.adminLockByMac(mac,getUsername());
        return success(aBoolean);
    }


    /**
     * 车辆上线
     */
    @PreAuthorize("@ss.hasPermi('system:device:online')")
    @Log(title = "车辆上线", businessType = BusinessType.ONLINE)
    @PostMapping("/device/online")
    public AjaxResult online(String sn)
    {
        logger.info("【车辆上线请求】：{}",sn);
        Boolean aBoolean =asDeviceService.online(sn);
        return success(aBoolean);
    }

    /**
     * 车辆下线
     */
    @PreAuthorize("@ss.hasPermi('system:device:online')")
    @Log(title = "车辆下线", businessType = BusinessType.OFFLINE)
    @PostMapping("/device/offline")
    public AjaxResult offline(String sn)
    {
        logger.info("【车辆下线请求】：{}",sn);
        Boolean aBoolean =asDeviceService.offline(sn, ServiceConstants.VEHICLE_STATUS_IN_OFFLINE);
        return success(aBoolean);
    }

    /**
     * 根据时间查询车辆轨迹
     */
    @PostMapping("/trajectory")
    public AjaxResult trajectory(String sn,String startTime,String endTime)
    {
        AjaxResult ajax = AjaxResult.success();
        if(StrUtil.isBlank(sn)){
            logger.info("没有sn号参数：【sn={}】",sn);
            return error("请传sn号参数"+"【sn="+sn+"】");
        }
        logger.info("【根据时间查询车辆轨迹】：{}",sn,startTime,endTime);
        String trajectory = asDeviceService.trajectory(sn,startTime,endTime);
        ajax.put(AjaxResult.DATA_TAG,trajectory);
        return ajax;
    }

    /**
     * 根据订单号查询车辆轨迹
     */
    @PostMapping("/trajectoryByOrderNo")
    public AjaxResult trajectoryByOrderNo(String orderNo)
    {
        AjaxResult ajax = AjaxResult.success();
        if(StrUtil.isBlank(orderNo)){
            logger.info("没有orderNo号参数：【orderNo={}】",orderNo);
            return error("请传订单号参数"+"【orderNo="+orderNo+"】");
        }
        logger.info("【根据订单号查询车辆轨迹】：orderNo={}",orderNo);
        String trajectory = asDeviceService.trajectoryByOrderNo(orderNo);
        ajax.put(AjaxResult.DATA_TAG,trajectory);
        return ajax;
    }


}
