package com.ruoyi.web.controller.system;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.EtAdminOrder;
import com.ruoyi.system.service.IEtAdminOrderService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 换电工单Controller
 *
 * @author 邱贞招
 * @date 2024-05-05
 */
@RestController
@RequestMapping("/system/adminOrder")
public class EtAdminOrderController extends BaseController
{
    @Autowired
    private IEtAdminOrderService etAdminOrderService;

    /**
     * 查询换电工单列表
     */
    @PreAuthorize("@ss.hasPermi('system:order:list')")
    @GetMapping("/list")
    public TableDataInfo list(EtAdminOrder etAdminOrder)
    {
        startPage();
        List<EtAdminOrder> list = etAdminOrderService.selectEtAdminOrderList(etAdminOrder);
        return getDataTable(list);
    }

    /**
     * 导出换电工单列表
     */
    @PreAuthorize("@ss.hasPermi('system:order:export')")
    @Log(title = "换电工单", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EtAdminOrder etAdminOrder)
    {
        List<EtAdminOrder> list = etAdminOrderService.selectEtAdminOrderList(etAdminOrder);
        ExcelUtil<EtAdminOrder> util = new ExcelUtil<EtAdminOrder>(EtAdminOrder.class);
        util.exportExcel(response, list, "换电工单数据");
    }

    /**
     * 获取换电工单详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:order:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(etAdminOrderService.selectEtAdminOrderById(id));
    }

    /**
     * 新增换电工单
     */
    @PreAuthorize("@ss.hasPermi('system:order:add')")
    @Log(title = "换电工单", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EtAdminOrder etAdminOrder)
    {
        return toAjax(etAdminOrderService.insertEtAdminOrder(etAdminOrder));
    }

    /**
     * 修改换电工单
     */
    @PreAuthorize("@ss.hasPermi('system:order:edit')")
    @Log(title = "换电工单", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EtAdminOrder etAdminOrder)
    {
        return toAjax(etAdminOrderService.updateEtAdminOrder(etAdminOrder));
    }

    /**
     * 删除换电工单
     */
    @PreAuthorize("@ss.hasPermi('system:order:remove')")
    @Log(title = "换电工单", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(etAdminOrderService.deleteEtAdminOrderByIds(ids));
    }
}
