package com.ruoyi.web.controller.system;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.EtAdminOrderHistory;
import com.ruoyi.system.service.IEtAdminOrderHistoryService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 工单履历Controller
 *
 * @author 邱贞招
 * @date 2024-05-06
 */
@RestController
@RequestMapping("/system/history")
public class EtAdminOrderHistoryController extends BaseController
{
    @Autowired
    private IEtAdminOrderHistoryService etAdminOrderHistoryService;

    /**
     * 查询工单履历列表
     */
    @PreAuthorize("@ss.hasPermi('system:history:list')")
    @GetMapping("/list")
    public TableDataInfo list(EtAdminOrderHistory etAdminOrderHistory)
    {
        startPage();
        List<EtAdminOrderHistory> list = etAdminOrderHistoryService.selectEtAdminOrderHistoryList(etAdminOrderHistory);
        return getDataTable(list);
    }

    /**
     * 导出工单履历列表
     */
    @PreAuthorize("@ss.hasPermi('system:history:export')")
    @Log(title = "工单履历", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EtAdminOrderHistory etAdminOrderHistory)
    {
        List<EtAdminOrderHistory> list = etAdminOrderHistoryService.selectEtAdminOrderHistoryList(etAdminOrderHistory);
        ExcelUtil<EtAdminOrderHistory> util = new ExcelUtil<EtAdminOrderHistory>(EtAdminOrderHistory.class);
        util.exportExcel(response, list, "工单履历数据");
    }

    /**
     * 获取工单履历详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:history:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(etAdminOrderHistoryService.selectEtAdminOrderHistoryById(id));
    }

    /**
     * 新增工单履历
     */
    @PreAuthorize("@ss.hasPermi('system:history:add')")
    @Log(title = "工单履历", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EtAdminOrderHistory etAdminOrderHistory)
    {
        return toAjax(etAdminOrderHistoryService.insertEtAdminOrderHistory(etAdminOrderHistory));
    }

    /**
     * 修改工单履历
     */
    @PreAuthorize("@ss.hasPermi('system:history:edit')")
    @Log(title = "工单履历", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EtAdminOrderHistory etAdminOrderHistory)
    {
        return toAjax(etAdminOrderHistoryService.updateEtAdminOrderHistory(etAdminOrderHistory));
    }

    /**
     * 删除工单履历
     */
    @PreAuthorize("@ss.hasPermi('system:history:remove')")
    @Log(title = "工单履历", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(etAdminOrderHistoryService.deleteEtAdminOrderHistoryByIds(ids));
    }
}
