package com.ruoyi.web.controller.system;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.EtCapitalFlow;
import com.ruoyi.system.domain.EtWithdraw;
import com.ruoyi.system.service.IEtCapitalFlowService;
import com.ruoyi.system.service.ISysConfigService;
import com.ruoyi.system.service.ISysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.util.List;

/**
 * 资金流水Controller
 *
 * @author 邱贞招
 * @date 2024-05-24
 */
@RestController
@RequestMapping("/system/flow")
public class EtCapitalFlowController extends BaseController
{
    @Autowired
    private IEtCapitalFlowService etCapitalFlowService;

    @Autowired
    private ISysDeptService deptService;

    @Autowired
    private ISysConfigService sysConfigService;

    /**
     * 查询资金流水列表
     */
    @PreAuthorize("@ss.hasPermi('system:flow:list')")
    @GetMapping("/list")
    public TableDataInfo list(EtCapitalFlow etCapitalFlow)
    {
        startPage();
        List<EtCapitalFlow> list = etCapitalFlowService.selectEtCapitalFlowList(etCapitalFlow);
        return getDataTable(list);
    }

    /**
     * 导出资金流水列表
     */
    @PreAuthorize("@ss.hasPermi('system:flow:export')")
    @Log(title = "资金流水", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EtCapitalFlow etCapitalFlow)
    {
        List<EtCapitalFlow> list = etCapitalFlowService.selectEtCapitalFlowList(etCapitalFlow);
        ExcelUtil<EtCapitalFlow> util = new ExcelUtil<EtCapitalFlow>(EtCapitalFlow.class);
        util.exportExcel(response, list, "资金流水数据");
    }

    /**
     * 获取资金流水详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:flow:query')")
    @GetMapping(value = "/{flowId}")
    public AjaxResult getInfo(@PathVariable("flowId") Long flowId)
    {
        return success(etCapitalFlowService.selectEtCapitalFlowByFlowId(flowId));
    }

    /**
     * 新增资金流水
     */
    @PreAuthorize("@ss.hasPermi('system:flow:add')")
    @Log(title = "资金流水", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EtCapitalFlow etCapitalFlow)
    {
        return toAjax(etCapitalFlowService.insertEtCapitalFlow(etCapitalFlow));
    }

    /**
     * 修改资金流水
     */
    @PreAuthorize("@ss.hasPermi('system:flow:edit')")
    @Log(title = "资金流水", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EtCapitalFlow etCapitalFlow)
    {
        return toAjax(etCapitalFlowService.updateEtCapitalFlow(etCapitalFlow));
    }

    /**
     * 删除资金流水
     */
    @PreAuthorize("@ss.hasPermi('system:flow:remove')")
    @Log(title = "资金流水", businessType = BusinessType.DELETE)
	@DeleteMapping("/{flowIds}")
    public AjaxResult remove(@PathVariable Long[] flowIds)
    {
        return toAjax(etCapitalFlowService.deleteEtCapitalFlowByFlowIds(flowIds));
    }

    /**
     * 管理员提现
     */
    @Log(title = "管理员提现", businessType = BusinessType.ADMINWITHDRAW)
    @PostMapping("/withdraw")
    public AjaxResult add(@RequestBody EtWithdraw etWithdraw)
    {
        logger.info("管理员提现请求：【{}】", JSON.toJSON(etWithdraw));
        Long deptId = etWithdraw.getDeptId();
        if(ObjectUtil.isNull(deptId)){
            deptId = getDeptId();
        }
        String minimumWithdrawalAmount = sysConfigService.selectConfigByKey("minimum.withdrawal.amount");//最低提现金额
        BigDecimal minimum = new BigDecimal(minimumWithdrawalAmount);
        if(etWithdraw.getAmount().compareTo(minimum) < 0){
            throw new RuntimeException("最低提现金额不能小于"+minimum);
        }
        SysDept sysDept = deptService.selectDeptById(deptId);
        String method = etWithdraw.getMethod();
        if(method.equals(ServiceConstants.WITHDRAW_METHOD_WX)){
            if(ObjectUtil.isNull(sysDept.getAppUserId())){
                throw new RuntimeException("运营商未绑定app用户,请联系管理员添加【绑定微信用户】");
            }
        }
        logger.info("【管理员提现】获取当前部门：【{}】", JSON.toJSON(sysDept));
        int i = deptService.adminWithdraw(etWithdraw.getAmount(),sysDept,method);
        return toAjax(i);
    }

    /**
     * 提现失败
     */
    @Log(title = "管理员提现失败", businessType = BusinessType.ADMINWITHDRAW)
    @PostMapping("/rejectWithdraw")
    public AjaxResult rejectWithdraw(@RequestBody EtCapitalFlow flow)
    {
        logger.info("管理员提现失败请求：【{}】", JSON.toJSON(flow));
        int i = deptService.rejectWithdraw(flow.getFlowId());
        return toAjax(i);
    }
}
