package com.ruoyi.web.controller.system;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.EtCommandLog;
import com.ruoyi.system.service.IEtCommandLogService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 命令日志Controller
 *
 * @author 邱贞招
 * @date 2024-07-06
 */
@RestController
@RequestMapping("/system/commandLog")
public class EtCommandLogController extends BaseController
{
    @Autowired
    private IEtCommandLogService etCommandLogService;

    /**
     * 查询命令日志列表
     */
    @PreAuthorize("@ss.hasPermi('system:commandLog:list')")
    @GetMapping("/list")
    public TableDataInfo list(EtCommandLog etCommandLog)
    {
        startPage();
        List<EtCommandLog> list = etCommandLogService.selectEtCommandLogList(etCommandLog);
        return getDataTable(list);
    }

    /**
     * 导出命令日志列表
     */
    @PreAuthorize("@ss.hasPermi('system:commandLog:export')")
    @Log(title = "命令日志", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EtCommandLog etCommandLog)
    {
        List<EtCommandLog> list = etCommandLogService.selectEtCommandLogList(etCommandLog);
        ExcelUtil<EtCommandLog> util = new ExcelUtil<EtCommandLog>(EtCommandLog.class);
        util.exportExcel(response, list, "命令日志数据");
    }

    /**
     * 获取命令日志详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:commandLog:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(etCommandLogService.selectEtCommandLogById(id));
    }

    /**
     * 新增命令日志
     */
    @PreAuthorize("@ss.hasPermi('system:commandLog:add')")
    @Log(title = "命令日志", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EtCommandLog etCommandLog)
    {
        return toAjax(etCommandLogService.insertEtCommandLog(etCommandLog));
    }

    /**
     * 修改命令日志
     */
    @PreAuthorize("@ss.hasPermi('system:commandLog:edit')")
    @Log(title = "命令日志", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EtCommandLog etCommandLog)
    {
        return toAjax(etCommandLogService.updateEtCommandLog(etCommandLog));
    }

    /**
     * 删除命令日志
     */
    @PreAuthorize("@ss.hasPermi('system:commandLog:remove')")
    @Log(title = "命令日志", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(etCommandLogService.deleteEtCommandLogByIds(ids));
    }
}
