package com.ruoyi.web.controller.system;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.EtCouponUserLog;
import com.ruoyi.system.service.IEtCouponClaimLogService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 优惠券领取记录Controller
 *
 * @author qzz
 * @date 2024-08-08
 */
@RestController
@RequestMapping("/system/claimLog")
public class EtCouponClaimLogController extends BaseController
{
    @Autowired
    private IEtCouponClaimLogService etCouponClaimLogService;

    /**
     * 查询优惠券领取记录列表
     */
    @PreAuthorize("@ss.hasPermi('system:claimLog:list')")
    @GetMapping("/list")
    public TableDataInfo list(EtCouponUserLog etCouponUserLog)
    {
        startPage();
        List<EtCouponUserLog> list = etCouponClaimLogService.selectEtCouponClaimLogList(etCouponUserLog);
        return getDataTable(list);
    }

    /**
     * 导出优惠券领取记录列表
     */
    @PreAuthorize("@ss.hasPermi('system:claimLog:export')")
    @Log(title = "优惠券领取记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EtCouponUserLog etCouponUserLog)
    {
        List<EtCouponUserLog> list = etCouponClaimLogService.selectEtCouponClaimLogList(etCouponUserLog);
        ExcelUtil<EtCouponUserLog> util = new ExcelUtil<EtCouponUserLog>(EtCouponUserLog.class);
        util.exportExcel(response, list, "优惠券领取记录数据");
    }

    /**
     * 获取优惠券领取记录详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:claimLog:query')")
    @GetMapping(value = "/{claimId}")
    public AjaxResult getInfo(@PathVariable("claimId") Long claimId)
    {
        return success(etCouponClaimLogService.selectEtCouponClaimLogByLogId(claimId));
    }

    /**
     * 新增优惠券领取记录
     */
    @PreAuthorize("@ss.hasPermi('system:claimLog:add')")
    @Log(title = "优惠券领取记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EtCouponUserLog etCouponUserLog)
    {
        return toAjax(etCouponClaimLogService.insertEtCouponClaimLog(etCouponUserLog));
    }

    /**
     * 修改优惠券领取记录
     */
    @PreAuthorize("@ss.hasPermi('system:claimLog:edit')")
    @Log(title = "优惠券领取记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EtCouponUserLog etCouponUserLog)
    {
        return toAjax(etCouponClaimLogService.updateEtCouponClaimLog(etCouponUserLog));
    }

    /**
     * 删除优惠券领取记录
     */
    @PreAuthorize("@ss.hasPermi('system:claimLog:remove')")
    @Log(title = "优惠券领取记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{claimIds}")
    public AjaxResult remove(@PathVariable Long[] claimIds)
    {
        return toAjax(etCouponClaimLogService.deleteEtCouponClaimLogByLogIds(claimIds));
    }
}
