package com.ruoyi.web.controller.system;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.EtFault;
import com.ruoyi.system.service.IEtFaultService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 故障上报Controller
 *
 * @author 邱贞招
 * @date 2024-04-06
 */
@RestController
@RequestMapping("/system/fault")
public class EtFaultController extends BaseController
{
    @Autowired
    private IEtFaultService etFaultService;

    /**
     * 查询故障上报列表
     */
    @PreAuthorize("@ss.hasPermi('system:fault:list')")
    @GetMapping("/list")
    public TableDataInfo list(EtFault etFault)
    {
        startPage();
        List<EtFault> list = etFaultService.selectEtFaultList(etFault);
        return getDataTable(list);
    }

    /**
     * 导出故障上报列表
     */
    @PreAuthorize("@ss.hasPermi('system:fault:export')")
    @Log(title = "故障上报", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EtFault etFault)
    {
        List<EtFault> list = etFaultService.selectEtFaultList(etFault);
        ExcelUtil<EtFault> util = new ExcelUtil<EtFault>(EtFault.class);
        util.exportExcel(response, list, "故障上报数据");
    }

    /**
     * 获取故障上报详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:fault:query')")
    @GetMapping(value = "/{faultId}")
    public AjaxResult getInfo(@PathVariable("faultId") Long faultId)
    {
        return success(etFaultService.selectEtFaultByFaultId(faultId));
    }

    /**
     * 新增故障上报
     */
    @PreAuthorize("@ss.hasPermi('system:fault:add')")
    @Log(title = "故障上报", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EtFault etFault)
    {
        return toAjax(etFaultService.insertEtFault(etFault));
    }

    /**
     * 修改故障上报
     */
    @PreAuthorize("@ss.hasPermi('system:fault:edit')")
    @Log(title = "故障上报", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EtFault etFault)
    {
        return toAjax(etFaultService.updateEtFault(etFault));
    }

    /**
     * 删除故障上报
     */
    @PreAuthorize("@ss.hasPermi('system:fault:remove')")
    @Log(title = "故障上报", businessType = BusinessType.DELETE)
	@DeleteMapping("/{faultIds}")
    public AjaxResult remove(@PathVariable Long[] faultIds)
    {
        return toAjax(etFaultService.deleteEtFaultByFaultIds(faultIds));
    }
}
