package com.ruoyi.web.controller.system;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.EtHardwareVersion;
import com.ruoyi.system.service.IEtHardwareVersionService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 硬件版本Controller
 *
 * @author qiuzhenzhao
 * @date 2024-07-22
 */
@RestController
@RequestMapping("/system/hardwareVersion")
public class EtHardwareVersionController extends BaseController
{
    @Autowired
    private IEtHardwareVersionService etHardwareVersionService;

    /**
     * 查询硬件版本列表
     */
    @PreAuthorize("@ss.hasPermi('system:hardwareVersion:list')")
    @GetMapping("/list")
    public TableDataInfo list(EtHardwareVersion etHardwareVersion)
    {
        startPage();
        List<EtHardwareVersion> list = etHardwareVersionService.selectEtHardwareVersionList(etHardwareVersion);
        return getDataTable(list);
    }

    /**
     * 查询硬件版本列表
     */
    @PreAuthorize("@ss.hasPermi('system:hardwareVersion:list')")
    @GetMapping("/list/exclude")
    public AjaxResult listExclude(EtHardwareVersion etHardwareVersion)
    {
        List<EtHardwareVersion> list = etHardwareVersionService.selectEtHardwareVersionListExclude(etHardwareVersion);
        return success(list);
    }

    /**
     * 导出硬件版本列表
     */
    @PreAuthorize("@ss.hasPermi('system:hardwareVersion:export')")
    @Log(title = "硬件版本", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EtHardwareVersion etHardwareVersion)
    {
        List<EtHardwareVersion> list = etHardwareVersionService.selectEtHardwareVersionList(etHardwareVersion);
        ExcelUtil<EtHardwareVersion> util = new ExcelUtil<EtHardwareVersion>(EtHardwareVersion.class);
        util.exportExcel(response, list, "硬件版本数据");
    }

    /**
     * 获取硬件版本详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:hardwareVersion:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(etHardwareVersionService.selectEtHardwareVersionById(id));
    }

    /**
     * 新增硬件版本
     */
    @PreAuthorize("@ss.hasPermi('system:hardwareVersion:add')")
    @Log(title = "硬件版本", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EtHardwareVersion etHardwareVersion)
    {
        return toAjax(etHardwareVersionService.insertEtHardwareVersion(etHardwareVersion));
    }

    /**
     * 修改硬件版本
     */
    @PreAuthorize("@ss.hasPermi('system:hardwareVersion:edit')")
    @Log(title = "硬件版本", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EtHardwareVersion etHardwareVersion)
    {
        return toAjax(etHardwareVersionService.updateEtHardwareVersion(etHardwareVersion));
    }

    /**
     * 删除硬件版本
     */
    @PreAuthorize("@ss.hasPermi('system:hardwareVersion:remove')")
    @Log(title = "硬件版本", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(etHardwareVersionService.deleteEtHardwareVersionByIds(ids));
    }
}
