package com.ruoyi.web.controller.system;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.EtMsgLog;
import com.ruoyi.system.service.IEtMsgLogService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 短信日志Controller
 *
 * @author qiuzhenzhen
 * @date 2024-07-24
 */
@RestController
@RequestMapping("/system/msgLog")
public class EtMsgLogController extends BaseController
{
    @Autowired
    private IEtMsgLogService etMsgLogService;

    /**
     * 查询短信日志列表
     */
    @PreAuthorize("@ss.hasPermi('system:msgLog:list')")
    @GetMapping("/list")
    public TableDataInfo list(EtMsgLog etMsgLog)
    {
        startPage();
        List<EtMsgLog> list = etMsgLogService.selectEtMsgLogList(etMsgLog);
        return getDataTable(list);
    }

    /**
     * 导出短信日志列表
     */
    @PreAuthorize("@ss.hasPermi('system:msgLog:export')")
    @Log(title = "短信日志", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EtMsgLog etMsgLog)
    {
        List<EtMsgLog> list = etMsgLogService.selectEtMsgLogList(etMsgLog);
        ExcelUtil<EtMsgLog> util = new ExcelUtil<EtMsgLog>(EtMsgLog.class);
        util.exportExcel(response, list, "短信日志数据");
    }

    /**
     * 获取短信日志详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:msgLog:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(etMsgLogService.selectEtMsgLogById(id));
    }

    /**
     * 新增短信日志
     */
    @PreAuthorize("@ss.hasPermi('system:msgLog:add')")
    @Log(title = "短信日志", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EtMsgLog etMsgLog)
    {
        return toAjax(etMsgLogService.insertEtMsgLog(etMsgLog));
    }

    /**
     * 修改短信日志
     */
    @PreAuthorize("@ss.hasPermi('system:msgLog:edit')")
    @Log(title = "短信日志", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EtMsgLog etMsgLog)
    {
        return toAjax(etMsgLogService.updateEtMsgLog(etMsgLog));
    }

    /**
     * 删除短信日志
     */
    @PreAuthorize("@ss.hasPermi('system:msgLog:remove')")
    @Log(title = "短信日志", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(etMsgLogService.deleteEtMsgLogByIds(ids));
    }
}
