package com.ruoyi.web.controller.system;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.EtOnlineLog;
import com.ruoyi.system.service.IEtOnlineLogService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 在线/离线日志Controller
 *
 * @author 邱贞招
 * @date 2024-07-13
 */
@RestController
@RequestMapping("/system/onlineLog")
public class EtOnlineLogController extends BaseController
{
    @Autowired
    private IEtOnlineLogService etOnlineLogService;

    /**
     * 查询在线/离线日志列表
     */
    @PreAuthorize("@ss.hasPermi('system:onlineLog:list')")
    @GetMapping("/list")
    public TableDataInfo list(EtOnlineLog etOnlineLog)
    {
        startPage();
        List<EtOnlineLog> list = etOnlineLogService.selectEtOnlineLogList(etOnlineLog);
        return getDataTable(list);
    }

    /**
     * 导出在线/离线日志列表
     */
    @PreAuthorize("@ss.hasPermi('system:onlineLog:export')")
    @Log(title = "在线/离线日志", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EtOnlineLog etOnlineLog)
    {
        List<EtOnlineLog> list = etOnlineLogService.selectEtOnlineLogList(etOnlineLog);
        ExcelUtil<EtOnlineLog> util = new ExcelUtil<EtOnlineLog>(EtOnlineLog.class);
        util.exportExcel(response, list, "在线/离线日志数据");
    }

    /**
     * 获取在线/离线日志详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:onlineLog:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(etOnlineLogService.selectEtOnlineLogById(id));
    }

    /**
     * 新增在线/离线日志
     */
    @PreAuthorize("@ss.hasPermi('system:onlineLog:add')")
    @Log(title = "在线/离线日志", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EtOnlineLog etOnlineLog)
    {
        return toAjax(etOnlineLogService.insertEtOnlineLog(etOnlineLog));
    }

    /**
     * 修改在线/离线日志
     */
    @PreAuthorize("@ss.hasPermi('system:onlineLog:edit')")
    @Log(title = "在线/离线日志", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EtOnlineLog etOnlineLog)
    {
        return toAjax(etOnlineLogService.updateEtOnlineLog(etOnlineLog));
    }

    /**
     * 删除在线/离线日志
     */
    @PreAuthorize("@ss.hasPermi('system:onlineLog:remove')")
    @Log(title = "在线/离线日志", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(etOnlineLogService.deleteEtOnlineLogByIds(ids));
    }
}
