package com.ruoyi.web.controller.system;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.EtPowerReplacement;
import com.ruoyi.system.service.IEtPowerReplacementService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 换电工单Controller
 *
 * @author 邱贞招
 * @date 2024-04-28
 */
@RestController
@RequestMapping("/system/replacement")
public class EtPowerReplacementController extends BaseController
{
    @Autowired
    private IEtPowerReplacementService etPowerReplacementService;

    /**
     * 查询换电工单列表
     */
    @PreAuthorize("@ss.hasPermi('system:replacement:list')")
    @GetMapping("/list")
    public TableDataInfo list(EtPowerReplacement etPowerReplacement)
    {
        startPage();
        List<EtPowerReplacement> list = etPowerReplacementService.selectEtPowerReplacementList(etPowerReplacement);
        return getDataTable(list);
    }

    /**
     * 导出换电工单列表
     */
    @PreAuthorize("@ss.hasPermi('system:replacement:export')")
    @Log(title = "换电工单", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EtPowerReplacement etPowerReplacement)
    {
        List<EtPowerReplacement> list = etPowerReplacementService.selectEtPowerReplacementList(etPowerReplacement);
        ExcelUtil<EtPowerReplacement> util = new ExcelUtil<EtPowerReplacement>(EtPowerReplacement.class);
        util.exportExcel(response, list, "换电工单数据");
    }

    /**
     * 获取换电工单详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:replacement:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(etPowerReplacementService.selectEtPowerReplacementById(id));
    }

    /**
     * 新增换电工单
     */
    @PreAuthorize("@ss.hasPermi('system:replacement:add')")
    @Log(title = "换电工单", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EtPowerReplacement etPowerReplacement)
    {
        return toAjax(etPowerReplacementService.insertEtPowerReplacement(etPowerReplacement));
    }

    /**
     * 修改换电工单
     */
    @PreAuthorize("@ss.hasPermi('system:replacement:edit')")
    @Log(title = "换电工单", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EtPowerReplacement etPowerReplacement)
    {
        return toAjax(etPowerReplacementService.updateEtPowerReplacement(etPowerReplacement));
    }

    /**
     * 删除换电工单
     */
    @PreAuthorize("@ss.hasPermi('system:replacement:remove')")
    @Log(title = "换电工单", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(etPowerReplacementService.deleteEtPowerReplacementByIds(ids));
    }
}
