package com.ruoyi.web.controller.system;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.domain.EtOperatingArea;
import com.ruoyi.system.domain.vo.ReconciliationVo;
import com.ruoyi.system.service.IEtOperatingAreaService;
import com.ruoyi.system.service.IEtOrderService;
import com.ruoyi.system.service.ISysUserService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

/**
 * 对账Controller
 *
 * @author qiuzhenzhao
 * @date 2024-06-17
 */
@RestController
@RequestMapping("/system")
public class EtReconciliationControllor extends BaseController {

    protected final Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private IEtOrderService etOrderService;

    @Autowired
    private IEtOperatingAreaService etOperatingAreaService;

    @Resource
    private ISysUserService sysUserService;

    /**
     * 查询对账列表
     */
    @PreAuthorize("@ss.hasPermi('system:reconciliation:list')")
    @GetMapping("/reconciliation")
    public AjaxResult list(String timeStart,String timeEnd,String areaId)
    {
        logger.info("【收入对账】请求参数：timeStart={},timeEnd={},areaId={}", timeStart,timeEnd,areaId);
        if(StrUtil.isBlank(timeStart)){
            timeStart = DateUtils.getDate();
        }
        if(StrUtil.isBlank(timeEnd)){
            timeEnd = DateUtils.getDate();
        }
        // 根据token获取运营区列表
        ReconciliationVo reconciliation = etOrderService.reconciliation(timeStart,timeEnd,"1",null,areaId);
        return AjaxResult.success(reconciliation);
    }

    /**
     * 根据token获取运营区列表
     */
    @GetMapping("/getAreaList")
    public AjaxResult getAreaList()
    {
        List<EtOperatingArea> longs;
        SysUser sysUser = sysUserService.selectUserById(getUserId());
        if(getUserId() == 1){
            longs = etOperatingAreaService.selectAreaListByDeptId2(null,null);
        }else{
            if(ObjectUtil.isNotNull(sysUser.getAreaId())){
                longs = etOperatingAreaService.selectAreaListByDeptId2(null,sysUser.getAreaId());
            }else{
                if(ObjectUtil.isNotNull(sysUser.getDeptId())){
                    longs = etOperatingAreaService.selectAreaListByDeptId2(sysUser.getDeptId(),null);
                }else{
                    throw new ServiceException("该用户都没有绑定运营区和运营商（至少绑定一个）");
                }
            }
        }
        logger.info("根据token获取运营区列表：【{}】", JSON.toJSON(longs));
        return success(longs);
    }
}
