package com.ruoyi.web.controller.system;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.EtTripLog;
import com.ruoyi.system.service.IEtTripLogService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 行程记录Controller
 *
 * @author 邱贞招
 * @date 2024-05-13
 */
@RestController
@RequestMapping("/system/log")
public class EtTripLogController extends BaseController
{
    @Autowired
    private IEtTripLogService etTripLogService;

    /**
     * 查询行程记录列表
     */
    @PreAuthorize("@ss.hasPermi('system:log:list')")
    @GetMapping("/list")
    public TableDataInfo list(EtTripLog etTripLog)
    {
        startPage();
        List<EtTripLog> list = etTripLogService.selectEtTripLogList(etTripLog);
        return getDataTable(list);
    }

    /**
     * 导出行程记录列表
     */
    @PreAuthorize("@ss.hasPermi('system:log:export')")
    @Log(title = "行程记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EtTripLog etTripLog)
    {
        List<EtTripLog> list = etTripLogService.selectEtTripLogList(etTripLog);
        ExcelUtil<EtTripLog> util = new ExcelUtil<EtTripLog>(EtTripLog.class);
        util.exportExcel(response, list, "行程记录数据");
    }

    /**
     * 获取行程记录详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:log:query')")
    @GetMapping(value = "/{tripId}")
    public AjaxResult getInfo(@PathVariable("tripId") Long tripId)
    {
        return success(etTripLogService.selectEtTripLogByTripId(tripId));
    }

    /**
     * 新增行程记录
     */
    @PreAuthorize("@ss.hasPermi('system:log:add')")
    @Log(title = "行程记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EtTripLog etTripLog)
    {
        return toAjax(etTripLogService.insertEtTripLog(etTripLog));
    }

    /**
     * 修改行程记录
     */
    @PreAuthorize("@ss.hasPermi('system:log:edit')")
    @Log(title = "行程记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EtTripLog etTripLog)
    {
        return toAjax(etTripLogService.updateEtTripLog(etTripLog));
    }

    /**
     * 删除行程记录
     */
    @PreAuthorize("@ss.hasPermi('system:log:remove')")
    @Log(title = "行程记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{tripIds}")
    public AjaxResult remove(@PathVariable Long[] tripIds)
    {
        return toAjax(etTripLogService.deleteEtTripLogByTripIds(tripIds));
    }
}
