package com.ruoyi.system.mapper;

import com.ruoyi.common.core.domain.entity.AsArticleClassify;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 分类管理 数据层
 *
 * @author ruoyi
 */
public interface AsArticleClassifyMapper
{
    /**
     * 查询分类管理数据
     *
     * @param classify 分类信息
     * @return 分类信息集合
     */
    public List<AsArticleClassify> selectClassifyList(AsArticleClassify classify);

    /**
     * 根据角色ID查询分类树信息
     *
     * @param roleId 角色ID
     * @param deptCheckStrictly 分类树选择项是否关联显示
     * @return 选中分类列表
     */
//    public List<Long> selectClassifyListByRoleId(@Param("roleId") Long roleId, @Param("deptCheckStrictly") boolean deptCheckStrictly);

    /**
     * 根据分类ID查询信息
     *
     * @param classifyId 分类ID
     * @return 分类信息
     */
    public AsArticleClassify selectClassifyById(Long classifyId);

    /**
     * 根据ID查询所有子分类
     *
     * @param classifyId 分类ID
     * @return 分类列表
     */
    public List<AsArticleClassify> selectChildrenClassifyById(Long classifyId);

    /**
     * 根据ID查询所有子分类（正常状态）
     *
     * @param classifyId 分类ID
     * @return 子分类数
     */
    public int selectNormalChildrenClassifyById(Long classifyId);

    /**
     * 是否存在子节点
     *
     * @param classifyId 分类ID
     * @return 结果
     */
    public int hasChildByClassifyId(Long classifyId);

    /**
     * 查询分类是否存在用户
     *
     * @param classifyId 分类ID
     * @return 结果
     */
    public int checkClassifyExistArticle(Long classifyId);

    /**
     * 校验分类名称是否唯一
     *
     * @param classifyName 分类名称
     * @param parentId 父分类ID
     * @return 结果
     */
    public AsArticleClassify checkClassifyNameUnique(@Param("classifyName") String classifyName, @Param("parentId") Long parentId);

    /**
     * 新增分类信息
     *
     * @param dept 分类信息
     * @return 结果
     */
    public int insertClassify(AsArticleClassify dept);

    /**
     * 修改分类信息
     *
     * @param dept 分类信息
     * @return 结果
     */
    public int updateClassify(AsArticleClassify dept);

    /**
     * 修改所在分类正常状态
     *
     * @param classifyIds 分类ID组
     */
    public void updateClassifyStatusNormal(Long[] classifyIds);

    /**
     * 修改子元素关系
     *
     * @param classifyIds 子元素
     * @return 结果
     */
    public int updateClassifyChildren(@Param("classifyIds") List<AsArticleClassify> classifyIds);

    /**
     * 删除分类管理信息
     *
     * @param classifyId 分类ID
     * @return 结果
     */
    public int deleteClassifyById(Long classifyId);
}
