package com.ruoyi.system.service;

import java.util.List;
import com.ruoyi.system.domain.EtCoupon;
import com.ruoyi.system.domain.vo.CouponVo;
import com.wechat.pay.java.service.payments.jsapi.model.PrepayWithRequestPaymentResponse;

/**
 * 优惠券Service接口
 *
 * @author 邱贞招
 * @date 2024-08-05
 */
public interface IEtCouponService
{
    /**
     * 查询优惠券
     *
     * @param couponId 优惠券主键
     * @return 优惠券
     */
    public EtCoupon selectEtCouponByCouponId(Long couponId);

    /**
     * 查询优惠券列表
     *
     * @param etCoupon 优惠券
     * @return 优惠券集合
     */
    public List<EtCoupon> selectEtCouponList(EtCoupon etCoupon);

    /**
     * 新增优惠券
     *
     * @param etCoupon 优惠券
     * @return 结果
     */
    public int insertEtCoupon(EtCoupon etCoupon);

    /**
     * 修改优惠券
     *
     * @param etCoupon 优惠券
     * @return 结果
     */
    public int updateEtCoupon(EtCoupon etCoupon);

    /**
     * 批量删除优惠券
     *
     * @param couponIds 需要删除的优惠券主键集合
     * @return 结果
     */
    public int deleteEtCouponByCouponIds(Long[] couponIds);

    /**
     * 删除优惠券信息
     *
     * @param couponId 优惠券主键
     * @return 结果
     */
    public int deleteEtCouponByCouponId(Long couponId);

    /**
     * 优惠券购买
     * @param userId
     * @param couponId
     * @return
     */
    PrepayWithRequestPaymentResponse couponBuy(Long userId, Long couponId);

    /**
     * 下发优惠券
     *
     * @param couponVo
     * @return 结果
     */
    public int issue(CouponVo couponVo);
}
