package com.ruoyi.system.service;

import java.util.List;
import com.ruoyi.system.domain.EtFeeRule;
import com.ruoyi.system.domain.SysPost;

/**
 * 收费方式Service接口
 *
 * @author 邱贞招
 * @date 2024-04-08
 */
public interface IEtFeeRuleService
{
    /**
     * 查询收费方式
     *
     * @param ruleId 收费方式主键
     * @return 收费方式
     */
    public EtFeeRule selectEtFeeRuleByRuleId(Long ruleId);

    /**
     * 查询收费方式
     *
     * @param ruleId 收费方式主键
     * @return 收费方式
     */
    public EtFeeRule selectEtFeeRuleByRuleIdIncludeDelete(Long ruleId);

    /**
     * 查询收费方式列表
     *
     * @param etFeeRule 收费方式
     * @return 收费方式集合
     */
    public List<EtFeeRule> selectEtFeeRuleList(EtFeeRule etFeeRule);

    /**
     * 根据运营区获取收费方式列表
     *
     * @param areaId 运营区id
     * @return 收费方式集合
     */
    public List<EtFeeRule> selectEtFeeRuleListByAreaId(Long areaId);


    /**
     * 新增收费方式
     *
     * @param etFeeRule 收费方式
     * @return 结果
     */
    public int insertEtFeeRule(EtFeeRule etFeeRule);

    /**
     * 修改收费方式
     *
     * @param etFeeRule 收费方式
     * @return 结果
     */
    public int updateEtFeeRule(EtFeeRule etFeeRule);

    /**
     * 批量删除收费方式
     *
     * @param ruleIds 需要删除的收费方式主键集合
     * @return 结果
     */
    public int deleteEtFeeRuleByRuleIds(Long[] ruleIds);

    /**
     * 删除收费方式信息
     *
     * @param ruleId 收费方式主键
     * @return 结果
     */
    public int deleteEtFeeRuleByRuleId(Long ruleId);


    /**
     * 根据区域ID获取收费方式选择框列表
     *
     * @param areaId 用户ID
     * @return 选中收费方式ID列表
     */
    public List<Long> selectRuleListByAreaId(Long areaId);

    /**
     * 根据区域ID获取收费方式名称选择框列表
     *
     * @param areaId 用户ID
     * @return 选中收费方式名称列表
     */
    public List<String> selectRuleNameListByAreaId(Long areaId);
}
