package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.core.domain.entity.AsArticleClassify;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.domain.EtArticle;
import com.ruoyi.system.domain.EtOperatingArea;
import com.ruoyi.system.mapper.AsArticleClassifyMapper;
import com.ruoyi.system.mapper.AsArticleMapper;
import com.ruoyi.system.mapper.EtOperatingAreaMapper;
import com.ruoyi.system.mapper.SysDictDataMapper;
import com.ruoyi.system.service.IAsArticleClassifyService;
import com.ruoyi.system.service.IAsArticleService;
import com.ruoyi.system.service.IWxPayService;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * 文章Service业务层处理
 *
 * @author qiuzhenzhao
 * @date 2023-12-06
 */
@Service
public class AsArticleServiceImpl implements IAsArticleService
{
    @Resource
    private AsArticleMapper asArticleMapper;

    @Resource
    private AsArticleClassifyMapper articleClassifyMapper;

    @Resource
    private EtOperatingAreaMapper etOperatingAreaMapper;

    @Autowired
    private IWxPayService wxPayService;

    /**
     * 查询文章
     *
     * @param articleId 文章主键
     * @return 文章
     */
    @Override
    public EtArticle selectAsArticleByArticleId(Long articleId)
    {
        EtArticle etArticle = asArticleMapper.selectAsArticleByArticleId(articleId);
        if(ObjectUtils.isNotEmpty(etArticle)){
            etArticle.setFormatCreateTime(DateUtils.getYYYY_MM_DD(etArticle.getCreateTime()));
        }
        return etArticle;
    }

    /**
     * 查询文章列表
     *
     * @param etArticle 文章
     * @return 文章
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<EtArticle> selectAsArticleListByIsolate(EtArticle etArticle)
    {
        List<EtArticle> etArticles = asArticleMapper.selectAsArticleList(etArticle);
        for (EtArticle etArticle1 : etArticles) {
            EtOperatingArea area = etOperatingAreaMapper.selectEtOperatingAreaByAreaId(etArticle1.getAreaId());
            SysDept sysDept = wxPayService.getDeptObjByAreaId(etArticle1.getAreaId());
            if(ObjectUtil.isNotNull(sysDept)){
                etArticle1.setDeptName(sysDept.getDeptName());
            }
            if(ObjectUtil.isNotNull(area)){
                etArticle1.setAreaName(area.getAreaName());
            }
            etArticle1.setFormatCreateTime(DateUtils.getYYYY_MM_DD(etArticle1.getCreateTime()));
        }
        return etArticles;
    }

    /**
     * 查询文章列表
     *
     * @param etArticle 文章 隔离
     * @return 文章
     */
    @Override
    public List<EtArticle> selectAsArticleList(EtArticle etArticle)
    {
        List<EtArticle> etArticles = asArticleMapper.selectAsArticleList(etArticle);
        for (EtArticle etArticle1 : etArticles) {
            etArticle.setFormatCreateTime(DateUtils.getYYYY_MM_DD(etArticle1.getCreateTime()));
        }
        return etArticles;
    }

    /**
     * 查询文章列表
     *
     * @param etArticle 文章 隔离
     * @return 文章
     */
    @Override
    public List<EtArticle> selectAsArticleListByApp(EtArticle etArticle)
    {
        List<EtArticle> etArticles = asArticleMapper.selectAsArticleListByApp(etArticle);
        List<EtArticle> mergedArticles = new ArrayList<>();
        Set<String> seenArticleIds = new HashSet<>();

        for (EtArticle etArticle1 : etArticles) {
            etArticle1.setFormatCreateTime(DateUtils.getYYYY_MM_DD(etArticle1.getCreateTime()));
            if (seenArticleIds.add(etArticle1.getArticleId().toString())) {
                mergedArticles.add(etArticle1);
            }
        }

        for (EtArticle etArticle1 : etArticles) {
            if (etArticle1.getClassifyId().equals("111")) {
                List<AsArticleClassify> classifies = articleClassifyMapper.selectChildrenClassifyById(Long.parseLong(etArticle1.getClassifyId()));
                for (AsArticleClassify classify : classifies) {
                    EtArticle etArticle2 = new EtArticle();
                    etArticle2.setClassifyId(classify.getClassifyId().toString());
                    List<EtArticle> etArticles1 = asArticleMapper.selectAsArticleList(etArticle2);
                    for (EtArticle etArticle2Item : etArticles1) {
                        if (seenArticleIds.add(etArticle2Item.getArticleId().toString())) {
                            mergedArticles.add(etArticle2Item);
                        }
                    }
                }
            }
        }
        etArticles = mergedArticles;
        return etArticles;
    }

    /**
     * 新增文章
     *
     * @param etArticle 文章
     * @return 结果
     */
    @Override
    public int insertAsArticle(EtArticle etArticle)
    {
        etArticle.setCreateTime(DateUtils.getNowDate());
        return asArticleMapper.insertAsArticle(etArticle);
    }

    /**
     * 修改文章
     *
     * @param etArticle 文章
     * @return 结果
     */
    @Override
    public int updateAsArticle(EtArticle etArticle)
    {
        etArticle.setUpdateTime(DateUtils.getNowDate());
        return asArticleMapper.updateAsArticle(etArticle);
    }

    /**
     * 批量删除文章
     *
     * @param articleIds 需要删除的文章主键
     * @return 结果
     */
    @Override
    public int deleteAsArticleByArticleIds(Long[] articleIds)
    {
        return asArticleMapper.deleteAsArticleByArticleIds(articleIds);
    }

    /**
     * 删除文章信息
     *
     * @param articleId 文章主键
     * @return 结果
     */
    @Override
    public int deleteAsArticleByArticleId(Long articleId)
    {
        return asArticleMapper.deleteAsArticleByArticleId(articleId);
    }

    /**
     * 根据文章id获取标签列表
     *
     * @param articleId 文章主键
     * @return 结果
     */
    @Override
    public String[] getTagList(Long articleId) {
        EtArticle etArticle = asArticleMapper.selectAsArticleByArticleId(articleId);
        String tag = etArticle.getTag();
        String[] split = tag.split(",");
        return split;
    }

//    /**
//     * 查询动态分类及数量
//     *
//     * @param
//     * @return 结果
//     */
//    @Override
//    public List<DynamicClassify> selectDynamicClassifyList() {
//        List<DynamicClassify> classifies = new ArrayList<>();
//        SysDictData dictData = new SysDictData();
//        dictData.setDictType("dynamic_classify");
//        List<SysDictData> sysDictData = dictDataMapper.selectDictDataList(dictData);
//        for (SysDictData one :sysDictData) {
//            DynamicClassify dynamicClassify = new DynamicClassify();
//            String dictValue = one.getDictValue();
//            dynamicClassify.setName(one.getDictLabel());
//            dynamicClassify.setValue(dictValue);
//            int classify = asArticleMapper.countDynamicClassify(dictValue);
//            dynamicClassify.setNum(classify);
//            classifies.add(dynamicClassify);
//        }
//        return classifies;
//    }
}
