package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.constant.*;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.CommonUtil;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.http.HttpUtils;
import com.ruoyi.common.utils.map.GeoUtils;
import com.ruoyi.common.utils.onenet.*;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.system.domain.*;
import com.ruoyi.system.domain.response.OrderResponse;
import com.ruoyi.system.domain.vo.*;
import com.ruoyi.system.mapper.*;
import com.ruoyi.system.service.*;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.jetbrains.annotations.NotNull;
import org.locationtech.jts.geom.Geometry;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionTemplate;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import static com.ruoyi.common.utils.SecurityUtils.getUsername;

/**
 * 设备Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-04-05
 */
@Slf4j
@Service
public class AsDeviceServiceImpl extends ServiceImpl<AsDeviceMapper, AsDevice> implements IAsDeviceService
{
    @Resource
    private AsDeviceMapper asDeviceMapper;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Resource
    private IEtOperatingAreaService etOperatingAreaService;

    @Autowired
    private IEtOrderService etOrderService;

    @Autowired
    private IEtFeeRuleService etFeeRuleService;

    @Autowired
    private IEtParkingAreaService parkingAreaService;

    @Resource
    private ISysDictDataService sysDictDataService;

    @Resource
    private IEtTripLogService tripLogService;

    @Resource
    private IEtModelService etModelService;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private IWxPayService wxPayService;

    @Autowired
    private ISysUserService userService;

    @Resource
    private EtOrderMapper etOrderMapper;

    @Resource
    private EtCommandLogMapper etCommandLogMapper;

    @Autowired
    private IEtOnlineLogService etOnlineLogService;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private ISysConfigService sysConfigService;

    @Resource
    private EtLocationLogMapper etLocationLogMapper;

    @Resource
    private EtCouponMapper etCouponMapper;

    @Resource
    private EtCouponClaimLogMapper etCouponClaimLogMapper;

    @Resource
    private AsUserMapper asUserMapper;


    @Value(value = "${iot.iotUrl}")
    private String iotUrl;

    @Value(value = "${iot.productId}")
    private String productId;

    @Value(value = "${iot.timeout}")
    private String timeout;

    @Value(value = "${iot.deviceUrl}")
    private String deviceUrl;

    /**
     * 查询设备
     *
     * @param deviceId 设备主键
     * @return 设备
     */
    @Override
    public AsDevice selectAsDeviceByDeviceId(Long deviceId)
    {
        AsDevice device = asDeviceMapper.selectAsDeviceByDeviceId(deviceId);
        String status = device.getStatus();
        if(ObjectUtil.isNotNull(status)){
            String typeName = sysDictDataService.selectDictLabel("as_device_status", status);
            device.setStatusStr(typeName);
            if(status.equals(ServiceConstants.VEHICLE_STATUS_NOT_BAND) && ObjectUtil.isNotNull(device.getAreaId())){
                device.setStatus(ServiceConstants.VEHICLE_STATUS_NOT_LISTING);
                int i = asDeviceMapper.updateAsDevice(device);
            }
        }
        Long areaId = device.getAreaId();
        if (ObjectUtil.isNotNull(areaId) && areaId!=0){
            EtOperatingArea etOperatingArea = etOperatingAreaService.selectEtOperatingAreaByAreaId(areaId);
            device.setAreaName(etOperatingArea.getAreaName());
        }
        if(ObjectUtil.isNotNull(areaId) && areaId!=0){
            SysDept sysDept = wxPayService.getDeptObjByAreaId(areaId);
            //https://dianche.chuantewulian.cn?sn=https://dche.ccttiot.com?sn=3000900
            device.setQrText(sysDept.getDomain()+"?sn="+device.getSn());
            device.setDeptName(sysDept.getDeptName());
        }else{
            device.setQrText("");
        }
        return device;
    }

    /**
     * 根据mac查询车辆实时信息
     *
     * @param mac 设备mac
     * @return 设备
     */
    @Override
    public AsDevice selectAsDeviceByMac(String mac) {
        return asDeviceMapper.selectAsDeviceByMac(mac);
    }

    /**
     * 根据sn号查询车辆实时信息  真实金额  实际
     *
     * @param sn 设备sn
     * @return 设备
     */
    @SneakyThrows
    @Override
    public AsDevice selectAsDeviceBySn(String sn) {
        AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(sn);
        if (ObjectUtil.isNull(asDevice)) {
            throw new ServiceException("设备不存在："+ sn);
        }
        Long areaId = asDevice.getAreaId();
        EtOperatingArea etOperatingArea;
        if (ObjectUtil.isNotNull(areaId) && areaId!=0) {
            etOperatingArea = etOperatingAreaService.selectEtOperatingAreaByAreaId(areaId);
            asDevice.setAreaName(etOperatingArea.getAreaName());
        }
        EtModel model = etModelService.selectEtModelByModelId(asDevice.getModelId());
        if(ObjectUtil.isNotNull(model)){
            asDevice.setModel(model.getModel());
        }
        if(StrUtil.isNotBlank(asDevice.getVoltage())){
            asDevice.setRemainingMileage(CommonUtil.getRemainingMileage(asDevice.getVoltage(),model.getFullVoltage(),model.getLowVoltage(),model.getFullEndurance()));
        }
        String msg = redisCache.getCacheObject(CacheConstants.CACHE_DEVICE_KEY + asDevice.getSn());
        if(StrUtil.isNotBlank(msg)){
            log.info("【根据sn号查询车辆实时信息】-========redis缓存中的数据：" + msg);
            LogEntry logEntry = JSONObject.parseObject(msg, LogEntry.class);
            log.info("【根据sn号查询车辆实时信息】============logEntry转换后的对象: logEntry---【{}】" , JSON.toJSONString(logEntry));
            LogEntry.LocationValue value = logEntry.getValue();
            // 坐标转换  WGS84 转 GCJ02
            double[] doubles = CommonUtil.coordinateConvert(value.getLon(),value.getLat());
            BigDecimal lon = new BigDecimal(doubles[1]).setScale(8, RoundingMode.HALF_UP);
            BigDecimal lat = new BigDecimal(doubles[0]).setScale(8, RoundingMode.HALF_UP);
            asDevice.setLongitude(lon.toString());
            asDevice.setLatitude(lat.toString());
        }
        //正在进行中的订单
        EtOrder order = new EtOrder();
        order.setSn(asDevice.getSn());
        List<EtOrder> etOrders = etOrderMapper.selectLatestOrderList(order);
        asDevice.setEtOrders(etOrders);
        return asDevice;
    }

    /**
     * 查询设备列表
     *
     * @param asDevice 设备
     * @return 设备
     */
    @Override
    public List<AsDevice> selectAsDeviceList(AsDevice asDevice)
    {
        List<AsDevice> asDevices = asDeviceMapper.selectAsDeviceList(asDevice);
        for (AsDevice asDevice1:asDevices){
            Long areaId = asDevice1.getAreaId();
            if (ObjectUtil.isNotNull(areaId) && areaId!=0){
                EtOperatingArea etOperatingArea = etOperatingAreaService.selectEtOperatingAreaByAreaId(areaId);
                asDevice1.setAreaName(etOperatingArea.getAreaName());
            }
            Long modelId = asDevice1.getModelId();
            if (ObjectUtil.isNotNull(modelId)){
                EtModel model  = etModelService.selectEtModelByModelId(modelId);
                if(ObjectUtil.isNotNull(model)){
                    asDevice1.setModel(model.getModel());
                }
            }
            String status = asDevice1.getStatus();
            if(ObjectUtil.isNotNull(status)){
                String typeName = sysDictDataService.selectDictLabel("as_device_status", status);
                asDevice1.setStatusStr(typeName);
            }
        }
        return asDevices;
    }

    /**
     * 查询设备列表
     *
     * @param asDevice 设备 隔离
     * @return 设备
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<AsDevice> selectAsDeviceListWithIsolate(AsDevice asDevice)
    {
        if(ServiceConstants.VEHICLE_STATUS_NOT_BAND.equals(asDevice.getStatus())){
            asDevice.setUnBand("1");
            asDevice.setStatus(null);
        }
        List<AsDevice> asDevices = asDeviceMapper.selectAsDeviceListWithIsolate(asDevice);
        for (AsDevice asDevice1:asDevices){
            Long areaId = asDevice1.getAreaId();
            if (ObjectUtil.isNotNull(areaId) && areaId!=0){
                EtOperatingArea etOperatingArea = etOperatingAreaService.selectEtOperatingAreaByAreaId(areaId);
                asDevice1.setAreaName(etOperatingArea.getAreaName());
            }
            Long modelId = asDevice1.getModelId();
            if (ObjectUtil.isNotNull(modelId)){
                EtModel model  = etModelService.selectEtModelByModelId(modelId);
                if(ObjectUtil.isNotNull(model)){
                    asDevice1.setModel(model.getModel());
                }
            }
            String status = asDevice1.getStatus();
            if(ObjectUtil.isNotNull(status)){
                String typeName = sysDictDataService.selectDictLabel("as_device_status", status);
                asDevice1.setStatusStr(typeName);
                if(status.equals(ServiceConstants.VEHICLE_STATUS_NOT_BAND) && ObjectUtil.isNotNull(asDevice1.getAreaId())){
                    asDevice1.setStatus(ServiceConstants.VEHICLE_STATUS_NOT_LISTING);
                    int i = asDeviceMapper.updateAsDevice(asDevice1);
                }
            }
            if(ObjectUtil.isNull(asDevice1.getAreaId())){
                asDevice1.setStatus(ServiceConstants.VEHICLE_STATUS_NOT_BAND);
                int i = asDeviceMapper.updateAsDevice(asDevice1);
            }
            if(ObjectUtil.isNotNull(areaId) && areaId!=0){
                SysDept sysDept = wxPayService.getDeptObjByAreaId(areaId);
                //https://dianche.chuantewulian.cn?sn=https://dche.ccttiot.com?sn=3000900
                asDevice1.setQrText(sysDept.getDomain()+"?sn="+asDevice1.getSn());
                asDevice1.setDeptName(sysDept.getDeptName());
            }else{
                asDevice1.setQrText("");
            }
        }
        return asDevices;
    }

    /**
     * 新增设备
     *
     * @param asDevice 设备
     * @return 结果
     */
    @Override
    public int insertAsDevice(AsDevice asDevice)
    {
        if(ObjectUtil.isNotNull(asDevice.getAreaId())){
            SysDept sysDept = wxPayService.getDeptObjByAreaId(asDevice.getAreaId());
            if(ObjectUtil.isNotNull(sysDept)){
                asDevice.setAppid(sysDept.getAppid());
                asDevice.setAppName(sysDept.getAppName());
                if(sysDept.getIsUsePlatformApp().equals("true")){
                    asDevice.setDeptId(100L);
                }else{
                    asDevice.setDeptId(sysDept.getDeptId());
                }
            }
        }
        asDevice.setCreateTime(DateUtils.getNowDate());
        return asDeviceMapper.insertAsDevice(asDevice);
    }

    /**
     * 修改设备
     *
     * @param asDevice 设备
     * @return 结果
     */
    @Override
    public int updateAsDevice(AsDevice asDevice)
    {
        if(ObjectUtil.isNotNull(asDevice.getAreaId())){
            SysDept sysDept = wxPayService.getDeptObjByAreaId(asDevice.getAreaId());
            if(ObjectUtil.isNotNull(sysDept)){
                asDevice.setAppid(sysDept.getAppid());
                asDevice.setAppName(sysDept.getAppName());
                if(sysDept.getIsUsePlatformApp().equals("true")){
                    asDevice.setDeptId(100L);
                }else{
                    asDevice.setDeptId(sysDept.getDeptId());
                }
            }
        }
        asDevice.setUpdateTime(DateUtils.getNowDate());
        int i = asDeviceMapper.updateAsDevice(asDevice);
        //预约后开始计费

        return i;
    }

    /**
     * 修改设备
     *
     * @param asDevice 设备
     * @return 结果
     */
    @Override
    public int updateAsDevice2(AsDevice asDevice)
    {
        AsDevice device = asDeviceMapper.selectAsDeviceByDeviceId(asDevice.getDeviceId());//
        if(!device.getStatus().equals(asDevice.getStatus())){
            if(asDevice.getStatus().equals(ServiceConstants.VEHICLE_STATUS_NOT_LISTING)){
                Boolean inOrderBySn = etOrderService.isInOrderBySn(device.getSn());
                if(inOrderBySn){
                    throw new ServiceException("还有正在骑行中的订单【"+device.getSn()+"】，不能回仓");
                }else{
                    //记录日志
                    asynchronousSaveLog(null,null,asDevice.getMac(),null,"回仓",null,getUsername());
                }
            }
            if(asDevice.getStatus().equals(ServiceConstants.VEHICLE_STATUS_IN_OFFLINE)){
                Boolean inOrderBySn = etOrderService.isInOrderBySn(device.getSn());
                if(inOrderBySn){
                    throw new ServiceException("还有正在骑行中的订单【"+device.getSn()+"】，不能禁用");
                }else{
                    //记录日志
                    asynchronousSaveLog(null,null,asDevice.getMac(),null,"禁用",null,getUsername());
                }
            }
            if(device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_NOT_LISTING) && asDevice.getStatus().equals(ServiceConstants.VEHICLE_STATUS_NORMAL)){
                //记录日志
                asynchronousSaveLog(null,null,asDevice.getMac(),null,"出仓",null,getUsername());
            }
            if(device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_IN_OFFLINE) && asDevice.getStatus().equals(ServiceConstants.VEHICLE_STATUS_NORMAL)){
                //记录日志
                asynchronousSaveLog(null,null,asDevice.getMac(),null,"解禁",null,getUsername());
            }
        }
        if(ObjectUtil.isNotNull(asDevice.getAreaId())){
            SysDept sysDept = wxPayService.getDeptObjByAreaId(asDevice.getAreaId());
            if(ObjectUtil.isNotNull(sysDept)){
                asDevice.setAppid(sysDept.getAppid());
                asDevice.setAppName(sysDept.getAppName());
                if(sysDept.getIsUsePlatformApp().equals("true")){
                    asDevice.setDeptId(100L);
                }else{
                    asDevice.setDeptId(sysDept.getDeptId());
                }
            }
        }
        asDevice.setUpdateTime(DateUtils.getNowDate());
        if(asDevice.getAreaId() == null){
            asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_NOT_BAND);
            asDevice.setOnlineStatus(ServiceConstants.VEHICLE_STATUS_OFFLINE);
            asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);
        }
        return asDeviceMapper.updateAsDevice2(asDevice);
    }

    /**
     * 根据SN修改设备
     *
     * @param asDevice 设备
     * @return 结果
     */
    @Override
    public int updateAsDeviceBySn(AsDevice asDevice)
    {
        asDevice.setUpdateTime(DateUtils.getNowDate());
        return asDeviceMapper.updateAsDeviceBySn(asDevice);
    }

    /**
     * 批量删除设备
     *
     * @param deviceIds 需要删除的设备主键
     * @return 结果
     */
    @Override
    public int deleteAsDeviceByDeviceIds(Long[] deviceIds)
    {
        return asDeviceMapper.deleteAsDeviceByDeviceIds(deviceIds);
    }

    /**
     * 一键解禁
     *
     * @param deviceIds 需要一键解禁的设备主键集合
     * @return 结果
     */
    @Override
    @Transactional
    public int oneClickOnline(Long[] deviceIds)
    {
        for (Long deviceId:deviceIds) {
            AsDevice device = asDeviceMapper.selectAsDeviceByDeviceId(deviceId);
            if(ObjectUtil.isNull(device)){
                throw new ServiceException("车辆【"+deviceId+"】不存在");
            }
            if(!device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_IN_OFFLINE)){
                throw new ServiceException("车辆【"+device.getSn()+"】非解禁状态，请重新选择！");
            }
            //记录日志
            asynchronousSaveLog(null,null,device.getMac(),null,"解禁",null,getUsername());
        }
        return asDeviceMapper.oneClickOnline(deviceIds);
    }

    /**
     * 一键出仓
     *
     * @param deviceIds 需要一键出仓的设备主键集合
     * @return 结果
     */
    @Override
    @Transactional
    public int oneClickListing(Long[] deviceIds)
    {
        for (Long deviceId:deviceIds) {
            AsDevice device = asDeviceMapper.selectAsDeviceByDeviceId(deviceId);
            if(ObjectUtil.isNull(device)){
                throw new ServiceException("车辆【"+deviceId+"】不存在");
            }
            if(!device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_NOT_LISTING)){
                throw new ServiceException("车辆【"+device.getSn()+"】非仓库中状态，请重新选择！");
            }
            //记录日志
            asynchronousSaveLog(null,null,device.getMac(),null,"出仓",null,getUsername());
        }
        return asDeviceMapper.oneClickOnline(deviceIds);
    }

    /**
     * 一键禁用
     *
     * @param deviceIds 需要一键解禁的设备主键集合
     * @return 结果
     */
    @Transactional
    @Override
    public int oneClickOffline(Long[] deviceIds)
    {
        for (Long deviceId:deviceIds) {
            AsDevice device = asDeviceMapper.selectAsDeviceByDeviceId(deviceId);
            Boolean inOrderBySn = etOrderService.isInOrderBySn(device.getSn());
            if(inOrderBySn){
                throw new ServiceException("还有正在骑行中的订单【"+device.getSn()+"】，不能禁用");
            }
            if(ObjectUtil.isNull(device)){
                throw new ServiceException("车辆【"+deviceId+"】不存在");
            }
            if(device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_IN_APPOINTMENT)){
                throw new ServiceException("车辆【"+device.getSn()+"】为‘预约中’状态不能禁用");
            }
            if(device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_IN_USING)){
                throw new ServiceException("车辆【"+device.getSn()+"】为‘使用中’状态不能禁用");
            }
            if(device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_TEMPORARILY_LOCK)){
                throw new ServiceException("车辆【"+device.getSn()+"】为‘临时停车’状态不能禁用");
            }
            if(device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_NOT_LISTING)){
                throw new ServiceException("车辆【"+device.getSn()+"】为‘未上架’状态不能禁用");
            }
            if(device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_ABANDON)){
                throw new ServiceException("车辆【"+device.getSn()+"】为‘废弃’状态不能禁用");
            }
            //记录日志
            asynchronousSaveLog(null,null,device.getMac(),null,"禁用",null,getUsername());
        }
        return asDeviceMapper.oneClickOffline(deviceIds);
    }

    /**
     * 一键入仓
     *
     * @param deviceIds 需要一键入仓的设备主键集合
     * @return 结果
     */
    @Override
    @Transactional
    public int oneClickWarehousing(Long[] deviceIds)
    {
        for (Long deviceId:deviceIds) {
            AsDevice device = asDeviceMapper.selectAsDeviceByDeviceId(deviceId);
            Boolean inOrderBySn = etOrderService.isInOrderBySn(device.getSn());//有进行中的订单，跳过
            if(inOrderBySn){
                throw new ServiceException("车辆【"+device.getSn()+"】有进行中的订单，不能入仓");
            }
            if(ObjectUtil.isNull(device)){
                throw new ServiceException("车辆【"+deviceId+"】不存在");
            }
            if(device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_IN_APPOINTMENT)){
                throw new ServiceException("车辆【"+device.getSn()+"】为‘预约中’状态不能入仓");
            }
            if(device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_IN_USING)){
                throw new ServiceException("车辆【"+device.getSn()+"】为‘使用中’状态不能入仓");
            }
            if(device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_TEMPORARILY_LOCK)){
                throw new ServiceException("车辆【"+device.getSn()+"】为‘临时停车’状态不能入仓");
            }
            if(device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_NOT_LISTING)){
                throw new ServiceException("车辆【"+device.getSn()+"】为‘仓库中’状态不能入仓");
            }
            if(device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_ABANDON)){
                throw new ServiceException("车辆【"+device.getSn()+"】为‘废弃’状态不能入仓");
            }
            //记录日志
            asynchronousSaveLog(null,null,device.getMac(),null,"入仓",null,getUsername());
        }
        return asDeviceMapper.oneClickWarehousing(deviceIds);
    }

    /**
     * 更新设备
     * 1. 发送更新版本
     * 2. 判断是否在线
     *
     * @param deviceIds 需要更新设备的设备主键集合
     * @return 结果
     */
    @Override
    public int refreshDevice(Long[] deviceIds)
    {
        for (Long deviceId:deviceIds) {
            AsDevice device = asDeviceMapper.selectAsDeviceByDeviceId(deviceId);
            //设备是否在线
            if(isOnline(device.getSn())){
                log.info("设备【"+device.getSn()+"】在线");
            }else{
                log.info("设备【"+device.getSn()+"】不在线");
            }
            if(updateVersion(device.getSn())){
                log.info("设备【"+device.getSn()+"】更新版本成功");
            }else{
                log.info("设备【"+device.getSn()+"】更新版本失败");
            }
            // 最新位置信息
            if(updateLatestLocation(device.getSn())){
                log.info("设备【"+device.getSn()+"】更新位置成功");
            }else{
                log.info("设备【"+device.getSn()+"】更新位置失败");
            }
        }
        return 1;
    }

    /**
     * 根据sn更新设备
     * @param sn sn
     * @return 结果
     */
    @Override
    public int refreshDeviceBySn(String sn)
    {
        AsDevice device = asDeviceMapper.selectAsDeviceBySn(sn);
        //设备是否在线
        if(isOnline(device.getSn())){
            log.info("设备【"+device.getSn()+"】在线");
        }else{
            log.info("设备【"+device.getSn()+"】不在线");
        }
        if(updateVersion(device.getSn())){
            log.info("设备【"+device.getSn()+"】更新版本成功");
        }else{
            log.info("设备【"+device.getSn()+"】更新版本失败");
        }
        // 最新位置信息
        if(updateLatestLocation(device.getSn())){
            log.info("设备【"+device.getSn()+"】更新位置成功");
        }else{
            log.info("设备【"+device.getSn()+"】更新位置失败");
        }
        return 1;
    }


    /**
     * 删除设备信息
     *
     * @param deviceId 设备主键
     * @return 结果
     */
    @Override
    public int deleteAsDeviceByDeviceId(Long deviceId)
    {
        return asDeviceMapper.deleteAsDeviceByDeviceId(deviceId);
    }

    /**
     * 根据经纬度查询附近500米的所有车辆
     * @param longitude 经度
     * @param latitude 纬度
     * @return 结果
     */
    @Override
    public List<AsDevice> vehicleLocalization(String longitude, String latitude,String deptId) {
        QueryWrapper<AsDevice> wrapper = new QueryWrapper<>();
        wrapper.eq("status", "1"); // 设备状态正常
        if(StrUtil.isBlank(deptId)){
            deptId  ="100";
        }
        wrapper.eq("dept_id", deptId);

        // 查询所有设备
        List<AsDevice> allDevices = asDeviceMapper.selectList(wrapper);

        List<AsDevice> nearbyDevices = new ArrayList<>();
        double targetLon = Double.parseDouble(longitude);
        double targetLat = Double.parseDouble(latitude);

        int nearbyVehicle = 500;// 默认距离小于等于 500 米
        String nearbyVehicles = sysConfigService.selectConfigByKey("nearby.vehicles");
        if(StrUtil.isNotBlank(nearbyVehicles)){
            nearbyVehicle = Integer.parseInt(nearbyVehicles);
        }
        for (AsDevice device : allDevices) {
            if(StrUtil.isNotBlank(device.getLongitude()) && StrUtil.isNotBlank(device.getLatitude())){
                double deviceLon = Double.parseDouble(device.getLongitude());
                double deviceLat = Double.parseDouble(device.getLatitude());
                double distance = GeoUtils.calculateDistance(targetLat, targetLon, deviceLat, deviceLon);

                if (distance <= nearbyVehicle) {
                    nearbyDevices.add(device);
                }
            }
        }
        return nearbyDevices;
    }

    /**
     * 所有车辆定位
     */
    @Override
    public List<AsDevice> allVehicleInfo(String powerStart, String powerEnd, String status,String sort,String areaId,String onlineStatus) {
        QueryWrapper<AsDevice> wrapper = new QueryWrapper<>();
        if(StrUtil.isNotBlank(areaId)){
            wrapper.eq("area_id",areaId);
        }
        if(StrUtil.isNotBlank(powerStart) && StrUtil.isNotBlank(powerEnd)){
            wrapper.between("remaining_power", Integer.parseInt(powerStart), Integer.parseInt(powerEnd));
        }
        if(StrUtil.isNotBlank(status)){
            if(StrUtil.equals(status,"-0")){
                wrapper.ne("status","0");
            }else{
                wrapper.in("status",status);
            }
        }
        if(StrUtil.isNotBlank(onlineStatus)){
            wrapper.in("online_status",onlineStatus);
        }
        if(StrUtil.isNotBlank(sort)){
            wrapper.orderBy(true,!StrUtil.equals(sort, "desc"),"remaining_power");
        }
        List<AsDevice> asDevices = asDeviceMapper.selectList(wrapper);
        for (AsDevice asDevice:asDevices){
            Long modelId = asDevice.getModelId();
            if (ObjectUtil.isNotNull(modelId)){
                EtModel model = etModelService.selectEtModelByModelId(modelId);
                if(ObjectUtil.isNotNull(model)){
                    asDevice.setModel(model.getModel());
                    Integer remainingMileage = 0;
                    if(StrUtil.isNotBlank(asDevice.getVoltage())){
                        remainingMileage = CommonUtil.getRemainingMileage(asDevice.getVoltage(), model.getFullVoltage(), model.getLowVoltage(), model.getFullEndurance());
                    }
                    Integer electricQuantity = CommonUtil.getElectricQuantity(asDevice.getVoltage(), model.getFullVoltage(), model.getLowVoltage());//电量百分百
                    asDevice.setRemainingMileage(remainingMileage);
                    asDevice.setRemainingPower(electricQuantity.toString());
                }
            }
        }
        return asDevices;
    }

    /**
     * 查询车辆数量
     */
    @Override
    public DeviceNumVo allVehicleNum(String powerStart, String powerEnd,String areaId) {
        DeviceNumVo deviceNumVo = new DeviceNumVo();
        QueryWrapper<AsDevice> wrapper = new QueryWrapper<>();
        if(StrUtil.isNotBlank(areaId)){
            wrapper.eq("area_id",areaId);
        }
        if(StrUtil.isNotBlank(powerStart) && StrUtil.isNotBlank(powerEnd)){
            wrapper.between("remaining_power", Integer.parseInt(powerStart),Integer.parseInt(powerEnd));
        }
        Integer allNum = asDeviceMapper.selectCount(wrapper);
        deviceNumVo.setAllNum(allNum);//所有车辆

        Integer ridingNum = setNum(powerStart, powerEnd,ServiceConstants.VEHICLE_STATUS_IN_USING,areaId);
        deviceNumVo.setRidingNum(ridingNum);//骑行中

        Integer inOperation = setInOperationNum(powerStart, powerEnd,ServiceConstants.VEHICLE_STATUS_NOT_LISTING,areaId);
        deviceNumVo.setInOperation(inOperation);//投放中

        Integer temporarilyLockNum = setNum(powerStart, powerEnd,ServiceConstants.VEHICLE_STATUS_TEMPORARILY_LOCK,areaId);
        deviceNumVo.setTemporarilyLockNum(temporarilyLockNum);//临时锁车

        Integer disabledNum = setNum(powerStart, powerEnd,ServiceConstants.VEHICLE_STATUS_IN_OFFLINE,areaId);
        deviceNumVo.setDisabledNum(disabledNum);//已禁用

        Integer normalNum = setNum(powerStart, powerEnd,ServiceConstants.VEHICLE_STATUS_NORMAL,areaId);
        deviceNumVo.setNormalNum(normalNum);//正常待租

        Integer inStashNum = setNum(powerStart, powerEnd,ServiceConstants.VEHICLE_STATUS_NOT_LISTING,areaId);
        deviceNumVo.setInStashNum(inStashNum);

        QueryWrapper<AsDevice> wrapperForOffline = new QueryWrapper<>();
        wrapperForOffline.eq("online_status","0");//在线状态：0-不在线；1-在线
        wrapperForOffline.ne("status","0");//
        if(StrUtil.isNotBlank(powerStart) && StrUtil.isNotBlank(powerEnd)){
            wrapperForOffline.between("remaining_power",Integer.parseInt(powerStart),Integer.parseInt(powerEnd));
        }
        if(StrUtil.isNotBlank(areaId)){
            wrapperForOffline.eq("area_id",areaId);
        }
        Integer offlineNum = asDeviceMapper.selectCount(wrapperForOffline);
        deviceNumVo.setOfflineNum(offlineNum);//已离线

        Integer inAppointmentNum = setNum(powerStart, powerEnd, ServiceConstants.VEHICLE_STATUS_IN_APPOINTMENT,areaId);
        deviceNumVo.setInAppointmentNum(inAppointmentNum);//预约中

        QueryWrapper<AsDevice> wrapperForDispatch = new QueryWrapper<>();
        wrapperForDispatch.eq("status","6");
        if(StrUtil.isNotBlank(powerStart) && StrUtil.isNotBlank(powerEnd)){
            wrapperForDispatch.between("remaining_power", Integer.parseInt(powerStart),Integer.parseInt(powerEnd));
        }
        if(StrUtil.isNotBlank(areaId)){
            wrapperForDispatch.eq("area_id",areaId);
        }
        Integer dispatchNum = asDeviceMapper.selectCount(wrapperForDispatch);
        deviceNumVo.setDispatchNum(dispatchNum);//调度设备
        return deviceNumVo;
    }

    private Integer setNum(String powerStart, String powerEnd,String status,String areaId) {
        QueryWrapper<AsDevice> wrapper = new QueryWrapper<>();
        wrapper.eq("status",status);
        if(StrUtil.isNotBlank(powerStart) && StrUtil.isNotBlank(powerEnd)){
            wrapper.between("remaining_power", Integer.parseInt(powerStart),Integer.parseInt(powerEnd));
        }
        if(StrUtil.isNotBlank(areaId)){
            wrapper.eq("area_id",areaId);
        }
        return asDeviceMapper.selectCount(wrapper);
    }

    private Integer setInOperationNum(String powerStart, String powerEnd,String status,String areaId) {
        QueryWrapper<AsDevice> wrapper = new QueryWrapper<>();
        wrapper.ne("status",status);
        if(StrUtil.isNotBlank(powerStart) && StrUtil.isNotBlank(powerEnd)){
            wrapper.between("remaining_power", Integer.parseInt(powerStart),Integer.parseInt(powerEnd));
        }
        if(StrUtil.isNotBlank(areaId)){
            wrapper.eq("area_id",areaId);
        }
        return asDeviceMapper.selectCount(wrapper);
    }


    /**
     * 编号开锁
     * @param orderVo 订单对象
     *      1.远程开锁
     *      2.更新车辆状态
     *      3.创建订单
     * @return
     */
    @SneakyThrows
    @Override
    @Transactional
    public OrderResponse snSwitch(EtOrderVo orderVo) {
        OrderResponse response = new OrderResponse();
        String orderNo = "";
        if(ObjectUtil.isNull(orderVo.getOrderNo())){
            orderNo = IdUtils.getOrderNo("qx");
        }else{
            orderNo = orderVo.getOrderNo();
        }
        String sessionId = IdUtils.randomUUIDByDigit(8);
        AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(orderVo.getSn());
        /** 1.获取token*/
        String token = Token.getToken();
        String finalOrderNo = orderNo;
        if(!"true".equals(orderVo.getIsBluetooth())){
            /** 2.发送命令*/
            ResponseVo responseVo = sendCommandWithResp(asDevice.getMac(), token, IotConstants.COMMAND_OPEN+IotConstants.COMMAND_FREQUENCY_5, "编号开锁", finalOrderNo);
            if(responseVo.getCode() != 0){
                throw new ServiceException("【扫码/编号开锁骑行】发送开锁命令失败");
            }
        }else{
            asDevice.setLongitude(orderVo.getLon());
            asDevice.setLatitude(orderVo.getLat());
            asDevice.setLastTime(DateUtils.getNowDate());
        }
        /** 3.更新车辆状态*/
        asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_OPEN);
        asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_IN_USING);
        asDevice.setIsAdminUnlocking("0");
        int device = asDeviceMapper.updateAsDevice(asDevice);
        if(device==0){
            log.info("【扫码/编号开锁骑行】更新车辆状态失败");
            throw new ServiceException("【扫码/编号开锁骑行】更新车辆状态失败");
        }
        /** 4.如果有预约订单则更新订单的预约结束时间，如果没有预约，则创建订单*/
        EtOrder order = etOrderService.selectEtOrderByOrderNo(finalOrderNo);
        if(ObjectUtil.isNotNull(order)){//有订单号：可能是套餐也可能是有预约
            if(ObjectUtil.isNotNull(order.getRuleId())){//套餐
                log.info("【扫码/编号开锁骑行】---预约扫码骑行");
                order.setSn(asDevice.getSn());
            }else{
                throw new ServiceException("ruleId未传");
            }
            order.setStatus(ServiceConstants.ORDER_STATUS_RIDING);
            order.setUnlockTime(DateUtils.getNowDate());
            order.setAppointmentEndTime(DateUtils.getNowDate());
            //计算预约费
            calculateAppointmentFee(order);
            order.setTotalFee(order.getAppointmentFee());
            order.setPayFee(order.getAppointmentFee());
            log.info("【扫码/编号开锁骑行--】,更新订单信息，{}",JSON.toJSON(order));
            int update = etOrderService.updateEtOrder(order);
            if(update==0){
                log.info("【扫码/编号开锁骑行】更新订单失败");
            }
        }else{
            log.info("【扫码/编号开锁骑行】---无订单号,正常开锁骑行订单");
            order = etOrderService.createOrder(orderVo, finalOrderNo);
            int etOrder = etOrderService.insertEtOrder(order);
            if(etOrder==0){
                log.info("【扫码/编号开锁骑行】保存订单失败");
                throw new ServiceException("【扫码/编号开锁骑行】保存订单失败");
            }
        }
        /** 5.记录行程*/
        int tripLog = tripLogService.tripLog(order.getOrderNo(),order.getSn(),ServiceConstants.TRIP_LOG_TYPE_UNLOCK_RIDE);
        if(tripLog==0){
            log.info("【扫码/编号开锁骑行】记录行程失败");
            throw new ServiceException("【扫码/编号开锁骑行】记录行程失败");
        }
        log.info("【扫码/编号开锁骑行】车辆开锁成功");
        response.setOrderNo(orderNo);
        response.setSessionId(sessionId);
        return response;
    }

    /**
     * 管理员开锁
     */
    @SneakyThrows
    @Override
    public Boolean unlocking(String sn,String userName) {
        AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(sn);
        /** 1.获取token*/
        String token = Token.getToken();
        Boolean execute = transactionTemplate.execute(e -> {
            /** 2.发送命令*/
            sendCommand(asDevice.getMac(), token,IotConstants.COMMAND_OPEN+IotConstants.COMMAND_FREQUENCY_20,"管理员开锁",null,userName);
            asDevice.setIsAdminUnlocking("1");
            if(!asDevice.getStatus().equals(ServiceConstants.VEHICLE_STATUS_NOT_LISTING)){
                asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_SCHEDULING);
            }
            asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_OPEN);
            int i = asDeviceMapper.updateAsDevice(asDevice);
            if(i>0){
                log.info("管理员开锁，更新设备状态成功");
            }
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("管理员开锁失败");
        return Boolean.TRUE;
    }

    /**
     * 管理员开锁
     */
    @SneakyThrows
    @Override
    public Boolean unlockingByMac(String mac,String userName) {
        AsDevice asDevice = asDeviceMapper.selectAsDeviceByMac(mac);
        /** 1.获取token*/
        String token = Token.getToken();
        Boolean execute = transactionTemplate.execute(e -> {
            /** 2.发送命令*/
            sendCommand(mac, token,IotConstants.COMMAND_OPEN+IotConstants.COMMAND_FREQUENCY_20,"管理员开锁",null,userName);
            asDevice.setIsAdminUnlocking("1");
            if(!asDevice.getStatus().equals(ServiceConstants.VEHICLE_STATUS_NOT_LISTING)){
                asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_SCHEDULING);
            }
            asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_OPEN);
            int i = asDeviceMapper.updateAsDevice(asDevice);
            if(i>0){
                log.info("管理员开锁，更新设备状态成功");
            }
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("管理员开锁失败");
        return Boolean.TRUE;
    }

    /**
     * 车辆解禁
     */
    @SneakyThrows
    @Override
    public Boolean online(String sn) {
        Boolean inOrderBySn = etOrderService.isInOrderBySn(sn);//有进行中的订单，跳过
        if(!inOrderBySn){
            AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(sn);
            asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_NORMAL);//车辆解禁
            int device = asDeviceMapper.updateAsDevice(asDevice);
            if(device==0){
                log.info("车辆解禁状态失败");
                return Boolean.FALSE;
            }else{
                //记录日志
                asynchronousSaveLog(null,null,asDevice.getMac(),null,"解禁",null,getUsername());
            }
        }
        return Boolean.TRUE;
    }

    /**
     * 车辆禁用
     */
    @SneakyThrows
    @Override
    public Boolean offline(String sn,String status) {
        Boolean inOrderBySn = etOrderService.isInOrderBySn(sn);
        if(inOrderBySn){
            throw new ServiceException("还有正在骑行中的订单【"+sn+"】，不能禁用");
        }
        AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(sn);
        if(ServiceConstants.VEHICLE_STATUS_IN_APPOINTMENT.equals(asDevice.getStatus())){
            throw new ServiceException("车辆处于预约中，不能禁用");
        }
        if(ServiceConstants.VEHICLE_STATUS_IN_USING.equals(asDevice.getStatus())){
            throw new ServiceException("车辆使用中，不能禁用");
        }
        if(ServiceConstants.VEHICLE_STATUS_TEMPORARILY_LOCK.equals(asDevice.getStatus())){
            throw new ServiceException("车辆临时停车中，不能禁用");
        }
        asDevice.setStatus(status);
        int device = asDeviceMapper.updateAsDevice(asDevice);
        if(device==0){
            log.info("车辆禁用状态失败");
            return Boolean.FALSE;
        }else{
            //记录日志
            asynchronousSaveLog(null,null,asDevice.getMac(),null,"解禁",null,getUsername());
        }
        return Boolean.TRUE;
    }

    /**
     * 根据条件模糊查询车辆信息
     * type:1-sn,2-mac,3-车牌号
     */
    @Override
    public List<AsDevice> deviceSearch(String key, String type, String areaId) {
        QueryWrapper<AsDevice> queryWrapper = new QueryWrapper<>();
        if(StrUtil.isNotBlank(areaId)){
            queryWrapper.eq("area_id",areaId);
        }
        switch (type){
            case "1":
                queryWrapper.like("sn", key);
                return asDeviceMapper.selectList(queryWrapper);
            case "2":
                queryWrapper.like("mac", key);
                return asDeviceMapper.selectList(queryWrapper);
            case "3":
                queryWrapper.like("vehicle_num", key);
                return asDeviceMapper.selectList(queryWrapper);
            default:
                return null;
        }
    }

    /**
     * 运营中车辆
     */
    @Override
    public String getInOperationDevice(String timeStart, String timeEnd,String areaId) {
        if(ObjectUtil.isNotEmpty(timeStart)&&ObjectUtil.isNotEmpty(timeEnd)){
            QueryWrapper<AsDevice> wrapper = new QueryWrapper<AsDevice>().in("status", "1", "2", "3", "4","6");
            wrapper.eq("area_id", areaId);
            return asDeviceMapper.selectCount(wrapper)+"";
        }
        return null;
    }

    /**
     * 有订单车辆
     */
    @Override
    public String getInOrderDevice(String timeStart, String timeEnd,String areaId) {
        if(ObjectUtil.isNotEmpty(timeStart)&&ObjectUtil.isNotEmpty(timeEnd)){
            return asDeviceMapper.getInOrderDevice(timeStart,timeEnd,areaId);
        }
        return null;
    }

    /**
     * 无订单车辆
     */
    @Override
    public String getNoOrderDevice(String timeStart, String timeEnd,String areaId) {
        if(ObjectUtil.isNotEmpty(timeStart)&&ObjectUtil.isNotEmpty(timeEnd)){
            return asDeviceMapper.getNoOrderDevice(timeStart,timeEnd,areaId);
        }
        return null;
    }

    /**
     * 根据车辆型号查询车辆数量
     */
    @Override
    public Integer selectCountByModelId(Long modelId) {
        QueryWrapper<AsDevice> wrapper = new QueryWrapper<>();
        wrapper.eq("model_id", modelId);
        Integer integer = asDeviceMapper.selectCount(wrapper);
        return integer;
    }

    /**
     * 根据运营区查询车辆数量
     */
    @Override
    public Integer selectCountByAreaId(Long areaId) {
        QueryWrapper<AsDevice> wrapper = new QueryWrapper<>();
        wrapper.eq("area_id", areaId);
        Integer integer = asDeviceMapper.selectCount(wrapper);
        return integer;
    }

    @Transactional
    @Override
    /** 发送命令*/
    public void sendCommand(String mac, String token,String command,String type,String orderNo,String userName) {
        String param = "device_name=" + mac + "&product_id=" + productId +"&timeout=" + timeout;
        String sendUrl = iotUrl+ IotConstants.ADDS_COMMAND + "?"+param;
        String result = HttpUtils.sendPostWithToken(sendUrl, command, token);
        log.info("【"+type+"】===>IOT请求调用结果:【{}】",result);
        JSONObject paramsObj = JSON.parseObject(result);
        String code = paramsObj.getString("code");

        if ("10500".equals(code)) { // 超时
            log.info("第一次请求超时，进行第二次请求...");
            result = HttpUtils.sendPostWithToken(sendUrl, command, token);
            log.info("【" + type + "】===>IOT第二次请求调用结果:【{}】", result);
            paramsObj = JSON.parseObject(result);
            code = paramsObj.getString("code");
        }

        asynchronousSaveLog(sendUrl,command,mac,result,type,orderNo,userName);
        //记录命令
        if (!HttpStatus.IOT_SUCCESS.equals(code))
        {
            // 异步更新在线状态
            asynchronousUpdateOnlineStatus(mac);
            throw new ServiceException(code+"-----"+ IotUtil.formatMsg(code));
        }
    }

    @Transactional
    @Override
    /** 发送命令*/
    public void sendCommand(String mac, String token,String command,String type,String orderNo,String userName,String msg) {
        String param = "device_name=" + mac + "&product_id=" + productId +"&timeout=" + timeout;
        String sendUrl = iotUrl+ IotConstants.ADDS_COMMAND + "?"+param;
        String result = HttpUtils.sendPostWithToken(sendUrl, command, token);
        log.info("【"+type+"】===>IOT请求调用结果:【{}】",result);
        JSONObject paramsObj = JSON.parseObject(result);
        String code = paramsObj.getString("code");

        if ("10500".equals(code)) { // 超时
            log.info("第一次请求超时，进行第二次请求...");
            result = HttpUtils.sendPostWithToken(sendUrl, command, token);
            log.info("【" + type + "】===>IOT第二次请求调用结果:【{}】", result);
            paramsObj = JSON.parseObject(result);
            code = paramsObj.getString("code");
        }

        asynchronousSaveLog(sendUrl,command,mac,result,type,orderNo,userName,msg);
        //记录命令
        if (!HttpStatus.IOT_SUCCESS.equals(code))
        {
            // 异步更新在线状态
            asynchronousUpdateOnlineStatus(mac);
            throw new ServiceException(code+"-----"+ IotUtil.formatMsg(code));
        }
    }

    private void asynchronousUpdateOnlineStatus(String mac) {
        AsDevice device = asDeviceMapper.selectAsDeviceByMac(mac);
        //异步更新在线状态
        scheduledExecutorService.schedule(() -> {
            if(device.getOnlineStatus().equals(ServiceConstants.VEHICLE_STATUS_ONLINE)){
                log.info("【接收onenet推送】异步更新在线状态"+JSON.toJSONString(device));
                device.setOnlineStatus(ServiceConstants.VEHICLE_STATUS_OFFLINE);
                int i = asDeviceMapper.updateAsDevice(device);
                if(i>0){
                    log.info("【接收onenet推送】异步保存在线状态成功");
                }
            }
            EtOnlineLog etOnlineLog = new EtOnlineLog();
            etOnlineLog.setMac(device.getMac());
            etOnlineLog.setSn(device.getSn());
            etOnlineLog.setOnlineStatus(ServiceConstants.VEHICLE_STATUS_OFFLINE);
            etOnlineLog.setLongitude(device.getLongitude());
            etOnlineLog.setLatitude(device.getLatitude());
            etOnlineLog.setCreateTime(DateUtils.getNowDate());
            int i1 = etOnlineLogService.insertEtOnlineLog(etOnlineLog);
            if(i1>0){
                log.info("【接收onenet推送】异步保存在线日志成功");
            }
        }, 0, TimeUnit.SECONDS);
    }

    /* 异步保存发送命令日志*/
    private void asynchronousSaveLog(String url,String command,String mac,String result,String type,String orderNo,String userName) {
        //异步保存发送命令日志
        scheduledExecutorService.schedule(() -> {
            EtCommandLog etCommandLog = new EtCommandLog();
            etCommandLog.setUrl(url);
            etCommandLog.setCommand(command);
            etCommandLog.setType(type);
            etCommandLog.setMac(mac);
            AsDevice device = asDeviceMapper.selectAsDeviceByMac(mac);
            etCommandLog.setSn(device.getSn());
            etCommandLog.setResult(result);
            etCommandLog.setLongitude(device.getLongitude());
            etCommandLog.setLatitude(device.getLatitude());
            etCommandLog.setCreateTime(DateUtils.getNowDate());
            if(ObjectUtil.isNotNull(result)){
                JSONObject paramsObj = JSON.parseObject(result);
                String code = paramsObj.getString("code");
                etCommandLog.setCallStatus(code);
            }else{
                etCommandLog.setCallStatus("0");
            }
            etCommandLog.setOrderNo(orderNo);
            etCommandLog.setCreateBy(userName);
            int i = etCommandLogMapper.insertEtCommandLog(etCommandLog);
            if(i>0){
                log.info("【发送命令】异步保存发送命令日志");
            }
        }, 0, TimeUnit.SECONDS);
    }

    /* 异步保存发送命令日志*/
    private void asynchronousSaveLog(String url,String command,String mac,String result,String type,String orderNo,String userName,String msg) {
        //异步保存发送命令日志
        scheduledExecutorService.schedule(() -> {
            EtCommandLog etCommandLog = new EtCommandLog();
            etCommandLog.setUrl(url);
            etCommandLog.setCommand(command);
            etCommandLog.setType(type);
            etCommandLog.setMac(mac);
            AsDevice device = asDeviceMapper.selectAsDeviceByMac(mac);
            etCommandLog.setSn(device.getSn());
            etCommandLog.setResult(result);
            etCommandLog.setMsg(msg);
            etCommandLog.setLongitude(device.getLongitude());
            etCommandLog.setLatitude(device.getLatitude());
            etCommandLog.setCreateTime(DateUtils.getNowDate());
            JSONObject paramsObj = JSON.parseObject(result);
            String code = paramsObj.getString("code");
            etCommandLog.setCallStatus(code);
            etCommandLog.setOrderNo(orderNo);
            etCommandLog.setCreateBy(userName);
            int i = etCommandLogMapper.insertEtCommandLog(etCommandLog);
            if(i>0){
                log.info("【发送命令】异步保存发送命令日志");
            }
        }, 0, TimeUnit.SECONDS);
    }

    /* 异步保存发送命令日志*/
    private void asynchronousSaveLogWithLocation(String url,String command,String mac,String result,String type,String orderNo,String userName,String lon,String lat) {
        //异步保存发送命令日志
        scheduledExecutorService.schedule(() -> {
            EtCommandLog etCommandLog = new EtCommandLog();
            etCommandLog.setUrl(url);
            etCommandLog.setCommand(command);
            etCommandLog.setType(type);
            etCommandLog.setMac(mac);
            AsDevice device = asDeviceMapper.selectAsDeviceByMac(mac);
            etCommandLog.setSn(device.getSn());
            etCommandLog.setResult(result);
            etCommandLog.setLongitude(device.getLongitude());
            etCommandLog.setLatitude(device.getLatitude());
            etCommandLog.setLon(lon);
            etCommandLog.setLat(lat);
            etCommandLog.setCreateTime(DateUtils.getNowDate());
            JSONObject paramsObj = JSON.parseObject(result);
            String code = paramsObj.getString("code");
            etCommandLog.setCallStatus(code);
            etCommandLog.setOrderNo(orderNo);
            etCommandLog.setCreateBy(userName);
            int i = etCommandLogMapper.insertEtCommandLog(etCommandLog);
            if(i>0){
                log.info("【发送命令】异步保存发送命令日志");
            }
        }, 0, TimeUnit.SECONDS);
    }

    @Override
    /** 发送命令*/
    public ResponseVo sendCommandWithResp(String mac, String token,String command,String type,String orderNo) {
        String param = "device_name=" + mac + "&product_id=" + productId +"&timeout=" + timeout;
        String sendUrl = iotUrl+ IotConstants.ADDS_COMMAND + "?"+param;
        String result = HttpUtils.sendPostWithToken(sendUrl, command, token);
        log.info("【"+type+"】===>IOT请求调用结果:【{}】",result);
        ResponseVo responseVo = JSON.parseObject(result, ResponseVo.class);

        if (responseVo.getCode() == 10500) { // 超时
            log.info("第一次请求超时，进行第二次请求...");
            result = HttpUtils.sendPostWithToken(sendUrl, command, token);
            log.info("【" + type + "】===>IOT第二次请求调用结果:【{}】", result);
            responseVo = JSON.parseObject(result, ResponseVo.class);
        }

        if(responseVo.getCode() != 0){
            asynchronousUpdateOnlineStatus(mac);
        }
        asynchronousSaveLog(sendUrl,command,mac,result,type,orderNo,null);
        return JSON.parseObject(result,ResponseVo.class);
    }

    @Override
    /** 发送命令*/
    public ResponseVo sendCommandWithResp(String mac, String token,String command,String type,String orderNo,String lon,String lat) {
        String param = "device_name=" + mac + "&product_id=" + productId +"&timeout=" + timeout;
        String sendUrl = iotUrl+ IotConstants.ADDS_COMMAND + "?"+param;
        String result = HttpUtils.sendPostWithToken(sendUrl, command, token);
        log.info("【"+type+"】===>IOT请求调用结果:【{}】",result);
        ResponseVo responseVo = JSON.parseObject(result, ResponseVo.class);

        if (responseVo.getCode() == 10500) { // 超时
            log.info("第一次请求超时，进行第二次请求...");
            result = HttpUtils.sendPostWithToken(sendUrl, command, token);
            log.info("【" + type + "】===>IOT第二次请求调用结果:【{}】", result);
            responseVo = JSON.parseObject(result, ResponseVo.class);
        }

        if(responseVo.getCode() != 0){
            asynchronousUpdateOnlineStatus(mac);
        }
        asynchronousSaveLogWithLocation(sendUrl,command,mac,result,type,orderNo,null,lon,lat);
        return JSON.parseObject(result,ResponseVo.class);
    }

    /** 查询数据点*/
    @Override
    public DataPointRes historyDatapoints(String mac, String token) {
        String param = "device_name=" + mac + "&product_id=" + productId +"&timeout=" + timeout;
        String sendUrl = iotUrl+ IotConstants.ADDS_CURRENT_DATAPOINTS + "?"+param;
        String result = HttpUtils.sendGetWithToken(sendUrl, null, token);
        log.info("【查询数据点】===>IOT请求调用结果:【{}】",result);
        return JSON.parseObject(result,DataPointRes.class);
    }

    /**
     * 响铃寻车
     * @param sn
     * @return
     */
    @SneakyThrows
    @Override
    public Boolean ring(String sn) {
        AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(sn);
        /** 1.获取token*/
        String token = Token.getToken();
        Boolean execute = transactionTemplate.execute(e -> {
            /** 2.发送命令*/
            sendCommand(asDevice.getMac(), token,IotConstants.COMMAND_PLAY1,"响铃寻车",null,null);
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("响铃寻车失败");
        return Boolean.TRUE;
    }

    /**
     * 开坐垫锁
     * @param sn
     * @return
     */
    @SneakyThrows
    @Override
    public Boolean seatCushionLock(String sn) {
        AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(sn);
        /** 1.获取token*/
        String token = Token.getToken();
        Boolean execute = transactionTemplate.execute(e -> {
            /** 2.发送命令*/
            sendCommand(asDevice.getMac(), token,IotConstants.COMMAND_HPEN,"开坐垫锁",null,null);
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("开坐垫锁失败");
        return Boolean.TRUE;
    }

    /**
     * 重启设备
     * @param sn
     * @return
     */
    @SneakyThrows
    @Override
    public Boolean reboot(String sn) {
        AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(sn);
        /** 1.获取token*/
        String token = Token.getToken();
        Boolean execute = transactionTemplate.execute(e -> {
            /** 2.发送命令*/
            sendCommand(asDevice.getMac(), token,IotConstants.COMMAND_REBOOT,"重启设备",null,null);
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("重启设备失败");
        return Boolean.TRUE;
    }

    /**
     * 重启设备
     * @param mac
     * @return
     */
    @SneakyThrows
    @Override
    public Boolean rebootByMac(String mac) {
        /** 1.获取token*/
        String token = Token.getToken();
        Boolean execute = transactionTemplate.execute(e -> {
            /** 2.发送命令*/
            sendCommand(mac, token,IotConstants.COMMAND_REBOOT,"重启设备",null,null);
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("重启设备失败");
        return Boolean.TRUE;
    }

    /**
     * 响铃寻车
     * @param mac
     * @return
     */
    @SneakyThrows
    @Override
    public Boolean ringByMac(String mac) {
        /** 1.获取token*/
        String token = Token.getToken();
        Boolean execute = transactionTemplate.execute(e -> {
            /** 2.发送命令*/
            sendCommand(mac, token,IotConstants.COMMAND_PLAY1,"响铃寻车",null,null);
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("响铃寻车失败");
        return Boolean.TRUE;
    }


    /**
     * 坐垫锁用mac
     * @param mac
     * @return
     */
    @SneakyThrows
    @Override
    public Boolean seatCushionLockByMac(String mac) {
        /** 1.获取token*/
        String token = Token.getToken();
        Boolean execute = transactionTemplate.execute(e -> {
            /** 2.发送命令*/
            sendCommand(mac, token,IotConstants.COMMAND_HPEN,"开坐垫锁",null,null);
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("开坐垫锁失败");
        return Boolean.TRUE;
    }

    /**
     * 临时锁车
     * @param orderNo
     * @return
     */
    @Transactional
    @SneakyThrows
    @Override
    public Boolean lock(String sn,String orderNo,String isBluetooth,String lon,String lat) {
        EtOrder order = etOrderService.selectEtOrderByOrderNo(orderNo);
        if(StrUtil.isBlank(sn)) {
            sn = order.getSn();
        }
        AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(sn);
        /** 1.获取token*/
        String token = Token.getToken();
        if(!"true".equals(isBluetooth)){
            /** 2.发送命令*/
            sendCommand(asDevice.getMac(), token,IotConstants.COMMAND_LLOSE+IotConstants.COMMAND_FREQUENCY_3600,"临时锁车",orderNo,order.getUserName());
        }else{
            asDevice.setLongitude(lon);
            asDevice.setLatitude(lat);
            asDevice.setLastTime(DateUtils.getNowDate());
        }
        asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);
        if(StrUtil.isNotBlank(orderNo)){//有订单号，则是用户临时锁车
            /**  改变车辆状态：4-临时锁车 */
            asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_TEMPORARILY_LOCK);//临时锁车
            /** 5.记录行程*/
            int tripLog = tripLogService.tripLog(orderNo, sn,ServiceConstants.TRIP_LOG_TYPE_TEMPORARILY_LOCK);
            if(tripLog==0){
                log.info("【临时锁车】记录行程失败");
                throw new ServiceException("【临时锁车】记录行程失败");
            }
        }
        int device = asDeviceMapper.updateAsDevice(asDevice);
        if(device==0){
            log.info("【临时解锁】改变车辆状态失败");
            throw new ServiceException("【临时锁车】改变车辆状态失败");
        }
        return Boolean.TRUE;
    }

    /**
     * 管理员锁车
     * @param sn
     * @return
     */
    @Transactional
    @SneakyThrows
    @Override
    public Boolean adminLock(String sn,String userName) {
        if(StrUtil.isBlank(sn))throw new ServiceException("sn不能为空");
        AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(sn);
        /** 2.发送命令*/
        sendCommand(asDevice.getMac(), Token.getToken(),IotConstants.COMMAND_CLOSE+IotConstants.COMMAND_FREQUENCY_3600,"管理员锁车",null,userName);
        asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);
        // 判断该sn是否有正在骑行中的订单，如果有骑行中的订单则修改为临时锁车
        EtOrder etOrder = etOrderService.getCurrentOrder(sn);
        if(ObjectUtil.isNotNull(etOrder)){
            asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_TEMPORARILY_LOCK);
        }else{
            if(!asDevice.getStatus().equals(ServiceConstants.VEHICLE_STATUS_NOT_LISTING)){
                asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_NORMAL);//管理员锁车
            }
        }
        asDevice.setIsAdminUnlocking("0");
        int device = asDeviceMapper.updateAsDevice(asDevice);
        if(device==0){
            log.info("【临时解锁】改变车辆状态失败");
            throw new ServiceException("【临时锁车】改变车辆状态失败");
        }
        return Boolean.TRUE;
    }

    /**
     * 管理员锁车
     * @param mac
     * @return
     */
    @Transactional
    @SneakyThrows
    @Override
    public Boolean adminLockByMac(String mac,String userName) {
        if(StrUtil.isBlank(mac))throw new ServiceException("mac不能为空");
        AsDevice asDevice = asDeviceMapper.selectAsDeviceByMac(mac);
        /** 2.发送命令*/
        sendCommand(asDevice.getMac(), Token.getToken(),IotConstants.COMMAND_CLOSE+IotConstants.COMMAND_FREQUENCY_3600,"管理员锁车",null,userName);
        asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);
        // 判断该sn是否有正在骑行中的订单，如果有骑行中的订单则修改为临时锁车
        EtOrder etOrder = etOrderService.getCurrentOrder(asDevice.getSn());
        if(ObjectUtil.isNotNull(etOrder)){
            asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_TEMPORARILY_LOCK);
        }else{
            if(!asDevice.getStatus().equals(ServiceConstants.VEHICLE_STATUS_NOT_LISTING)){
                asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_NORMAL);//管理员锁车
            }
        }
        asDevice.setIsAdminUnlocking("0");
        int device = asDeviceMapper.updateAsDevice(asDevice);
        if(device==0){
            log.info("【临时解锁】改变车辆状态失败");
            throw new ServiceException("【临时锁车】改变车辆状态失败");
        }
        return Boolean.TRUE;
    }

    /**
     * 临时解锁
     */
    @Transactional
    @SneakyThrows
    @Override
    public Boolean unlock(String sn, String orderNo,String isBluetooth,String lon,String lat) {
        EtOrder order = etOrderService.selectEtOrderByOrderNo(orderNo);
        if(order==null)throw new ServiceException("订单不存在");
        if(StrUtil.isBlank(sn)) {
            sn = order.getSn();
        }
        AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(sn);
        /** 1.获取token*/
        String token = Token.getToken();
        String finalSn = sn;
        if(!"true".equals(isBluetooth)){
            /** 2.发送命令*/
            sendCommand(asDevice.getMac(), token,IotConstants.COMMAND_OPEN+IotConstants.COMMAND_FREQUENCY_5,"临时解锁",orderNo,order.getUserName());
        }else{
            asDevice.setLongitude(lon);
            asDevice.setLatitude(lat);
            asDevice.setLastTime(DateUtils.getNowDate());
        }
        if(StrUtil.isNotBlank(orderNo)){//有订单号，则是用户骑行中解锁
            /**  改变车辆状态：3-骑行中 */
            asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_IN_USING);//骑行中
            asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_OPEN);
            int device = asDeviceMapper.updateAsDevice(asDevice);
            if(device==0){
                log.info("【临时解锁】改变车辆状态失败");
                throw new ServiceException("【临时解锁】改变车辆状态失败");
            }
            /** 5.记录行程*/
            int tripLog = tripLogService.tripLog(orderNo, finalSn,ServiceConstants.TRIP_LOG_TYPE_TEMPORARILY_UNLOCK);
            if(tripLog==0){
                log.info("【临时解锁】记录行程失败");
                throw new ServiceException("【临时解锁】记录行程失败");
            }
        }else{
            /**  改变车辆锁状态：1-开  */
            asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_OPEN);
            int device = asDeviceMapper.updateAsDevice(asDevice);
            if(device==0){
                log.info("【临时解锁】改变车辆状态失败");
                throw new ServiceException("【临时解锁】改变车辆状态失败");
            }
        }
        return Boolean.TRUE;
    }

    /**
     * 车辆预约
     */
    @Transactional
    @Override
    public OrderResponse deviceAppointment(EtOrderVo appointmentVo) {
        OrderResponse orderResponse = new OrderResponse();
        String orderNo = IdUtils.getOrderNo("qx");;
        AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(appointmentVo.getSn());
        //创建订单
        EtOrder order = etOrderService.createOrder(appointmentVo, orderNo);
        int i = etOrderService.insertEtOrder(order);
        if(i==0){
            throw new ServiceException("【车辆预约】:创建订单失败");
        }
        orderResponse.setOrderNo(orderNo);
        orderResponse.setSessionId(IdUtils.randomUUIDByDigit(8));
        //改变车辆状态
        asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_IN_APPOINTMENT);
        int u = asDeviceMapper.updateAsDevice(asDevice);
        if(u==0){
            throw new ServiceException("【车辆预约】:更新车辆状态失败");
        }

        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(asDevice.getAreaId());
        int delay = 60 * area.getTimeoutMinutes();
        //定时取消预约
        scheduledExecutorService.schedule(() -> {
            log.error("【车辆超时预约】系统自动取消");
            EtOrder order1 = etOrderService.selectEtOrderByOrderNo(order.getOrderNo());
            log.info("【定时取消预约】重新获取订单信息：{}",JSON.toJSON(order1));
            if(order1.getPaid().equals(ServiceConstants.ORDER_PAY_STATUS_PAID)){//已支付订单，跳过
                log.error("【车辆超时预约】订单已支付，跳过");
                return;
            }
            log.error("【车辆超时预约】订单未支付，系统自动处理");
            //未支付   订单更新最后预约时间，并结束订单，做超出预约时间标记
            order.setStatus(ServiceConstants.ORDER_STATUS_CANCEL_APPOINTMENT);
            order.setAppointmentEndTime(new Date());
            order.setAppointmentTimeout("1");
            //计算预约费
            BigDecimal appointmentServiceFee = area.getAppointmentServiceFee();
            BigDecimal fee = appointmentServiceFee.multiply(new BigDecimal(area.getTimeoutMinutes()).divide(new BigDecimal(10)));
            order.setAppointmentFee(fee);
            order.setTotalFee(fee);
            order.setPayFee(fee);
            log.info("【车辆超时预约】,更新订单信息，{}",JSON.toJSON(order));
            int update = etOrderService.updateEtOrder(order);
            if(update==0){
                throw new ServiceException("【车辆超时预约】:更新订单状态失败");
            }
            // 改变车辆状态
            asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_NORMAL);//定时取消预约
            asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_OPEN);
            int device = asDeviceMapper.updateAsDevice(asDevice);
            if(device==0){
                log.error("【车辆超时预约】更新车辆状态失败");
                throw new ServiceException("【车辆超时预约】更新车辆状态失败");
            }
        }, delay, TimeUnit.SECONDS);
        return orderResponse;
    }

    /**
     * 取消预约
     * 1.发送  开锁命令
     * 2.更新车辆状态  status 为 ServiceConstants.VEHICLE_STATUS_NORMAL   开锁状态为  开
     * 3.更新订单状态 status 为 ServiceConstants.ORDER_STATUS_CANCEL_APPOINTMENT  记录预约结束时间，计算并更新预约费
     */
    @Transactional
    @SneakyThrows
    @Override
    public OrderResponse cancelAppointment(EtOrderVo appointmentVo, AsDevice asDevice) {
        OrderResponse orderResponse = new OrderResponse();
        String orderNo = appointmentVo.getOrderNo();
        EtOrder order = etOrderService.selectEtOrderByOrderNo(orderNo);
        /** 5.记录行程*/
        int tripLog = tripLogService.tripLog(order.getOrderNo(),order.getSn(),ServiceConstants.TRIP_LOG_TYPE_UNLOCK_RIDE);
        if(tripLog==0){
            log.info("【套餐取消预约】记录行程失败");
            throw new ServiceException("【套餐取消预约】记录行程失败");
        }
        //更新订单状态 status 为 ServiceConstants.ORDER_STATUS_CANCEL_APPOINTMENT  记录预约结束时间，计算并更新预约费
        order.setAppointmentEndTime(DateUtils.getNowDate());
        //计算预约费
        calculateAppointmentFee(order);
        order.setTotalFee(order.getAppointmentFee());
        order.setPayFee(order.getAppointmentFee());
        order.setStatus(ServiceConstants.ORDER_STATUS_CANCEL_APPOINTMENT);
        log.info("【取消预约】更新订单信息，{}",JSON.toJSON(order));
        int i = etOrderService.updateEtOrder(order);
        if(i==0){
            throw new ServiceException("【取消预约】:更新订单失败");
        }
        asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_NORMAL);//取消预约
        asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_OPEN);
        int device = asDeviceMapper.updateAsDevice(asDevice);
        if(device==0){
            log.error("【取消预约】更新车辆状态失败");
            throw new ServiceException("【取消预约】更新车辆状态失败");
        }
        orderResponse.setOrderNo(orderNo);
        orderResponse.setSessionId(IdUtils.randomUUIDByDigit(8));
        return orderResponse;
    }

    /**
     * 计算预约费
     * @param order
     */
    private void calculateAppointmentFee(EtOrder order) {
        int minutes = DateUtils.timeDifferenceInMinutes(order.getAppointmentEndTime(), order.getAppointmentStartTime());//预约分钟数
        //minutes   取模
        if(minutes==0){
            minutes = 1;
        }
        BigDecimal divide = new BigDecimal(minutes).divide(new BigDecimal(10));
        BigDecimal result = divide.setScale(0, RoundingMode.UP);
        //BigDecimal divide 取整 有小数点则+1
        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(order.getAreaId());
        BigDecimal appointmentServiceFee = area.getAppointmentServiceFee();
        Integer timeoutMinutes = area.getTimeoutMinutes();
        if(minutes>timeoutMinutes){
            //预约分钟数超过最多保留时间
            throw new ServiceException("预约时间超过最多保留分钟");
        }else{
            //根据开始时间和结束时间计算费用  预约费 = (预约结束时间 - 开始时间) * 预约服务费
            BigDecimal appointmentFee = appointmentServiceFee.multiply(result);
            log.info("【计算预约费】：预约分钟数："+minutes+"，取整："+result+"，预约费："+appointmentFee);
            order.setAppointmentFee(appointmentFee);
        }
    }


    /**
     * 还车
     */
    @SneakyThrows
    @Override
    @Transactional
    public Boolean returnVehicle(String orderNo,String returnType,String isBluetooth,String lon,String lat,String voltage,String isInParkingArea) {
        if(StrUtil.isNotBlank(orderNo)){
            EtOrder order = etOrderService.selectEtOrderByOrderNo(orderNo);
            if(ObjectUtil.isNull(order)){
                throw new ServiceException("订单不存在");
            }
            if(!ServiceConstants.ORDER_TYPE_RIDING.equals(order.getType())){
                throw new ServiceException("该订单非骑行订单");
            }
            if(!ServiceConstants.ORDER_STATUS_RIDING.equals(order.getStatus())){
                throw new ServiceException("该订单状态非骑行中");
            }
            EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(order.getAreaId());
            if(ServiceConstants.RETURN_TYPE_NORMAL.equals(returnType)){
                //判断是否在禁停区内，如果在禁停区内，不能还车。  noParkingArea
                if(isNoParkingArea(order.getSn(), order.getAreaId())){
                    throw new ServiceException("在禁停区内，不能还车");
                }
                // 停车点还车  todo
                if(area.getParkingReturn().equals("1")){
                    if(StrUtil.isNotBlank(isInParkingArea) && !"true".equals(isInParkingArea)){
                        throw new ServiceException("不在停车点内，不能还车");
                    }
                }
                // 运营区外还车
                if(area.getAreaOutReturn().equals("0")){
                    if(!isAreaZone(order.getSn(), area)){
                        throw new ServiceException("在运营区外，不能还车");
                    }
                }
            }
            /** 1. 记录还车时间*/
            order.setReturnType(returnType);
            order.setStatus(ServiceConstants.ORDER_STATUS_RIDING_END);
            order.setReturnTime(DateUtils.getNowDate());
//            order.setReturnTime(DateUtils.dateTime(DateUtils.YYYY_MM_DD_HH_MM_SS, "2024-07-23 17:09:32"));//2024-06-28 18:15:56
            String token = Token.getToken();
            AsDevice device = asDeviceMapper.selectAsDeviceBySn(order.getSn());
            if(ObjectUtil.isNull(device)){
                String usedSn = order.getUsedSn();
                // 判断字符串中是否包含逗号
                if (usedSn.contains(",")) {
                    // 使用逗号分割字符串
                    String[] parts = usedSn.split(",");
                    // 获取最后一个对象
                    String lastPart = parts[parts.length - 1];
                    order.setSn(lastPart);
                } else {
                    order.setSn(usedSn);
                }
            }
            if(ServiceConstants.RETURN_TYPE_NORMAL.equals(returnType)){
                if(!"true".equals(isBluetooth)){
                    /** 2. 车辆远程关锁*/
                    ResponseVo responseVo = sendCommandWithResp(device.getMac(), token, IotConstants.COMMAND_CLOSE + IotConstants.COMMAND_FREQUENCY_3600, "还车关锁",orderNo,lon,lat);
                    if(responseVo.getCode()!=0){
                        log.info("【还车关锁】远程关锁失败");
                        throw new ServiceException("远程关锁失败");
                    }
                }else{
                    // 更新定位
                    device.setLongitude(lon);
                    device.setLatitude(lat);
                    if(StrUtil.isNotBlank(voltage)){
                        BigDecimal divide = new BigDecimal(voltage).divide(new BigDecimal(10));
                        device.setVoltage(divide.toString());
                        EtModel model = etModelService.selectEtModelByModelId(device.getModelId());
                        Integer remainingMileage = 0;
                        if(StrUtil.isNotBlank(device.getVoltage())){
                            remainingMileage = CommonUtil.getRemainingMileage(device.getVoltage(), model.getFullVoltage(), model.getLowVoltage(), model.getFullEndurance());
                        }
                        Integer electricQuantity = CommonUtil.getElectricQuantity(device.getVoltage(), model.getFullVoltage(), model.getLowVoltage());//电量百分百
                        device.setRemainingMileage(remainingMileage);
                        device.setRemainingPower(electricQuantity.toString());
                    }
                    device.setLastTime(DateUtils.getNowDate());
                }
            }else{
                if(ObjectUtil.isNotNull(device)){
                    ResponseVo responseVo = sendCommandWithResp(device.getMac(), token, IotConstants.COMMAND_CLOSE + IotConstants.COMMAND_FREQUENCY_3600, "辅助还车关锁",orderNo);
                    if(responseVo.getCode()!=0){
                        log.info("【还车关锁】远程关锁失败");
                    }
                }
            }
            if(ObjectUtil.isNotNull(device)){
                /** 4. 更新车辆状态*/
                device.setStatus(ServiceConstants.VEHICLE_STATUS_NORMAL);//还车
                device.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);
                int deviceUpdate = asDeviceMapper.updateAsDevice(device);
                if(deviceUpdate==0){
                    log.info("【还车关锁】更新车辆状态失败");
                    throw new ServiceException("【还车关锁】更新车辆状态失败");
                }
            }
            /** 3. 计算订单费用，保存订单总金额*/
            order = calculateOrderFee(order,isInParkingArea);
            /** 4. 优惠券计算价格
             *      order中有couponId则有使用优惠券   */
            if(ObjectUtil.isNotNull(order.getCouponId())){
                order = calculateCoupon(order);
            }
            /** 6.计算行车距离*/
            String tripRouteStr = order.getTripRouteStr();
            if(StrUtil.isNotBlank(tripRouteStr)){
                double[][] doubles = GeoUtils.parseJsonTrack(tripRouteStr);
                double v = GeoUtils.calculateTotalDistance(doubles);
                order.setDistance((int)Math.round(v));
            }
            // 开启押金抵扣  直接将骑行费抵扣押金并退款
            /**
             *  1. 判断是否开启押金抵扣
             *  2. 获取最后一次押金
             *  3. 将抵扣后的金额 = 押金-骑行订单金额
             *      *      如果金额大于0，
             *      *      如果金额小于0，押金订单金额 = 0，更新骑行订单状态为未支付，
             *  4. 更新押金订单金额（做一个押金抵扣的标记）
             *  5. 更新骑行订单状态为已支付，支付方式为 押金抵扣
             *  6. 更新用户余额
             * */
//            String isDepositDeduction = area.getIsDepositDeduction();
//            /** 1. 判断是否开启押金抵扣*/
//            if(ObjectUtil.isNotNull(isDepositDeduction) && "1".equals(isDepositDeduction)){
//                if(ServiceConstants.RETURN_VERIFY_NO.equals(area.getReturnVerify())){//关闭还车审核，订单直接用押金支付
//                    depositDeduction(order);
//                }else{
//                    throw new ServiceException("拍照审核和押金点开不能同时开启");
//                }
//            }
            log.info("【还车】，更新订单状态和还车时间，{}",JSON.toJSON(order));
            int i = etOrderService.updateEtOrder(order);
            if(i==0){
                throw new ServiceException("更新订单状态失败");
            }
            /** 5.记录行程*/
            if(ObjectUtil.isNotNull(device)){
                int tripLog = tripLogService.tripLog(order.getOrderNo(),device.getSn(),ServiceConstants.TRIP_LOG_TYPE_RETRUN_LOCK);
                if(tripLog==0){
                    log.info("【还车关锁】记录行程失败");
                    throw new ServiceException("【还车关锁】记录行程失败");
                }
            }
            log.info("还车成功");
            return Boolean.TRUE;
        }else{
            throw new ServiceException("订单号不能为空");
        }
    }

//    /** 押金抵扣 */
//    private void depositDeduction(EtOrder order) {
////        EtFeeRule rule = etFeeRuleService.selectEtFeeRuleByRuleIdIncludeDelete(order.getRuleId());
////        Integer autoRefundDeposit = rule.getAutoRefundDeposit();
////        log.info("【还车关锁】进入押金抵扣环节--------"+autoRefundDeposit+"个小时后退还押金");
//        /** 2. 获取最后一次押金*/
//        EtOrder depositOrder = etOrderService.getDepositOrder(order.getUserId());
//        BigDecimal deposit = depositOrder.getTotalFee();
//        BigDecimal ridingFee = order.getTotalFee();
//        BigDecimal afterDeductionFee;
//        String mark;
//        if(deposit.compareTo(ridingFee) <= 0){// 骑行费大于押金，订单为未支付, 抵扣后的金额 = 骑行费 - 押金
//            afterDeductionFee = BigDecimal.ZERO;
//            mark = "押金抵扣成功，骑行费【"+ridingFee+"】大于押金【"+deposit+"】";
//            order.setStatus(ServiceConstants.ORDER_STATUS_RIDING_END);
//            order.setPayFee(deposit);
//        }else{
//            // 押金大于订单金额  扣除后
//            afterDeductionFee = deposit.subtract(ridingFee);
//            mark = "押金抵扣成功，骑行费【"+ridingFee+"】小于押金【"+deposit+"】,扣除后金额【"+afterDeductionFee+"】";
//            order.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
//        }
//        /** 更新骑行订单*/
//        order.setPaid(ServiceConstants.ORDER_PAY_STATUS_PAID);
//        order.setPayTime(DateUtils.getNowDate());
//        order.setPayType(ServiceConstants.PAY_TYPE_YJ);
//        order.setMark(mark);
//        order.setDepositDeduction(ServiceConstants.IS_DEPOSIT_DEDUCTION);
//        /** 更新押金订单*/
//        depositOrder.setDepositDeduction(ServiceConstants.IS_DEPOSIT_DEDUCTION);
//        int updateEtOrder1 = etOrderMapper.updateEtOrder(depositOrder);
//        if(updateEtOrder1 == 0){
//            throw new ServiceException("押金抵扣失败，更新押金订单失败");
//        }
//        /** 押金抵扣后生成资金流水记录 */
//        callbackService.capitalFlowRecords(order,ServiceConstants.FLOW_TYPE_INCOME,ServiceConstants.ORDER_TYPE_RIDING,ServiceConstants.OWNER_TYPE_OPERATOR,null,ServiceConstants.PAY_TYPE_YJ);
//        //创建一个定时器，计算出退还时间后，执行退款操作
//        // 往后推autoRefundDeposit小时执行
////        if(autoRefundDeposit!=null){
////            scheduledExecutorService.schedule(() -> {
//                if(afterDeductionFee.compareTo(BigDecimal.ZERO) > 0){
//                    /** 退款剩余押金*/
//                    Refund refund = wxPayService.refund(depositOrder, "押金抵扣退款",afterDeductionFee,IdUtils.getOrderNo("ref"));
//                    /** 2.记录退款表  创建退款对象*/
//                    depositOrder.setReason("押金抵扣退款");
//                    EtRefund refund1= etOrderService.createRefund(depositOrder, afterDeductionFee, null, null, null, null, refund.getOutRefundNo(),ServiceConstants.REFUND_TYPE_DEPOSIT);
//                    int i = etRefundService.insertEtRefund(refund1);
//                    if(i == 0){
//                        log.info("【押金抵扣】保存退款对象失败");
//                        throw new ServiceException("【押金抵扣】，保存退款对象失败");
//                    }
//                    /** 更新用户余额*/
//                    AsUser asUser = asUserService.selectUserById(depositOrder.getUserId());
//                    if(asUser!=null){
//                        // 更新用户并更新缓存
//                        asUser.setBalance(BigDecimal.ZERO);
//                        if (asUserService.updateUserProfile(asUser) > 0)
//                        {
//                            log.info("【押金抵扣成功】更新用户信息成功:"+ JSON.toJSON(asUser));
//                        }else{
//                            throw new ServiceException("【押金抵扣】，更新用户信息失败");
//                        }
//                    }
//                }
////            }, autoRefundDeposit , TimeUnit.HOURS);
////        }
//    }

     /**
     * 使用优惠券后的价格
      *  1. 先判断是否过期？并且是否是未使用的状态
      *  2. 再判断是哪种类型的优惠券，
      *     如果是
      *     时间卡：骑行价格直接改成0
      *     贵宾卡：所有价格改成0
      *     折扣卡：总金额 * 折扣比例  次数减一
      *     抵用券：总金额 - 抵用金额  次数减一
     */
    private EtOrder calculateCoupon(EtOrder order) {
        Long couponId = order.getCouponId();
        EtCoupon etCoupon = etCouponMapper.selectEtCouponByCouponId(couponId);
        Long userId = order.getUserId();
        AsUser asUser = asUserMapper.selectUserById(userId);

        EtCouponUserLog etCouponUserLog = new EtCouponUserLog();
        etCouponUserLog.setUserId(userId);
        etCouponUserLog.setCouponId(couponId);
        etCouponUserLog.setStatus(ServiceConstants.COUPON_STATUS_UNUSED);
        List<EtCouponUserLog> etCouponUserLogs = etCouponClaimLogMapper.selectEtCouponClaimLogList(etCouponUserLog);
        EtCouponUserLog etCouponUserLog1 = null;
        if(etCouponUserLogs !=null && etCouponUserLogs.size()>0){
            etCouponUserLog1 = etCouponUserLogs.get(0);
        }

        if(etCoupon.getType().equals(asUser.getVipType()) || asUser.getExpirationTime().compareTo(new Date()) < 0){
            throw new ServiceException("会员已过期");
        }

        if((etCoupon.getType().equals(ServiceConstants.COUPON_TYPE_DISCOUNT_CARD) || etCoupon.getType().equals(ServiceConstants.COUPON_TYPE_VOUCHER))
                && isCouponExpired(etCouponUserLog1)){
            throw new ServiceException("优惠券已过期");
        }

        if(etCouponUserLog1.getLimitNum() == 0){
            throw new ServiceException("可用次数不足");
        }

        if(etCoupon.getType().equals(ServiceConstants.COUPON_TYPE_TIME_CARD)){
            order.setPayFee(order.getTotalFee().subtract(order.getRidingFee()));//会员
            order.setRidingFee(BigDecimal.ZERO);
        }else if(etCoupon.getType().equals(ServiceConstants.COUPON_TYPE_VIP_CARD)){//贵宾卡
            order.setPayFee(BigDecimal.ZERO);
        }else if(etCoupon.getType().equals(ServiceConstants.COUPON_TYPE_DISCOUNT_CARD)){//折扣卡
            order.setPayFee(order.getTotalFee().multiply(etCoupon.getDiscountPercent()).setScale(2, RoundingMode.HALF_UP));
        }else if(etCoupon.getType().equals(ServiceConstants.COUPON_TYPE_VOUCHER)){//抵用券
            order.setPayFee(order.getTotalFee().subtract(etCoupon.getDiscountAmount()));
        }else{
            throw new ServiceException("优惠券类型错误");
        }

        // 扣除一次使用次数
        if(etCoupon.getType().equals(ServiceConstants.COUPON_TYPE_DISCOUNT_CARD) || etCoupon.getType().equals(ServiceConstants.COUPON_TYPE_VOUCHER) && etCouponUserLog1.getLimitNum() > 0){
            etCouponClaimLogMapper.deductLimitNum(etCouponUserLog1.getLogId());
        }
        return order;
    }

    /** 判断优惠券是否过期 */
    private boolean isCouponExpired(EtCouponUserLog etCouponUserLog) {
        if(etCouponUserLog!=null){
            return etCouponUserLog.getExpirationTime().compareTo(new Date()) < 0;
        }else{
            return true;
        }
    }

    /**
     * 计算订单费用
     */
    private EtOrder calculateOrderFee(EtOrder order,String isInParkingArea) {
        String type = order.getType();
        if(type.equals("1")){//骑行订单,正常骑行（包含预约费），计时收费，根据开锁时间、起步价和起步时长、时长费和时长分钟等参数，如果有预约，还要加上预约费计算费用,还要判断封顶费用（根据开锁时间）
            String rentalUnit = order.getRentalUnit();//租赁单位

            //根据订单的开始时间和还车时间计算出在第几个计费周期，在第一个计费周期内，并没有超过封顶费用，则根据规则算出骑行费，
            Integer cycle = null;
            Date startTime = order.getUnlockTime();
            Date endTime = order.getReturnTime();
            String chargingCycle = order.getChargingCycle();
            String chargingCycleValue = order.getChargingCycleValue();

            if(ServiceConstants.CHARGING_CYCLE_HOUR.equals(chargingCycle)) {//订单生成后__小时  第几个
                cycle = inHowManyChargingCycle(startTime, endTime, chargingCycleValue);
                BigDecimal ridingFee;
                if(cycle == 1){
                    order.setCycle(1);//在第几个计费周期内 rule, rule.getRidingRuleJson(), rentalUnit
                    ridingFee = calculateRidingFee(order,order.getUnlockTime(), order.getReturnTime());
                }else {
                    // 判断一个计费周期是否超过封顶费用，如果超出  骑行费 = (第几个周期 - 1) *  封顶费用 + 骑行费
                        // 如果不超出，骑行费 =  骑行费
                    Date oneCycleAfter = DateUtils.getTimeAfterXHours(order.getUnlockTime(), Integer.parseInt(chargingCycleValue));
                    if(isFeeExceedingCap(order.getUnlockTime(),oneCycleAfter,order,order.getRentalUnit())){
                        // 如果超出 骑行费 = (第几个周期 - 1) *  封顶费用 + 骑行费
                        order.setCycle(cycle);
                        Date afterXHours = DateUtils.getTimeAfterXHours(order.getUnlockTime(), Integer.parseInt(chargingCycleValue)*(cycle-1));
                        BigDecimal remainingFee = calculateRidingFee(order,afterXHours,order.getReturnTime());
                        ridingFee = new BigDecimal(cycle - 1).multiply(order.getCappedAmount()).add(remainingFee);
                    }else{
                        ridingFee  = calculateRidingFee(order, order.getUnlockTime(), order.getReturnTime());
                    }
                }
                order.setRidingFee(ridingFee);
            }else if(ServiceConstants.CHARGING_CYCLE_CUSTOM.equals(chargingCycle)){//自定义时刻
                // todo 自定义计费周期时刻  获取到自定义时刻，判断骑行的这段时间是否在自定义时刻里    ____.___.___|___________|
                SimpleDateFormat timeFormat = new SimpleDateFormat(DateUtils.DATE_FORMAT_HHMMSS);
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(order.getUnlockTime());

                // 将自定义时间点设为当天
                Date customTime = null;
                try {
                    customTime = timeFormat.parse(chargingCycleValue);
                } catch (ParseException e) {
                    e.printStackTrace();
                }
                if (customTime != null) {
                    calendar.set(Calendar.HOUR_OF_DAY, customTime.getHours());
                    calendar.set(Calendar.MINUTE, customTime.getMinutes());
                    calendar.set(Calendar.SECOND, customTime.getSeconds());
                }
                Date customCycleTime = calendar.getTime();

                BigDecimal ridingFee;
                if (order.getUnlockTime().before(customCycleTime) && endTime.after(customCycleTime)) {//如果自定义时间在骑行时间段内，分段计算骑行费用，并考虑封顶费用。
                    BigDecimal firstPeriodFee = calculateRidingFee(order,order.getUnlockTime(), customCycleTime);
                    BigDecimal secondPeriodFee = calculateRidingFee(order,customCycleTime, endTime);

                    if (firstPeriodFee.compareTo(order.getCappedAmount()) > 0) {
                        firstPeriodFee = order.getCappedAmount();
                    }
                    if (secondPeriodFee.compareTo(order.getCappedAmount()) > 0) {
                        secondPeriodFee = order.getCappedAmount();
                    }

                    ridingFee = firstPeriodFee.add(secondPeriodFee);
                } else {// 如果自定义时间不在骑行时间段内，直接计算骑行费用并考虑封顶费用。
                    ridingFee = calculateRidingFee(order,order.getUnlockTime(), endTime);
                    if (ridingFee.compareTo(order.getCappedAmount()) > 0) {
                        ridingFee = order.getCappedAmount();
                    }
                }

                order.setRidingFee(ridingFee);
            }else{
                if(rentalUnit.equals(ServiceConstants.RENTAL_UNIT_DAY)){

                }
            }

            EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(order.getAreaId());
            /** 停车点外调度费*/
            // 根据车辆的定位判断是否在停车区内
            if(StrUtil.isNotBlank(isInParkingArea) && !"true".equals(isInParkingArea)){
                BigDecimal vehicleManagementFee = area.getVehicleManagementFee();//车辆停车点外调度费
                order.setManageFee(vehicleManagementFee);
            }else{
                order.setManageFee(BigDecimal.ZERO);
            }
            /** 调度费*/
            //判断是否在运营区内
            if(StrUtil.isNotBlank(order.getSn()) && !isAreaZone(order.getSn(), area)){
                BigDecimal dispatchFee = area.getDispatchFee();//调度费
                order.setDispatchFee(dispatchFee);
            }
            BigDecimal totalFee = BigDecimal.ZERO;
            if (order != null) {
                BigDecimal ridingFee;
                // 确保所有费用都是BigDecimal.ZERO，如果它们没有被设置
                if (order.getManageFee() == null) {
                    order.setManageFee(BigDecimal.ZERO);
                }
                if (order.getDispatchFee() == null) {
                    order.setDispatchFee(BigDecimal.ZERO);
                }
                if (order.getAppointmentFee() == null) {
                    order.setAppointmentFee(BigDecimal.ZERO);
                }
                if (order.getRidingFee() == null || order.getRidingFee().compareTo(BigDecimal.ZERO) == 0) {
                    order.setRidingFee(BigDecimal.ZERO);
                    totalFee = BigDecimal.ZERO;//如果骑行费是0，说明在免费骑行时间内，那订单的总金额也是0
                }else{
                    ridingFee = order.getRidingFee();
                    totalFee = ridingFee.add(order.getManageFee()).add(order.getDispatchFee()).add(order.getAppointmentFee());
                }
            }
            log.info("【计算订单费用】骑行费：" + order.getRidingFee() + "，车辆停车点外调度费：" + order.getManageFee() + "，调度费：" + order.getDispatchFee() + "，预约费：" + order.getAppointmentFee() + "，总计：" + totalFee);
            order.setTotalFee(totalFee);
            order.setPayFee(totalFee);
        }else {
            throw new ServiceException("订单类型错误");
        }
        return order;
    }


    /**
     * 判断一个计费周期是否超过封顶费用
     */
    @NotNull
    private Boolean isFeeExceedingCap(Date startTime, Date endTime, EtOrder order, String rentalUnit) {
        Integer minutes = DateUtils.timeDifferenceInMinutes(endTime,startTime);//计算相隔多少分钟
        StartingRuleVo startingRule = JSONObject.parseObject(order.getRidingRuleJson(), StartingRuleVo.class);
        String timeoutPrice = startingRule.getTimeoutPrice();//超时价格
        Integer timeoutTime = Integer.parseInt(startingRule.getTimeoutTime());//超时时间

        if(rentalUnit.equals(ServiceConstants.RENTAL_UNIT_HOURS)){//租赁单位：hours-小时
            timeoutTime = timeoutTime*60;
        }else if(rentalUnit.equals(ServiceConstants.RENTAL_UNIT_DAY)){//租赁单位：day-天
            timeoutTime = timeoutTime*60*24;
        }

        double ceil = Math.ceil(minutes / timeoutTime) +1 ;
        BigDecimal ridingFee = new BigDecimal(ceil * Double.parseDouble(timeoutPrice));
        return ridingFee.compareTo(order.getCappedAmount())>0;
    }

    /**
     * 计算骑行费
     */
    @NotNull
    private BigDecimal calculateRidingFee(EtOrder order,Date unlockTime,Date returnTime) {
        BigDecimal ridingFee = BigDecimal.ZERO;
        //没有预约，根据订单中的开锁时间计算费用：骑行费用： ((还车时间-开锁时间转成分钟数)/ 时长分钟)-起步价分钟数 * 时长费 + 起步价
        String  rentalUnit = order.getRentalUnit();

        //在免费骑行时间内，骑行费为0
        Integer minutes = DateUtils.timeDifferenceInMinutes(returnTime,unlockTime);//计算相隔多少分钟
        Integer freeRideTime = order.getFreeRideTime();
        if(freeRideTime!=0 && freeRideTime!=null && minutes<=freeRideTime){
            return ridingFee;
        }

        if(order.getRidingRule().equals(ServiceConstants.RIDING_RULE_STARTING_PRICE_MODE)){//1-起步价计费
            StartingRuleVo startingRule = JSONObject.parseObject(order.getRidingRuleJson(), StartingRuleVo.class);
            String startingPrice = startingRule.getStartingPrice();//起步价
            Integer startingTime = Integer.parseInt(startingRule.getStartingTime());//起步时间
            String timeoutPrice = startingRule.getTimeoutPrice();//超时价格
            Integer timeoutTime = Integer.parseInt(startingRule.getTimeoutTime());//超时时间

            if(rentalUnit.equals(ServiceConstants.RENTAL_UNIT_HOURS)){//租赁单位：hours-小时
                startingTime = startingTime*60;
                timeoutTime = timeoutTime*60;
            }else if(rentalUnit.equals(ServiceConstants.RENTAL_UNIT_DAY)){//租赁单位：day-天
                startingTime = startingTime*60*24;
                timeoutTime = timeoutTime*60*24;
            }

            //判断免费骑行时间不得大于起步时间
            if(freeRideTime>startingTime){
                throw new ServiceException("免费骑行时长不得大于起步分钟！");
            }

            if(ObjectUtils.isNotEmpty(returnTime) && ObjectUtils.isNotEmpty(unlockTime)){
                if(minutes<startingTime){
                    //小于起步分钟，按照起步价计算费用
                    ridingFee = new BigDecimal(startingPrice);
                }else{
                    //大于起步分钟，按照时长费计算费用
                    //minutes除以startingMinutes得到的结构四舍五入
                    minutes = minutes - startingTime;//扣掉起步分钟数后
                    double ceil = Math.ceil(minutes / timeoutTime) +1 ;
                    ridingFee = new BigDecimal(ceil * Double.parseDouble(timeoutPrice));
                    BigDecimal startingPriceB = new BigDecimal(startingPrice);
                    ridingFee = ridingFee.add(startingPriceB);
                }
            }
        }else{//2-区间计费
            IntervalRuleVo intervalRuleVo = JSONObject.parseObject(order.getRidingRuleJson(), IntervalRuleVo.class);
            List<IntervalRuleVo.IntervalRule> rule1 = intervalRuleVo.getRule();
            //骑行时长
            Integer duration = DateUtils.timeDifferenceInMinutes(returnTime,unlockTime);//计算相隔多少分钟
            BigDecimal durationInMinutes = new BigDecimal(duration);
            if(rentalUnit.equals(ServiceConstants.RENTAL_UNIT_HOURS)){//租赁单位：hours-小时
                BigDecimal durationInHours = durationInMinutes.divide(BigDecimal.valueOf(60), 0, RoundingMode.CEILING);
                duration = durationInHours.setScale(0, RoundingMode.CEILING).intValue();
            }else if(rentalUnit.equals(ServiceConstants.RENTAL_UNIT_DAY)){//租赁单位：day-天
                BigDecimal durationInDays = durationInMinutes.divide(BigDecimal.valueOf(60*24), 0, RoundingMode.CEILING); // 转换为天数并向上取整
                duration = durationInDays.setScale(0, RoundingMode.CEILING).intValue(); // 转换为整数天数
            }
            //判断在哪个计费区间内，
            for (IntervalRuleVo.IntervalRule intervalRule : rule1) {
                Integer start = intervalRule.getStart();
                Integer end = intervalRule.getEnd();
                Integer unit = intervalRule.getEachUnit();
                BigDecimal fee = intervalRule.getFee();
                log.info("骑行时长---duration: " + duration + ", start: " + start + ", end: " + end + ", unit: " + unit + ", fee: " + fee);
                // 判断duration是否在当前区间内
                if (duration > start && duration <= end) {
                    // duration位于当前区间[start, end]内
                    System.out.println("费用为: " + fee + "元/每" + unit + formatUnit(rentalUnit));
                    ridingFee = new BigDecimal(fee.toString()).multiply(new BigDecimal(duration / unit));
                } else if (duration == end && intervalRule.getEnd() != 9999) {
                    log.info("不在区间内--骑行时长---duration: " + duration + ", start: " + start + ", end: " + end + ", unit: " + unit + ", fee: " + fee);
                    // 特殊处理结束边界为具体值的情况，避免与默认无限大区间混淆
                    System.out.println("达到区间的上限，但不落入下一个区间，费用为: " + fee + "元/每" + unit + formatUnit(rentalUnit));
                    throw new ServiceException("达到区间的上限，但不落入下一个区间，费用为: " + fee + "元/每" + unit + formatUnit(rentalUnit));
                }
            }
        }
        return ridingFee;
    }

    /**
     * 格式化单位
     */
    private String formatUnit(String rentalUnit) {
        if(ServiceConstants.RENTAL_UNIT_MINUTES.equals(rentalUnit)){
            return "分钟";
        }else if(ServiceConstants.RENTAL_UNIT_HOURS.equals(rentalUnit)){
            return "小时";
        } else if(ServiceConstants.RENTAL_UNIT_DAY.equals(rentalUnit)){
            return "天";
        } else {
            return "";
        }
    }


    /**
     * 计算出在第几个计费周期中
     */
    public Integer inHowManyChargingCycle(Date startTime, Date endTime,String chargingCycleValue) {
        // 计算两个时间的毫秒差
        long diffInMillies = endTime.getTime() - startTime.getTime();

        // 将毫秒差转换为小时
        long hours = TimeUnit.MILLISECONDS.toHours(diffInMillies);

        if(hours==0){
            return 1;
        }
        // 计算出在第几个计费周期中
        BigDecimal cycle = BigDecimal.valueOf(hours).divide(new BigDecimal(chargingCycleValue), 0, RoundingMode.CEILING);
        return cycle.intValue();
    }

//    /**
//     * 是否在停车区内
//     */
//    private Boolean isParkingZone(String sn,Long areaId) {
//        Boolean inCircle = false;
//        EtParkingArea parkingArea = new EtParkingArea();
//        parkingArea.setAreaId(areaId);
//        parkingArea.setStatus("0");
//        List<EtParkingArea> parkingAreas = parkingAreaService.selectEtParkingAreaList(parkingArea);
//        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(areaId);
//        if(ObjectUtil.isNull(parkingAreas) || parkingAreas.size() == 0){
//            log.info("运营区【{}】没有停车区,",areaId);
//            throw new ServiceException("运营区【{"+area.getAreaName()+"}】没有停车区");
//        }
//        double tolerance = area.getError(); // 误差距离
//        for (EtParkingArea etParkingArea : parkingAreas) {
//            if(etParkingArea.getType().equals(ServiceConstants.PARKING_AREA_TYPE_PARKFING)){
//                AsDevice device = asDeviceMapper.selectAsDeviceBySn(sn);
//                String latitude = device.getLatitude();
//                String longitude = device.getLongitude();
//                Geometry geometry = GeoUtils.fromWkt(etParkingArea.getBoundary());
//                if(ObjectUtil.isNotNull(etParkingArea.getError())){
//                    tolerance = etParkingArea.getError();
//                }
//                inCircle = GeoUtils.isInPolygonWithTolerance(longitude, latitude, geometry, tolerance);
//                if(inCircle){
//                    log.info("车辆【{}】在停车区【{}】内",sn,etParkingArea.getParkingName());
//                    inCircle = true;
//                    break;
//                }else{
//                    log.info("车辆【{}】不在停车区【{}】内",sn,etParkingArea.getParkingName());
//                }
//            }
//        }
//        return inCircle;
//    }

    private Boolean isParkingZoneByLocation(String longitude, String latitude,Long areaId) {
        Boolean inCircle = false;
        EtParkingArea parkingArea = new EtParkingArea();
        parkingArea.setAreaId(areaId);
        parkingArea.setStatus("0");
        parkingArea.setType(ServiceConstants.PARKING_AREA_TYPE_PARKFING);
        List<EtParkingArea> parkingAreas = parkingAreaService.selectEtParkingAreaList(parkingArea);
        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(areaId);
        if(ObjectUtil.isNull(parkingAreas) || parkingAreas.size() == 0){
            log.info("运营区【{}】没有停车区,",areaId);
            return true;
//            throw new ServiceException("运营区【{}】没有停车区"+areaId.toString());
        }
        double tolerance = area.getError(); // 误差距离
        for (EtParkingArea etParkingArea : parkingAreas) {
            Geometry geometry = GeoUtils.fromWkt(etParkingArea.getBoundary());
            if(ObjectUtil.isNotNull(etParkingArea.getError())){
                tolerance = etParkingArea.getError();
            }
            inCircle = GeoUtils.isInPolygonWithTolerance(longitude, latitude, geometry, tolerance);
            if(inCircle){
                log.info("位置【{}，{}】在停车区【{}】内",longitude,latitude,etParkingArea.getParkingName());
                inCircle = true;
                break;
            }else{
                log.info("位置【{}，{}】不在停车区【{}】内",longitude,latitude,etParkingArea.getParkingName());
            }
        }
        return inCircle;
    }

    /**
     * 是否在运营区内
     */
    @Override
    public Boolean isAreaZone(String sn,EtOperatingArea area) {
        Boolean inCircle  = false;
        AsDevice device = asDeviceMapper.selectAsDeviceBySn(sn);
        String latitude = device.getLatitude();
        String longitude = device.getLongitude();
        log.info("定位：{}，{},=============运营区【{}】，边界：【{}】",longitude,latitude,area.getAreaName(),area.getBoundaryStr());
        Geometry geometry = GeoUtils.fromWkt(area.getBoundary());
        inCircle = GeoUtils.isInCircle(longitude, latitude, geometry);
        if(inCircle){
            log.info("车辆【{}】在运营区【{}】内",sn,area.getAreaName());
            inCircle = true;
        }else{
            log.info("车辆【{}】不在运营区【{}】内",sn,area.getAreaName());
        }
        return inCircle;
    }

    /**
     * 是否禁停区内
     */
    public boolean isNoParkingArea(String sn,Long areaId) {
        Boolean inCircle = false;
        EtParkingArea parkingArea = new EtParkingArea();
        parkingArea.setAreaId(areaId);
        parkingArea.setStatus("0");
        List<EtParkingArea> parkingAreas = parkingAreaService.selectEtParkingAreaList(parkingArea);
        if(ObjectUtil.isNull(parkingAreas) || parkingAreas.size() == 0){
            log.info("运营区【{}】没有禁停区,",areaId);
            return false;
//            throw new ServiceException("运营区【{}】没有禁停区"+areaId.toString());
        }
        for (EtParkingArea etParkingArea : parkingAreas) {
            if(etParkingArea.getType().equals(ServiceConstants.PARKING_AREA_TYPE_NO_PARKFING)){
                AsDevice device = asDeviceMapper.selectAsDeviceBySn(sn);
                String latitude = device.getLatitude();
                String longitude = device.getLongitude();
                Geometry geometry = GeoUtils.fromWkt(etParkingArea.getBoundary());
                inCircle = GeoUtils.isInCircle(longitude, latitude, geometry);
                if(inCircle){
                    log.info("车辆【{}】在禁停区【{}】内",sn,etParkingArea.getParkingName());
                    inCircle = true;
                    break;
                }else{
                    log.info("车辆【{}】不在禁停区【{}】内",sn,etParkingArea.getParkingName());
                }
            }
        }
        return inCircle;
    }

    /**
     * 是否禁行区内
     */
    @Override
    public boolean isNoRidingArea(String sn,Long areaId) {
        Boolean isNoRiding = false;
        EtParkingArea parkingArea = new EtParkingArea();
        parkingArea.setAreaId(areaId);
        parkingArea.setStatus("0");
        List<EtParkingArea> parkingAreas = parkingAreaService.selectEtParkingAreaList(parkingArea);
        if(ObjectUtil.isNull(parkingAreas) || parkingAreas.size() == 0){
            log.info("运营区【{}】没有禁行区,",areaId);
//            throw new ServiceException("运营区【{}】没有禁行区"+areaId.toString());
        }
        for (EtParkingArea etParkingArea : parkingAreas) {
            if(etParkingArea.getType().equals(ServiceConstants.PARKING_AREA_TYPE_BANNED_RIDING)){
                AsDevice device = asDeviceMapper.selectAsDeviceBySn(sn);
                String latitude = device.getLatitude();
                String longitude = device.getLongitude();
                Geometry geometry = GeoUtils.fromWkt(etParkingArea.getBoundary());
                isNoRiding = GeoUtils.isInCircle(longitude, latitude, geometry);
                if(isNoRiding){
//                    log.info("车辆【{}】在禁行区【{}】内",sn,etParkingArea.getParkingName());
                    isNoRiding = true;
                    break;
                }else{
//                    log.info("车辆【{}】不在禁行区【{}】内",sn,etParkingArea.getParkingName());
                }
            }
        }
        return isNoRiding;
    }

    /**
     * 是否禁行区内
     */
    @Override
    public boolean isNoRidingAreaWithTolerance(String sn,Long areaId,int tolerance) {
        Boolean isNoRiding = false;
        EtParkingArea parkingArea = new EtParkingArea();
        parkingArea.setAreaId(areaId);
        parkingArea.setStatus("0");
        List<EtParkingArea> parkingAreas = parkingAreaService.selectEtParkingAreaList(parkingArea);
        if(ObjectUtil.isNull(parkingAreas) || parkingAreas.size() == 0){
            log.info("运营区【{}】没有禁行区,",areaId);
//            throw new ServiceException("运营区【{}】没有禁行区"+areaId.toString());
        }
        for (EtParkingArea etParkingArea : parkingAreas) {
            if(etParkingArea.getType().equals(ServiceConstants.PARKING_AREA_TYPE_BANNED_RIDING)){
                AsDevice device = asDeviceMapper.selectAsDeviceBySn(sn);
                String latitude = device.getLatitude();
                String longitude = device.getLongitude();
                Geometry geometry = GeoUtils.fromWkt(etParkingArea.getBoundary());
                isNoRiding = GeoUtils.isInPolygonWithTolerance(longitude, latitude, geometry,tolerance);
                if(isNoRiding){
                    log.info("车辆【{}】在禁行区（{}米）【{}】内",sn,tolerance,etParkingArea.getParkingName());
                    isNoRiding = true;
                    break;
                }else{
                    log.info("车辆【{}】不在禁行区（{}米）【{}】内",sn,tolerance,etParkingArea.getParkingName());
                }
            }
        }
        return isNoRiding;
    }

    /**
     * 是否低电量
     */
    @Override
    public boolean isLowBattery(String sn) {
        AsDevice device = asDeviceMapper.selectAsDeviceBySn(sn);
        String remainingPower = device.getRemainingPower();
        log.info("车辆【{}】电量为【{}】",sn,remainingPower);
        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(device.getAreaId());
        if(Integer.parseInt(remainingPower) < Integer.parseInt(area.getUndercharge())){
            log.info("车辆【{}】电量为【{}】低于【{}】",sn,remainingPower,area.getUndercharge());
            return true;
        }
        return false;
    }

    /**
     * 判断是否在运营时间内
     */
    @Override
    public Boolean isOperatingTime(String sn) {
        AsDevice device = asDeviceMapper.selectAsDeviceBySn(sn);
        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(device.getAreaId());
        if(area.getAreaTime().equals(ServiceConstants.AREA_TIME_CUSTOM) && !DateUtils.isInTime(area.getAreaTimeStart(),area.getAreaTimeEnd())){
            log.info("不在运营时间内，运营时间为：【{}】--【{}】",area.getAreaTimeStart(),area.getAreaTimeEnd());
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }

    /**
     * 获取所有车辆
     */
    @Override
    public List<AsDevice> allVehicleByArea(Long areaId) {
        QueryWrapper<AsDevice> wrapper = new QueryWrapper<>();
        wrapper.eq("area_id",areaId);
        wrapper.eq("status","1");
        List<AsDevice> asDevices = asDeviceMapper.selectList(wrapper);
        // for循环
        for (AsDevice asDevice : asDevices) {
            EtModel model = etModelService.selectEtModelByModelId(asDevice.getModelId());
            Integer remainingMileage = CommonUtil.getRemainingMileage(asDevice.getVoltage(), model.getFullVoltage(), model.getLowVoltage(), model.getFullEndurance());
            asDevice.setRemainingMileage(remainingMileage);
        }
        return asDevices;
    }

    /**
     * 更新位置信息
     */
    @Override
    public int updateLocation(AsDevice device) {
        UpdateWrapper<AsDevice> wrapper = new UpdateWrapper<>();
        wrapper.eq("sn",device.getSn())
                .set("longitude",device.getLongitude())
                .set("latitude",device.getLatitude())
                .set("remaining_power",device.getRemainingPower())
                .set("last_time",device.getLastTime())
                .set("last_location_time",device.getLastLocationTime())
                .set("voltage",device.getVoltage())
                .set("satellites",device.getSatellites())
                .set("quality",device.getQuality())
                .set("gps",device.getGps())
                .set("signal_strength",device.getSignalStrength());
        return asDeviceMapper.update(null,wrapper);
    }

    /**
     * 判断是否在线
     */
    @SneakyThrows
    @Override
    public boolean isOnline(String sn) {
        AsDevice device = asDeviceMapper.selectAsDeviceBySn(sn);
        ResponseVo responseVo = sendCommandWithResp(device.getMac(), Token.getToken(), "111", "是否在线",null);
        if(responseVo.getCode() == 0){
            return true;
        }
        asynchronousUpdateOnlineStatus(device.getMac());
        return false;
    }

    /**
     * 检查SN是否唯一
     */
    @Override
    public boolean checkSNUnique(AsDevice asDevice) {
        Long deviceId = StringUtils.isNull(asDevice.getDeviceId()) ? -1L : asDevice.getDeviceId();
        AsDevice info = asDeviceMapper.checkSNUnique(asDevice.getSn());
        if (StringUtils.isNotNull(info) && info.getDeviceId().longValue() != deviceId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 检查SN是否唯一
     */
    @Override
    public boolean checkMACUnique(AsDevice asDevice) {
        Long deviceId = StringUtils.isNull(asDevice.getDeviceId()) ? -1L : asDevice.getDeviceId();
        AsDevice info = asDeviceMapper.checkMACUnique(asDevice.getMac());
        if (StringUtils.isNotNull(info) && info.getDeviceId().longValue() != deviceId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 根据mac号判断是否有绑定过
     */
    @Override
    public boolean isBandByMac(String mac) {
        AsDevice info = asDeviceMapper.checkMACUnique(mac);
        if (StringUtils.isNull(info))
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 判断是否在停车区
     */
    @Override
    public IsInParkingAreaVo isInParkingArea(String longitude, String latitude,String areaId,String sn){
        // 线判断车的定位是否在停车区内，如果在停车区内则直接返回，如果不在拿手机的定位再判断是否在停车区内
        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(Long.parseLong(areaId));
        IsInParkingAreaVo isInParkingAreaVo = new IsInParkingAreaVo();
        if(StringUtils.isNotEmpty(sn)){
            AsDevice device = asDeviceMapper.selectAsDeviceBySn(sn);
            double[] latestLocation = getLatestLocation(device.getMac());
            if(ObjectUtil.isNotNull(latestLocation)){
                BigDecimal longitude1 = new BigDecimal(latestLocation[1]).setScale(8, RoundingMode.HALF_UP);
                BigDecimal latitude1 = new BigDecimal(latestLocation[0]).setScale(8, RoundingMode.HALF_UP);
                log.info("【判断是否在停车区】，lon:{}，lat:{}",longitude1,latitude1);
                Boolean parkingZoneByLocation = isParkingZoneByLocation(longitude1.toString(), latitude1.toString(), Long.parseLong(areaId));
                if(parkingZoneByLocation){
                    isInParkingAreaVo.setIsInParkingArea(true);
                    isInParkingAreaVo.setParkingReturn("1".equals(area.getParkingReturn()));
                    return isInParkingAreaVo;
                }
            }
        }
        Boolean parkingZoneByLocation = isParkingZoneByLocation(longitude, latitude, Long.parseLong(areaId));
        isInParkingAreaVo.setIsInParkingArea(parkingZoneByLocation);
        //停车点还车：0-关闭；1-开启
        isInParkingAreaVo.setParkingReturn("1".equals(area.getParkingReturn()));
        return isInParkingAreaVo;
    }

    /**
     * 查询版本并更新
     */
    @Override
    @SneakyThrows
    public boolean updateVersion(String sn) {
        AsDevice device = asDeviceMapper.selectAsDeviceBySn(sn);
        if(device!=null){
            String version = device.getVersion();
            if(StrUtil.isBlank(version)){
                String token = Token.getToken();
                DataPointRes datapoints = historyDatapoints(asDeviceMapper.selectAsDeviceBySn(sn).getMac(), token);
                if(datapoints.getCode() == 0){
                    Data data = datapoints.getData();
                    List<Datastream> datastreams = data.getDevices();
                    for (Datastream datastream: datastreams) {
                        List<Datapoint> datapointList = datastream.getDatastreams();
                        if(ObjectUtil.isNotNull(datapointList)){
                            for (Datapoint datapoint:datapointList) {
                                if(datapoint.getId().equals(IotConstants.ONENET_VER)){
                                    Integer value = (Integer)datapoint.getValue();
                                    log.info("【查询数据点】===>获取到版本值：【{}】", value);
                                    if(value != null){
                                        AsDevice device1 = new AsDevice();
                                        device1.setVersion(value.toString());
                                        device1.setSn(sn);
                                        if(asDeviceMapper.updateAsDeviceBySn(device1) > 0){
                                            log.info("【查询数据点】===>更新版本成功");
                                            return true;
                                        }else{
                                            log.info("【查询数据点】===>更新版本失败");
                                        }
                                    }
                                }else{

                                }
                            }
                        }
                    }
                }
            }
        }
        return false;
    }

    /**
     * 根据时间查询车辆轨迹
     */
    @Override
    public String trajectory(String sn, String startTime,String endTime) {
        AsDevice device = asDeviceMapper.selectAsDeviceBySn(sn);
        List<EtLocationLog> etLocationLogs = etLocationLogMapper.selectEtLocationLogListByCreateTime(device.getMac(), startTime, endTime);
        // 将etLocationLogs中的longitude和latitude 转换成 [[120.2534,27.1048],[120.2556,27.1051],[120.2569,27.1057]] 这样的格式
        List<double[]> coordinatesList = new ArrayList<>();

        for (EtLocationLog log : etLocationLogs) {
            double longitude = Double.parseDouble(log.getLongitude());
            double latitude = Double.parseDouble(log.getLatitude());
            if(longitude != 0.0 && latitude != 0.0){
                coordinatesList.add(new double[]{longitude, latitude});
            }
        }
        StringBuilder trajectoryStr = new StringBuilder();
        int size = coordinatesList.size();

        trajectoryStr.append("[");  // 在循环之前添加一个 "["

        for (int i = 0; i < size; i++) {
            double[] coordinate = coordinatesList.get(i);
            trajectoryStr.append("[").append(coordinate[0]).append(", ").append(coordinate[1]).append("]");

            if (i < size - 1) {
                trajectoryStr.append(",");
            }
        }
        trajectoryStr.append("]");  // 在循环之后添加一个 "]"
        return trajectoryStr.toString();
    }

    /**
     * 根据时间查询车辆轨迹
     */
    @Override
    public String trajectoryByOrderNo(String orderNo) {
        EtOrder order = etOrderMapper.selectEtOrderByOrderNo(orderNo);
        String endTime;
        if(ObjectUtil.isNull(order.getReturnTime())){
            endTime = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, DateUtils.getNowDate());
        }else{
            endTime = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, order.getReturnTime());
        }
        String startTime = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, order.getUnlockTime());
        AsDevice device = asDeviceMapper.selectAsDeviceBySn(order.getSn());
        List<EtLocationLog> etLocationLogs = etLocationLogMapper.selectEtLocationLogListByCreateTime(device.getMac(), startTime, endTime);
        // 将etLocationLogs中的longitude和latitude 转换成 [[120.2534,27.1048],[120.2556,27.1051],[120.2569,27.1057]] 这样的格式
        List<double[]> coordinatesList = new ArrayList<>();

        for (EtLocationLog log : etLocationLogs) {
            double longitude = Double.parseDouble(log.getLongitude());
            double latitude = Double.parseDouble(log.getLatitude());
            coordinatesList.add(new double[]{longitude, latitude});
        }
        StringBuilder trajectoryStr = new StringBuilder();
        int size = coordinatesList.size();

        trajectoryStr.append("[");  // 在循环之前添加一个 "["

        for (int i = 0; i < size; i++) {
            double[] coordinate = coordinatesList.get(i);
            trajectoryStr.append("[").append(coordinate[0]).append(", ").append(coordinate[1]).append("]");

            if (i < size - 1) {
                trajectoryStr.append(",");
            }
        }
        trajectoryStr.append("]");  // 在循环之后添加一个 "]"
        return trajectoryStr.toString();
    }


    /**
     * sn和mac号绑定
     * 注册onenet设备
     */
    @SneakyThrows
    @Override
    public int bandSn(AsDevice asDevice) {
        AsDevice device = asDeviceMapper.selectAsDeviceByMac(asDevice.getMac());
        if(StringUtils.isNotNull(device)){
            throw new ServiceException("该MAC号已经存在");
        }else{
            // 调用onenet接口
            CreateDeviceVo createDeviceVo = new CreateDeviceVo();
            createDeviceVo.setDevice_name(asDevice.getMac());
            createDeviceVo.setProduct_id(productId);
            String result = HttpUtils.sendPostWithToken(deviceUrl,JSON.toJSONString(createDeviceVo), Token.getToken());
            log.info("【sn和mac号绑定】===>IOT请求调用结果:【{}】",result);
            JSONObject paramsObj = JSON.parseObject(result);
            String code = paramsObj.getString("code");
            //记录命令
            if (!HttpStatus.IOT_SUCCESS.equals(code))
            {
                throw new ServiceException(code+"-----"+ paramsObj.getString("msg"));
            }
            try{
                int insert = asDeviceMapper.insert(asDevice);
                if(insert==0){
                    throw new ServiceException("该SN已经存在，请勿重复绑定！");
                }
            }catch (Exception e){
                log.error("该SN已经存在",e.getMessage());
                throw new ServiceException("该SN已经存在，请勿重复绑定！");
            }
        }
        return 1;
    }

    /**
     * 根据mac修改sn
     */
    @SneakyThrows
    @Override
    public int updateSn(AsDevice asDevice) {
        AsDevice device = new AsDevice();
        device.setSn(asDevice.getSn());
        device.setHardwareVersionId(asDevice.getHardwareVersionId());
        device.setMac(asDevice.getMac());
        int i = asDeviceMapper.updateAsDeviceByMac(device);
        if(i>0){
            log.info("【sn和mac号绑定】===>mac【{}】已经绑定过:更新sn【{}】成功",device.getMac(),device.getSn());
        }
        return 1;
    }


    /**
     * 更新最新的位置信息
     */
    @Override
    @SneakyThrows
    public boolean updateLatestLocation(String sn) {
        AsDevice device = asDeviceMapper.selectAsDeviceBySn(sn);
        if(device!=null){
            String version = device.getVersion();
            if(StrUtil.isBlank(version)){
                String token = Token.getToken();
                DataPointRes datapoints = historyDatapoints(asDeviceMapper.selectAsDeviceBySn(sn).getMac(), token);
                if(datapoints.getCode() == 0){
                    Data data = datapoints.getData();
                    List<Datastream> datastreams = data.getDevices();
                    for (Datastream datastream: datastreams) {
                        List<Datapoint> datapointList = datastream.getDatastreams();
                        if(ObjectUtil.isNotNull(datapointList)){
                            for (Datapoint datapoint:datapointList) {
                                if(datapoint.getId().equals(IotConstants.ONENET_LOCATION)){
                                    String string = JSON.toJSONString(datapoint.getValue());
                                    String at = datapoint.getAt();
                                    if(StrUtil.isNotBlank(string)){
                                        LocationVo locationVo = JSONObject.parseObject(string, LocationVo.class);
                                        log.info("【手动更新】： locationVo---【{}】" , JSON.toJSONString(locationVo));
                                        double[] doubles = CommonUtil.coordinateConvert(locationVo.getLon(), locationVo.getLat());
                                        BigDecimal lat = new BigDecimal(doubles[0]).setScale(8, RoundingMode.HALF_UP);
                                        BigDecimal lon = new BigDecimal(doubles[1]).setScale(8, RoundingMode.HALF_UP);
                                        log.info("转换后的GCJ02经纬度：" + lon + "---" + lat);
                                        if(BigDecimal.ZERO.compareTo(lon) != 0 && BigDecimal.ZERO.compareTo(lat) != 0){
                                            /** 计算电量和里程后更新设备*/
                                            int i = updateDevice(at,locationVo, device, lon, lat);
                                            if(i > 0){
                                                log.info("【手动更新】===>更新设备成功");
                                                return true;
                                            }else{
                                                log.info("【手动更新】===>更新设备失败");
                                                return false;
                                            }
                                        }else{
                                            noLocationUpdateDevice(at,locationVo,device);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        return false;
    }

    /**
     * 获取最新的位置信息
     */
    @Override
    @SneakyThrows
    public double[] getLatestLocation(String mac) {
        AsDevice device = asDeviceMapper.selectAsDeviceByMac(mac);
        String token = Token.getToken();
        DataPointRes datapoints = historyDatapoints(mac, token);
        if(datapoints.getCode() == 0){
            Data data = datapoints.getData();
            List<Datastream> datastreams = data.getDevices();
            for (Datastream datastream: datastreams) {
                List<Datapoint> datapointList = datastream.getDatastreams();
                if(ObjectUtil.isNotNull(datapointList)){
                    for (Datapoint datapoint:datapointList) {
                        if(datapoint.getId().equals(IotConstants.ONENET_LOCATION)){
                            String string = JSON.toJSONString(datapoint.getValue());
                            String at = datapoint.getAt();
                            if(StrUtil.isNotBlank(string)){
                                LocationVo locationVo = JSONObject.parseObject(string, LocationVo.class);
                                log.info("【手动更新】： locationVo---【{}】" , JSON.toJSONString(locationVo));
                                double[] doubles = CommonUtil.coordinateConvert(locationVo.getLon(), locationVo.getLat());
                                BigDecimal lat = new BigDecimal(doubles[0]).setScale(8, RoundingMode.HALF_UP);
                                BigDecimal lon = new BigDecimal(doubles[1]).setScale(8, RoundingMode.HALF_UP);
                                log.info("转换后的GCJ02经纬度：" + lon + "---" + lat);
                                if(BigDecimal.ZERO.compareTo(lon) != 0 && BigDecimal.ZERO.compareTo(lat) != 0){
                                    /** 计算电量和里程后更新设备*/
                                    int i = updateDevice(at,locationVo, device, lon, lat);
                                    return doubles;
                                }else{
                                    noLocationUpdateDevice(at,locationVo,device);
                                }
                            }
                        }
                    }
                }
            }
        }
        return null;
    }

    /** 计算电量和里程后更新设备*/
    public int updateDevice(String at,LocationVo locationVo, AsDevice device, BigDecimal lon, BigDecimal lat) {
        device.setLatitude(lat.toString());
        device.setLongitude(lon.toString());
        Integer bat = locationVo.getBat();
        BigDecimal divide = new BigDecimal(bat).divide(new BigDecimal(10));
        log.info("保存电压：" + divide);
        device.setVoltage(divide.toString());//电压
        // 根据电压计算续航里程
        EtModel model = etModelService.selectEtModelByModelId(device.getModelId());
        if(ObjectUtil.isNotNull(model)){
            Integer remainingMileage = 0;
            if(StrUtil.isNotBlank(device.getVoltage())){
                remainingMileage = CommonUtil.getRemainingMileage(device.getVoltage(), model.getFullVoltage(), model.getLowVoltage(), model.getFullEndurance());
            }
            Integer electricQuantity = CommonUtil.getElectricQuantity(device.getVoltage(), model.getFullVoltage(), model.getLowVoltage());//电量百分百
            device.setRemainingMileage(remainingMileage);
            device.setRemainingPower(electricQuantity.toString());
        }
        device.setLastTime(DateUtils.parseDate(at));
        device.setLastLocationTime(DateUtils.parseDate(at));
        device.setGps("1");
        // 信号强度
        device.setSignalStrength(locationVo.getCsq());
        device.setSatellites(locationVo.getS());
        device.setQuality(locationVo.getQ());
        int i = updateLocation(device);
        return i;
    }

    /** 无定位更新设备 */
    private void noLocationUpdateDevice(String at,LocationVo locationVo, AsDevice device) {
        Integer bat = locationVo.getBat();
        BigDecimal divide = new BigDecimal(bat).divide(new BigDecimal(10));
        log.info("保存电压：" + divide);
        device.setVoltage(divide.toString());//电压
        EtModel model = etModelService.selectEtModelByModelId(device.getModelId());
        // 根据电压计算续航里程
        if(ObjectUtil.isNotNull(model)){
            Integer remainingMileage = 0;
            if(StrUtil.isNotBlank(device.getVoltage())){
                remainingMileage = CommonUtil.getRemainingMileage(device.getVoltage(), model.getFullVoltage(), model.getLowVoltage(), model.getFullEndurance());
            }
            Integer electricQuantity = CommonUtil.getElectricQuantity(device.getVoltage(), model.getFullVoltage(), model.getLowVoltage());//电量百分百
            device.setRemainingMileage(remainingMileage);
            device.setRemainingPower(electricQuantity.toString());
        }
        device.setLastTime(DateUtils.parseDate(at));
        device.setGps("0");
        device.setSignalStrength(locationVo.getCsq());
        device.setSatellites(0);
        device.setQuality(locationVo.getQ());
        int i = updateLocation(device);
        if(i>0){
            log.info("未获取到定位===============保存电压等数值成功===========>" + device.getSn());
        }
    }



    /**
     * 判断是否靠近边界
     */
//    @Override
//    public boolean isCloseToTheBoundary(String sn, EtOperatingArea area) {
//        Boolean isClose = false;
//        AsDevice device = asDeviceMapper.selectAsDeviceBySn(sn);
//        String latitude = device.getLatitude();
//        String longitude = device.getLongitude();
//        Geometry geometry = GeoUtils.fromWkt(area.getBoundary());
//
//        double distanceToBoundary = GeoUtils.distanceToBoundary(longitude, latitude, geometry);
//        if(distanceToBoundary <= 3.0) {
//            log.info("车辆【{}】靠近【{}】运营边界",sn,area.getAreaName());
//            isClose = true;
//        }else{
//            log.info("车辆【{}】为靠近靠近【{}】运营边界",sn,area.getAreaName());
//        }
//        return isClose;
//    }


}
