package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.aliyuncs.dysmsapi.model.v20170525.SendSmsResponse;
import com.aliyuncs.exceptions.ClientException;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ruoyi.common.constant.CacheConstants;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.pay.wx.domain.NotifyEventType;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SendAliSmsUtil;
import com.ruoyi.common.utils.SendSmsVo;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.http.HttpUtils;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.system.domain.*;
import com.ruoyi.system.domain.vo.AttachVo;
import com.ruoyi.system.mapper.*;
import com.ruoyi.system.service.*;
import com.wechat.pay.java.core.RSAAutoCertificateConfig;
import com.wechat.pay.java.core.notification.Notification;
import com.wechat.pay.java.core.notification.NotificationConfig;
import com.wechat.pay.java.core.notification.NotificationParser;
import com.wechat.pay.java.core.notification.RequestParam;
import com.wechat.pay.java.service.payments.model.Transaction;
import com.wechat.pay.java.service.profitsharing.model.CreateOrderReceiver;
import com.wechat.pay.java.service.profitsharing.model.OrdersEntity;
import com.wechat.pay.java.service.profitsharing.model.ReceiverType;
import com.wechat.pay.java.service.refund.model.Refund;
import com.wechat.pay.java.service.refund.model.RefundNotification;
import com.wechat.pay.java.service.refund.model.Status;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;


/**
 * 订单支付回调 CallbackService 实现类
 */
@Service
public class CallbackServiceImpl implements CallbackService {

    private static final Logger logger = LoggerFactory.getLogger(CallbackServiceImpl.class);

    @Autowired
    private IEtOrderService orderService;

    @Resource
    private EtOrderMapper etOrderMapper;

    @Resource
    private EtRefundMapper etRefundMapper;

    @Autowired
    private IAsDeviceService asDeviceService;

    @Autowired
    private IAsUserService userService;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private IEtRefundService etRefundService;

    @Autowired
    private IEtFeeRuleService etFeeRuleService;

    @Autowired
    private IWxPayService wxPayService;

    @Autowired
    private IEtOperatingAreaService etOperatingAreaService;

    @Autowired
    private IEtCapitalFlowService etCapitalFlowService;

    @Resource
    private SysUserMapper userMapper;

    @Resource
    private AsUserMapper asUserMapper;

    @Resource
    private AsDeviceMapper asDeviceMapper;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private IEtDividendDetailService dividendDetailService;

    @Resource
    private EtAreaDeptMapper etAreaDeptMapper;

    @Autowired
    private ISysDeptService deptService;

    @Resource
    private EtCallbackLogMapper callbackLogMapper;

    @Autowired
    private CallbackService callbackService;

    @Resource
    private EtCouponMapper etCouponMapper;

    @Resource
    private EtCouponClaimLogMapper etCouponClaimLogMapper;

    @Value("${aliyun.accessKeyId}")
    private String accessKeyId;

    @Value("${aliyun.accessKeySecret}")
    private String accessKeySecret;

    @Value("${aliyun.signName}")
    private String signName;

    @Value("${aliyun.templateCode}")
    private String templateCode;

    @Value("${aliyun.templateCode2}")
    private String templateCode2;


    @Resource
    private EtMsgLogMapper etMsgLogMapper;


    /**
     * 微信支付回调
     */
    @Override
//    @SneakyThrows
    @Transactional
    public void weChat(HttpServletRequest request) {
        String body = HttpUtils.getBody(request);
        logger.info("【微信支付回调】接收对象（未验签） : " + JSON.toJSONString(body));
        EtCallbackLog etCallbackLog = new EtCallbackLog();
        etCallbackLog.setBody2(body);
        etCallbackLog.setType("1");
        // 解析通知数据
        Notification notification = JSON.parseObject(body, Notification.class);
        String outTradeNo;

        //异步保存回调日志
        asynchronousSaveCallbackLog(etCallbackLog);

        // 支付成功通知
        if (NotifyEventType.TRANSACTION_SUCCESS.getValue().equals(notification.getEventType())) {
            // 验签、解密并转换成 Transaction
            String wechatpaySerial = request.getHeader("Wechatpay-Serial");
            logger.info("证书序列号：" + wechatpaySerial);
            Transaction transaction = checkAndParse(request, body, Transaction.class);
            if (Transaction.TradeStateEnum.SUCCESS.equals(transaction.getTradeState())) {
                // 充值成功后的业务处理
                logger.info("【微信支付回调】交易对象（验签后） : " + JSON.toJSONString(transaction));
                etCallbackLog.setBody(JSON.toJSONString(transaction));
                String transactionId = transaction.getTransactionId();
                AttachVo attachVo = JSONObject.parseObject(transaction.getAttach(),AttachVo.class);
                logger.info("【微信支付回调】附加信息 : " + JSON.toJSONString(attachVo));
                outTradeNo = transaction.getOutTradeNo();
                EtOrder order = orderService.selectEtOrderByOutTradeNo(outTradeNo);
                logger.info("【微信支付回调】订单信息 : " + JSON.toJSONString(order));

                AsUser asUser = asUserMapper.selectUserById(order.getUserId());
                /** 支付回调逻辑  1. 处理预约还是开锁  电压 */
                AsDevice asDevice = null;
                if(StrUtil.isNotBlank(order.getSn())){
                    asDevice = asDeviceMapper.selectAsDeviceBySn(order.getSn());
                }
                //先判断是骑行订单还是押金，如果是骑行订单
                // 还要区分是取消预约支付
                //      如果是,更新订单状态，pay_time、pay_type（默认是wx）、appointment_fee等于total_fee，状态改为4 订单结束
                // 如果是扫码骑行后还车支付
                //      更新订单状态，pay_time、pay_type（默认是wx）、total_fee等于预约费+骑行费+管理费+调度费，状态改为4 订单结束，保存微信内部订单号 callbackVo.getOutTradeNo()
                order.setPaid("1");
                order.setPayTime(DateUtils.getNowDate());
                order.setPayType(ServiceConstants.PAY_TYPE_WX);
                order.setLocking("0");
                if(attachVo.getType().equals(ServiceConstants.BUSINESS_TYPE_RIDING)){
                    logger.info("【微信支付回调】骑行支付");
                    // 1-骑行支付  关锁
                    EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(order.getAreaId());
                    order.setMark("骑行支付");
                    logger.info("=================【微信支付回调】11111111==================");
                    if(ServiceConstants.RETURN_VERIFY_YES.equals(area.getReturnVerify())){
                        logger.info("【微信支付回调】还车-----需要-----拍照审核");
                        order.setStatus(ServiceConstants.ORDER_STATUS_TO_BE_AUDIT);//如果还车需要拍照审核，状态为待审核
                        BigDecimal amount = order.getPayFee();

                        // 异步处理短信
                        if("1".equals(area.getMsgSwitch())){
                            asynchronousMsg(order, amount);
                        }
                    }else{
                        logger.info("【微信支付回调】还车-----不需要-----拍照审核");
                        order.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
                        // 还车结算___小时后自动退押金---创建一个定时器TimerTask，计算出退还时间后，执行退款操作
                        logger.info("=================【微信支付回调】22222222==================");
                        // 退还押金处理
                        refundDeposit(area.getDeposit(), order, asUser);
                        logger.info("=================【微信支付回调】33333333==================");
                        // 用户付款通知
                        if("1".equals(area.getMsgSwitch())){
                            asynchronousMsg2(order);
                        }
                    }
                    asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_NORMAL);//还车后车辆正常运营
                    asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);
                    // 新增资金流水记录
                    EtCapitalFlow capitalFlow = capitalFlowRecords(order, ServiceConstants.FLOW_TYPE_INCOME, ServiceConstants.ORDER_TYPE_RIDING, ServiceConstants.OWNER_TYPE_OPERATOR, null, ServiceConstants.PAY_TYPE_WX);
                    logger.info("=================【骑行支付回调-新增资金流水记录后】=================={}",JSON.toJSON(capitalFlow));
                    order.setHandlingCharge(capitalFlow.getHandlingCharge());
                    order.setPlatformServiceFee(capitalFlow.getPlatformServiceFee());
                    order.setOperatorDividend(capitalFlow.getOperatorDividend());
                    order.setCost(getCost(order.getPayFee()));
                    // 发起分账
//                    BigDecimal dividendAmount= order.getPayFee().subtract(capitalFlow.getHandlingCharge()).subtract(capitalFlow.getPlatformServiceFee());//分账金额（订单实际支付金额-平台服务费-支付手续费）
//                    logger.info("=================【微信支付回调】分账金额=================={}",dividendAmount);
//                    dividendHandle(transactionId, order, area,dividendAmount);
                    logger.info("=================【微信支付回调】4444444==================");
                }else if(attachVo.getType().equals(ServiceConstants.BUSINESS_TYPE_APPOINTMENT)){
                    logger.info("【微信支付回调】取消预约支付");
                    // 2-取消预约支付
                    order.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
                    order.setMark("取消预约支付");
                    asDevice.setStatus(ServiceConstants.VEHICLE_STATUS_NORMAL);//取消预约支付后车辆正常运营
                    asDevice.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);
                }else if(attachVo.getType().equals(ServiceConstants.ORDER_TYPE_COUPON)){
                    /** 优惠券订单  */
                    logger.info("【微信支付回调】优惠券支付");
                    // 3-优惠券支付
                    order.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
                    order.setMark("优惠券支付");
                    // 优惠券成功处理逻辑
                    couponSuccessHandle(order);
                }else if(attachVo.getType().equals(ServiceConstants.BUSINESS_TYPE_DEPOSIT)){
                    logger.info("【微信支付回调】押金支付");
                    // 4-押金支付
                    order.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
                    asUser.setBalance(order.getTotalFee());
                    order.setMark("押金支付");

                    // 新增资金流水记录
//                    capitalFlowRecords(order,ServiceConstants.FLOW_TYPE_INCOME,ServiceConstants.ORDER_TYPE_DEPOSIT);
                    // 删除用户缓存
                    String token = attachVo.getToken();
                    logger.info("【微信支付回调】删除用户缓存:"+token);
                    if (StringUtils.isNotNull(token))
                    {
                        redisCache.deleteObject(CacheConstants.LOGIN_TOKEN_KEY + token);
                    }
                }else{
                    logger.error("【微信支付回调】 : 支付场景不存在");
                    throw new ServiceException("【微信支付回调】支付场景不存在");
                }
                if(ObjectUtil.isNotNull(asDevice)){
                    int device = asDeviceService.updateAsDevice(asDevice);
                    if(device==0){
                        logger.error("【微信支付回调】更新车辆状态失败");
                        throw new ServiceException("【微信支付回调】更新车辆状态失败");
                    }
                }
                logger.info("=================【微信支付回调】开始更新订单信息=================={}",JSON.toJSON(order));
                int updateEtOrder = orderService.updateEtOrder(order);
                if(updateEtOrder==0){
                    logger.error("【微信支付回调】更新订单信息失败");
                    throw new ServiceException("【微信支付回调】更新订单信息失败");
                }
                if(attachVo.getType().equals(ServiceConstants.BUSINESS_TYPE_DEPOSIT)){
                    logger.info("=================【微信支付回调】开始更新用户信息==================");
                    int updateUser = userService.updateUser(asUser);
                    if(updateUser==0){
                        logger.error("【微信支付回调】更新用户押金失败");
                        throw new ServiceException("【微信支付回调】更新用户押金失败");
                    }
                }
                logger.info("=================【微信支付回调】全部结束！！！！！==================");
            }
        }
    }

    private BigDecimal getCost(BigDecimal payFee) {
        // todo 获取到微信的成本
        BigDecimal bigDecimal = new BigDecimal(0.54).divide(new BigDecimal(100), 6, BigDecimal.ROUND_HALF_UP);
        BigDecimal cost = bigDecimal.multiply(payFee).setScale(2, BigDecimal.ROUND_HALF_UP);
        logger.info("【保存资金流水记录--订单支付】 成本==============bigDecimal=====================："+bigDecimal);
        return cost;
    }

    private void couponSuccessHandle(EtOrder order) {
        logger.info("【微信支付回调】优惠券支付-------------1");
        /**
          *   购买分为抵用券订单，会员卡订单、折扣卡订单
          *   1. 插入一条记录，et_coupon_user_log  获取方式是购买
          *   2. 增加订单记录，类型是优惠券订单
          *   3. 增加账变, 所属人为运营商
          * */
        // 1. 插入一条记录，et_coupon_user_log  获取方式是购买
        Long userId = order.getUserId();
        Long couponId = order.getCouponId();
        AsUser asUser = asUserMapper.selectUserById(userId);
        if(ObjectUtil.isNull(asUser)){
            throw new ServiceException("用户【"+userId+"】不存在");
        }
        logger.info("【微信支付回调】优惠券支付-------------2");
        EtCoupon etCoupon = etCouponMapper.selectEtCouponByCouponId(couponId);
        if(ObjectUtil.isNull(etCoupon)){
            throw new ServiceException("优惠券【"+couponId+"】不存在");
        }
        logger.info("【微信支付回调】优惠券支付-------------3");
        EtCouponUserLog etCouponUserLog = new EtCouponUserLog();
        etCouponUserLog.setCouponId(couponId);
        etCouponUserLog.setUserId(userId);
        etCouponUserLog.setUserName(asUser.getUserName());
        etCouponUserLog.setCreateTime(DateUtils.getNowDate());
        etCouponUserLog.setGainMethod(ServiceConstants.COUPON_GAIN_METHOD_BUY);
        etCouponUserLog.setAreaId(etCoupon.getAreaId());
        etCouponUserLog.setAreaName(etCoupon.getAreaName());
        etCouponUserLog.setLimitNum(Integer.parseInt(etCoupon.getLimitNum()));
        if(ObjectUtil.isNull(etCoupon.getValidityUnit())){
            throw new ServiceException("优惠券【有效期单位--validityUnit】为空");
        }
        if(ObjectUtil.isNull(etCoupon.getValidityValue())){
            throw new ServiceException("优惠券【有效期值--validityValue】为空");
        }
        etCouponUserLog.setExpirationTime(DateUtils.addValidity(new Date(),etCoupon.getValidityUnit(),etCoupon.getValidityValue()));
        if(ServiceConstants.COUPON_TYPE_TIME_CARD.equals(etCoupon.getType()) || ServiceConstants.COUPON_TYPE_VIP_CARD.equals(etCoupon.getType())){
            logger.info("【微信支付回调】优惠券支付-------------4");
            AsUser asUser1 = new AsUser(userId);
            asUser1.setVipType(etCoupon.getType());
            Date expirationTime;
            if(ObjectUtil.isNull(asUser.getExpirationTime())){
                expirationTime = DateUtils.getNowDate();
            }else{
                expirationTime = asUser.getExpirationTime();
            }
            asUser1.setExpirationTime(DateUtils.addValidity(expirationTime,etCoupon.getValidityUnit(),etCoupon.getValidityValue()));
            asUser1.setUserId(userId);
            int updateUser = asUserMapper.updateUser(asUser1);
        }
        int i1 = etCouponClaimLogMapper.insertEtCouponClaimLog(etCouponUserLog);
        if(i1 == 0){
            throw new ServiceException("保存优惠券领取记录失败");
        }
        logger.info("【微信支付回调】优惠券支付-------------5");
        // 增加优惠券账变
        callbackService.capitalFlowRecords(order,ServiceConstants.FLOW_TYPE_INCOME,
                        ServiceConstants.ORDER_TYPE_COUPON_INCOME,ServiceConstants.OWNER_TYPE_OPERATOR,null,ServiceConstants.PAY_TYPE_WX);
        logger.info("【微信支付回调】优惠券支付-------------6");
    }

    private void asynchronousMsg(EtOrder order, BigDecimal amount) {
        scheduledExecutorService.schedule(() -> {
            /** 发送一个短信给运营商*/
            SysDept sysDept = wxPayService.getDeptObjByAreaId(order.getAreaId());
            logger.info("【微信支付回调】还车拍照审核，获取到运营商：【{}】",JSON.toJSON(sysDept));

            JSONObject jsonObject = new JSONObject();
            jsonObject.put("name", amount);

            String phone = sysDept.getPhone();
            SendSmsVo sendSmsVo = new SendSmsVo();
            sendSmsVo.setMobile(phone);
            sendSmsVo.setTemplateCode(templateCode);
            sendSmsVo.setParam(jsonObject.toJSONString());
            sendSmsVo.setSignName(signName);
            SendSmsResponse response = null;
            logger.info("【微信支付回调】向阿里云发送短信，请求,----------【{}】", JSON.toJSONString(sendSmsVo));
            try {
                response = SendAliSmsUtil.sendVerifyCode(accessKeyId,accessKeySecret,sendSmsVo);
            } catch (ClientException e) {
                e.printStackTrace();
            }
            logger.info("【微信支付回调】发送阿里云短信成功,返回----------【{}】",JSON.toJSONString(response));
            //记录资金流水
            order.setPayFee(new BigDecimal(0.1));
            callbackService.capitalFlowRecords(order,ServiceConstants.FLOW_TYPE_DISBURSE,
                    ServiceConstants.ORDER_TYPE_MSG,ServiceConstants.OWNER_TYPE_OPERATOR,null,ServiceConstants.PAY_TYPE_YE);

            // 短信日志
            EtMsgLog etMsgLog = new EtMsgLog();
            etMsgLog.setPhone(phone);
            etMsgLog.setContent("您名下有一笔租车订单已归还，订单金额："+ amount +"元。");
            etMsgLog.setType(ServiceConstants.MSG_TYPE_VEHICLE_RETURN_AUDIT_NOTIFY);
            etMsgLog.setSignName(signName);
            etMsgLog.setCreateTime(DateUtils.getNowDate());
            etMsgLog.setTemplateCode(templateCode);
            int i = etMsgLogMapper.insertEtMsgLog(etMsgLog);
            if(i>0){
                logger.info("【微信支付回调】短信日志记录成功");
            }else{
                logger.info("【微信支付回调】短信日志记录失败");
            }
        }, 0 , TimeUnit.HOURS);
    }

    private void asynchronousMsg2(EtOrder order) {
        scheduledExecutorService.schedule(() -> {
            /** 发送一个短信给运营商*/
            SysDept sysDept = wxPayService.getDeptObjByAreaId(order.getAreaId());
            logger.info("【用户付款通知】用户付款通知，获取到运营商：【{}】",JSON.toJSON(sysDept));
            if(ObjectUtil.isNull(sysDept.getAppUserId())){
                throw new ServiceException("该运营商【"+sysDept.getDeptName()+"】未绑定微信用户");
            }
            AsUser asUser1 = asUserMapper.selectUserById(sysDept.getAppUserId());
            AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(order.getSn());

            JSONObject jsonObject = new JSONObject();
            jsonObject.put("order", order.getOrderNo());
            jsonObject.put("cheno", asDevice.getVehicleNum());
            jsonObject.put("snno", order.getSn());
            jsonObject.put("fee", order.getPayFee());

            String phone = asUser1.getPhonenumber();
            SendSmsVo sendSmsVo = new SendSmsVo();
            sendSmsVo.setMobile(phone);
            sendSmsVo.setTemplateCode(templateCode2);
            sendSmsVo.setParam(jsonObject.toJSONString());
            sendSmsVo.setSignName(signName);
            SendSmsResponse response = null;
            logger.info("【用户付款通知】向阿里云发送短信，请求,----------【{}】", JSON.toJSONString(sendSmsVo));
            try {
                response = SendAliSmsUtil.sendVerifyCode(accessKeyId,accessKeySecret,sendSmsVo);
            } catch (ClientException e) {
                e.printStackTrace();
            }
            logger.info("【用户付款通知】发送阿里云短信成功,返回----------【{}】",JSON.toJSONString(response));
            //记录资金流水
            order.setPayFee(new BigDecimal(0.1));
            callbackService.capitalFlowRecords(order,ServiceConstants.FLOW_TYPE_DISBURSE,
                    ServiceConstants.ORDER_TYPE_MSG,ServiceConstants.OWNER_TYPE_OPERATOR,null,ServiceConstants.PAY_TYPE_YE);

            // 短信日志
            EtMsgLog etMsgLog = new EtMsgLog();
            etMsgLog.setPhone(phone);
            etMsgLog.setContent("您的订单："+order.getOrderNo()+"，车牌："+asDevice.getVehicleNum()+"，SN："+order.getSn()+"，用户已付款"+order.getPayFee()+"元，可在小程序管理端查看。");
            etMsgLog.setType(ServiceConstants.MSG_TYPE_USER_PAYMENT_NOTIFY);
            etMsgLog.setSignName(signName);
            etMsgLog.setCreateTime(DateUtils.getNowDate());
            etMsgLog.setTemplateCode(templateCode);
            int i = etMsgLogMapper.insertEtMsgLog(etMsgLog);
            if(i>0){
                logger.info("【用户付款通知】短信日志记录成功");
            }else{
                logger.info("【用户付款通知】短信日志记录失败");
            }
        }, 0 , TimeUnit.HOURS);
    }

    private void asynchronousSaveCallbackLog(EtCallbackLog etCallbackLog) {
        //开异步线程保存回调参数
        scheduledExecutorService.schedule(() -> {
            logger.info("【微信支付回调】异步保存回调参数");
            etCallbackLog.setCreateTime(DateUtils.getNowDate());
            int i = callbackLogMapper.insertEtCallbackLog(etCallbackLog);
            if(i>0){
                logger.info("【微信支付回调】异步保存回调参数成功");
            }
        }, 0, TimeUnit.SECONDS);
    }

    /**
     * 分账处理
     *  1.根据订单号查询分账记录，如果有记录直接返回
     *  2.根据区域id查询合伙人列表，根据合伙人分红比例分账，保存分账明细表
     *  3.请求分账
     *  分账金额：dividendAmount 分账金额（订单实际支付金额-平台服务费-支付手续费）
     */
    @Override
    public boolean dividendHandle(String transactionId, EtOrder order, EtOperatingArea area, BigDecimal dividendAmount) {
        List<EtDividendDetail> etDividendDetails = dividendDetailService.selectEtDividendDetailByOrderNo(order.getOrderNo());
        if(ObjectUtil.isNotNull(etDividendDetails) && etDividendDetails.size()>0){
            return true;
        }
        logger.info("=================【微信支付回调】24小时后开始请求分账==================");
        logger.info("区域对象====="+JSON.toJSONString(area));
        logger.info("订单对象====="+JSON.toJSONString(order));
        // 请求分账
//        List<CreateOrderReceiver> receivers = new ArrayList<>();
        // 获取到合伙人的openid
        SysUser sysUser = new SysUser();
        sysUser.setUserType("03");
        sysUser.setAreaId(area.getAreaId());
        List<SysUser> sysUsers = userMapper.selectUserList(sysUser);

        if(ObjectUtil.isNotNull(sysUsers) && sysUsers.size()>0){
            for (SysUser sysUser1 : sysUsers){
                // 判断是否过合作时间、是否禁用
                boolean cooperationExpired = DateUtils.isCooperationExpired(sysUser1.getCooperationTime());
                if(!cooperationExpired && sysUser1.getStatus().equals("0")){
                    EtDividendDetail etDividendDetail = new EtDividendDetail();
                    AsUser asUser1 = asUserMapper.selectUserById(sysUser1.getAppUserId());
                    if(asUser1!=null && asUser1.getWxopenid()!=null){
                        logger.info("=============系统用户：sysUser1============"+JSON.toJSONString(sysUser1));
                        CreateOrderReceiver receiver = new CreateOrderReceiver();
                        receiver.setType(ReceiverType.PERSONAL_OPENID.name());
                        receiver.setAccount(asUser1.getWxopenid());
                        logger.info("=================分账金额（订单实际支付金额-平台服务费-支付手续费）：dividendAmount=================="+dividendAmount);
                        BigDecimal divide = new BigDecimal(sysUser1.getDividendProportion()).divide(new BigDecimal(100), 2, BigDecimal.ROUND_HALF_UP);
                        logger.info("=================分账比例%=================="+divide);
                        BigDecimal multiply = dividendAmount.multiply(divide);
                        logger.info(sysUser1.getUserName()+"分账比例："+sysUser1.getDividendProportion()+"%，分账金额："+multiply);
                        receiver.setAmount(multiply.multiply(new BigDecimal(100)).longValue());
                        receiver.setDescription(area.getAreaName()+"共享电动车自动分账");
                        etDividendDetail.setAreaId(area.getAreaId());
                        etDividendDetail.setType(ServiceConstants.PROFITSHARING_TYPE_PARTNER);
                        etDividendDetail.setPartnerId(sysUser1.getUserId());
                        etDividendDetail.setOrderNo(order.getOrderNo());
                        etDividendDetail.setTotalAmount(order.getPayFee());
                        etDividendDetail.setCreateTime(DateUtils.getNowDate());
                        etDividendDetail.setDividendProportion(sysUser1.getDividendProportion());
                        etDividendDetail.setDividendAmount(multiply);
                        logger.info("【微信支付回调】保存分账明细 === " + JSON.toJSONString(etDividendDetail));
                        int i = dividendDetailService.insertEtDividendDetail(etDividendDetail);
                        if(i==0){
                            throw new ServiceException("保存分账明细失败");
                        }
                        order.setPayFee(multiply);
                        callbackService.capitalFlowRecords(order,ServiceConstants.FLOW_TYPE_INCOME,ServiceConstants.ORDER_TYPE_RIDING,ServiceConstants.OWNER_TYPE_PARTNER,sysUser1,ServiceConstants.PAY_TYPE_WX);
                    }
                }else{
                    logger.info("=================【微信支付回调】合伙人【{}】已禁用或已过期合作期==================",sysUser1.getUserName());
                }
            }
        }

        // 计算平台服务费
//        logger.info("=================【微信支付回调】计算平台服务费==================");
//        platformServiceFee(order, area);

//        List<EtAreaDept> areaId1 = etAreaDeptMapper.selectList(new QueryWrapper<EtAreaDept>().eq("area_id", order.getAreaId()));
//        if (ObjectUtil.isNotEmpty(areaId1) && areaId1.size() > 0){
//            EtAreaDept etAreaDept = areaId1.get(0);
//            SysDept sysDept = deptService.selectDeptById(etAreaDept.getDeptId());
//            logger.info("获取到运营商对象：【{}】",JSON.toJSON(sysDept));
//            OrdersEntity ordersEntity = wxPayService.createOrder(sysDept,transactionId,receivers);
//            if(ordersEntity!=null){
//                logger.info("【微信支付回调】发起分账响应：【{}】",JSON.toJSON(ordersEntity));
//            }else{
//                logger.info("【微信支付回调】发起分账失败");
//                throw new ServiceException("发起分账失败");
//            }
//        }else{
//            logger.info("区域：【{}】没有绑定运营商",order.getAreaId());
//            throw new ServiceException("区域：【"+order.getAreaId()+"】没有绑定运营商");
//        }
        return false;
    }

    /**
     * 计算平台服务费
     */
    private void platformServiceFee(EtOrder order, EtOperatingArea area) {
//        CreateOrderReceiver receiver = new CreateOrderReceiver();
        SysDept cxPlatform = deptService.selectDeptById(100L);//创享电动车
//        receiver.setType(ReceiverType.MERCHANT_ID.name());
//        receiver.setAccount(cxPlatform.getMerchantId());
//        receiver.setDescription("平台服务费");
        BigDecimal divide = new BigDecimal(cxPlatform.getPlatformServiceFee()).divide(new BigDecimal(100), 2, BigDecimal.ROUND_HALF_UP);
        BigDecimal platformServiceFee = order.getTotalFee().multiply(divide);
        logger.info("平台服务费比例："+cxPlatform.getPlatformServiceFee()+"%，分账金额："+platformServiceFee);
//        receiver.setAmount(platformServiceFee.longValue());//平台服务费 = 平台服务费比例 * 订单金额
//        receivers.add(receiver);

        EtDividendDetail etDividendDetail = new EtDividendDetail();
        etDividendDetail.setAreaId(area.getAreaId());
        etDividendDetail.setType("2");
        etDividendDetail.setOrderNo(order.getOrderNo());
        etDividendDetail.setType(ServiceConstants.PROFITSHARING_TYPE_PLATFORM);
        etDividendDetail.setTotalAmount(order.getTotalFee());
        etDividendDetail.setCreateTime(DateUtils.getNowDate());
        etDividendDetail.setDividendProportion(Integer.parseInt(cxPlatform.getPlatformServiceFee()));
        etDividendDetail.setDividendAmount(platformServiceFee);
        etDividendDetail.setDividendItem("1,2");
        logger.info("【微信支付回调】平台服务费的分账明细保存 === " + JSON.toJSONString(etDividendDetail));
        int i = dividendDetailService.insertEtDividendDetail(etDividendDetail);
        if(i==0){
            throw new ServiceException("保存分账明细失败");
        }
    }

    /**
     *  退还押金定时任务
     */
    private void refundDeposit(String deposit, EtOrder order, AsUser asUser) {
        Integer autoRefundDeposit = order.getAutoRefundDeposit();

        logger.info("【微信支付回调】进入退还押金");
        if(autoRefundDeposit!=null){
            //创建一个定时器，计算出退还时间后，执行退款操作
            // 往后推autoRefundDeposit小时执行
            scheduledExecutorService.schedule(() -> {
                // 如果退款成功直接返回
                List<EtRefund> refund2 = etRefundMapper.selectEtRefundByOrderNo(order.getOrderNo());
                if(ObjectUtil.isNotNull(refund2) && refund2.size() == 1 && refund2.get(0).getRefundResult().equals(Constants.SUCCESS2)){
                    return;
                }
                logger.info("【微信支付回调】退还押金定时任务开始执行！！！！！！！！");
                // 退款
                Long userId = order.getUserId();
                EtOrder etOrder = new EtOrder();
                etOrder.setUserId(userId);
                etOrder.setPaid(ServiceConstants.ORDER_PAY_STATUS_PAID);
                etOrder.setType(ServiceConstants.ORDER_TYPE_DEPOSIT);
                etOrder.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
                List<EtOrder> etOrders = etOrderMapper.selectEtOrderList(etOrder);
                if (etOrders.size() == 0 || ObjectUtil.isNull(etOrders) ) {
                    throw new ServiceException("押金充值记录不存在");
                }
                Optional<EtOrder> latestOrder = etOrders.stream()
                        .max(Comparator.comparing(EtOrder::getPayTime));
                if (latestOrder.isPresent()) {
                    EtOrder newestOrder = latestOrder.get();
                    logger.info("【微信支付回调】最后一次押金充值记录 : " + JSON.toJSONString(newestOrder));
                    // 处理找到的最新支付时间的订单
                    if(newestOrder.getTotalFee().compareTo(new BigDecimal(deposit))!=0){
                        throw new ServiceException("押金充值记录与当前运营区的押金不同");
                    }
                    String reason = autoRefundDeposit + "个小时后自动退押金";
                    String outRefundNo = IdUtils.getOrderNo("ref");
                    Refund refund = wxPayService.refund(newestOrder,reason, newestOrder.getTotalFee(), outRefundNo);
                    newestOrder.setReason(reason);
                    EtRefund refund1= orderService.createRefund(newestOrder, newestOrder.getTotalFee(), null, null, null, null, refund.getOutRefundNo(),ServiceConstants.REFUND_TYPE_DEPOSIT);
                    int i = etRefundService.insertEtRefund(refund1);
                    if(i>0){
                        logger.info("【自动退款】保存退款对象成功");
                    }
                    // 新增资金流水记录
//                    capitalFlowRecords(newestOrder,ServiceConstants.FLOW_TYPE_DISBURSE,ServiceConstants.ORDER_TYPE_DEPOSIT_REFUND);

                    // 更新用户信息，清除缓存
                    asUser.setBalance(BigDecimal.ZERO);
                    int updateUser = userService.updateUser(asUser);
                    if(updateUser>0){
                        logger.info("【微信支付回调】退还押金,更新用户余额成功！");
                    }
                    logger.info("=================【微信支付回调】退还押金定时任务结束！！！==================");
                } else {
                    throw new ServiceException("没有找到押金充值记录");
                }
            }, autoRefundDeposit , TimeUnit.HOURS);
        }else{
            logger.info("【微信支付回调】还车结算___小时后自动退押金参数为空");
            throw new ServiceException("还车结算___小时后自动退押金参数为空");
        }
    }

    /**
     * 资金流水记录
     * */
    @Override
    public EtCapitalFlow capitalFlowRecords(EtOrder order,String type,String busType,String ownerType,SysUser user,String payType) {
        EtCapitalFlow capitalFlow = new EtCapitalFlow();
        if(ownerType.equals(ServiceConstants.OWNER_TYPE_OPERATOR)){//运营商
            SysDept sysDept = wxPayService.getDeptObjByAreaId(order.getAreaId());
            String separateAccount = sysDept.getSeparateAccount();
            capitalFlow.setAreaId(order.getAreaId());
            capitalFlow.setOrderNo(order.getOrderNo());
            capitalFlow.setOutTradeNo(order.getOutTradeNo());
            capitalFlow.setType(type);
            capitalFlow.setBusType(busType);
            capitalFlow.setOwnerType(ownerType);
            capitalFlow.setOwnerId(sysDept.getDeptId());
            capitalFlow.setOwner(sysDept.getDeptName());
            capitalFlow.setAmount(order.getPayFee());
            String handlingCharge1 = sysDept.getHandlingCharge();
            logger.info("【微信支付回调--保存资金流水记录】 获取到配置手续费==============handlingCharge=====================："+handlingCharge1);
            BigDecimal bigDecimal = new BigDecimal(handlingCharge1).divide(new BigDecimal(1000), 6, BigDecimal.ROUND_HALF_UP);
            logger.info("【微信支付回调--保存资金流水记录】 转换后手续费==============bigDecimal=====================："+bigDecimal);
            BigDecimal handlingCharge = bigDecimal.multiply(order.getPayFee()).setScale(2, BigDecimal.ROUND_HALF_UP);
            logger.info("【微信支付回调--保存资金流水记录】 计算出的手续费==============handlingCharge=====================："+handlingCharge);
            BigDecimal serviceFeeScale = new BigDecimal(sysDept.getPlatformServiceFee()).divide(new BigDecimal(100), 6, BigDecimal.ROUND_HALF_UP);
            BigDecimal platformServiceFee = serviceFeeScale.multiply(order.getPayFee());
            logger.info("【微信支付回调--保存资金流水记录】 计算出的平台服务费==============platformServiceFee=====================："+platformServiceFee);
            capitalFlow.setPlatformServiceFee(platformServiceFee);
            capitalFlow.setHandlingCharge(handlingCharge);//手续费
            if(busType.equals(ServiceConstants.ORDER_TYPE_WITHDRAW)){//提现，需要手续费，不需要平台服务费
                BigDecimal separateAccountFee = order.getPayFee();
                capitalFlow.setPartnerDividend(BigDecimal.ZERO);
                capitalFlow.setAmount(order.getPayFee().subtract(handlingCharge));
                capitalFlow.setPlatformServiceFee(BigDecimal.ZERO);
                capitalFlow.setOperatorDividend(separateAccountFee.negate());
                if(StrUtil.isNotBlank(separateAccount) && separateAccount.equals("N")){
                    capitalFlow.setOperatorBalance(sysDept.getBalance().subtract(separateAccountFee));
                    deptService.changeDeptBalance(separateAccountFee.negate(),sysDept.getDeptId());
                }else{
                    capitalFlow.setOperatorBalance(BigDecimal.ZERO);
                }
                logger.info("【微信支付回调--保存资金流水记录】 ==============支出=====================：");
            }else if(busType.equals(ServiceConstants.ORDER_TYPE_VEHICLE_DAMAGE)){
                BigDecimal separateAccountFee = order.getPayFee();
                capitalFlow.setAmount(order.getPayFee());
                capitalFlow.setHandlingCharge(BigDecimal.ZERO);//手续费
                capitalFlow.setPartnerDividend(BigDecimal.ZERO);// 合伙人分账
                capitalFlow.setPlatformServiceFee(BigDecimal.ZERO);//平台服务费
                capitalFlow.setOperatorDividend(separateAccountFee);
                if(StrUtil.isNotBlank(separateAccount) && separateAccount.equals("N")){
                    capitalFlow.setOperatorBalance(sysDept.getBalance().add(separateAccountFee));
                    deptService.changeDeptBalance(separateAccountFee,sysDept.getDeptId());
                }else{
                    capitalFlow.setOperatorBalance(BigDecimal.ZERO);
                }
            }else {
                logger.info("【微信支付回调--保存资金流水记录】 ==============业务类型=====================："+busType);
                BigDecimal partnerDividend = BigDecimal.ZERO;
                BigDecimal separateAccountFee = order.getPayFee().subtract(handlingCharge).subtract(platformServiceFee);
                logger.info("【微信支付回调--保存资金流水记录】 ==============扣掉手续费和服务费之后的金额，这个金额拿来分账=====================："+separateAccountFee);
                BigDecimal operatorDividend = separateAccountFee;
//                if(sysDept.getIsProfitSharing().equals("true")){//需要分账
//                    logger.info("【微信支付回调--保存资金流水记录】 ==============需要分账====================="+sysDept.getIsProfitSharing());
                //获取所有合伙人列表
                SysUser sysUser = new SysUser();
                sysUser.setUserType("03");
                sysUser.setAreaId(order.getAreaId());
                List<SysUser> sysUsers = userMapper.selectUserList(sysUser);
                double totalDividendProportion = sysUsers.stream()
                        .mapToDouble(SysUser::getDividendProportion)
                        .sum();//算出总的分成比例
                BigDecimal decimal = new BigDecimal(totalDividendProportion).divide(new BigDecimal(100), 2, BigDecimal.ROUND_HALF_UP);
                partnerDividend = operatorDividend.multiply(decimal);
                operatorDividend = operatorDividend.subtract(partnerDividend);
//                }
                logger.info("【微信支付回调--保存资金流水记录】 ==============partnerDividend=====================："+partnerDividend);
                logger.info("【微信支付回调--保存资金流水记录】 ==============operatorDividend=====================："+operatorDividend);
                if(type.equals(ServiceConstants.FLOW_TYPE_INCOME)){
                    capitalFlow.setPartnerDividend(partnerDividend);
                    capitalFlow.setOperatorDividend(operatorDividend);
                    if(StrUtil.isNotBlank(separateAccount) && separateAccount.equals("N")){
                        capitalFlow.setOperatorBalance(sysDept.getBalance().add(operatorDividend));
                        deptService.changeDeptBalance(operatorDividend,sysDept.getDeptId());
                    }else{
                        capitalFlow.setOperatorBalance(BigDecimal.ZERO);
                    }
                    logger.info("【微信支付回调--保存资金流水记录】 ==============收入=====================：");
                }else{
                    capitalFlow.setPartnerDividend(partnerDividend.negate());
                    capitalFlow.setOperatorDividend(operatorDividend.negate());
                    if(StrUtil.isNotBlank(separateAccount) && separateAccount.equals("N")){
                        capitalFlow.setOperatorBalance(sysDept.getBalance().subtract(operatorDividend));
                        deptService.changeDeptBalance(operatorDividend.negate(),sysDept.getDeptId());
                    }else{
                        capitalFlow.setOperatorBalance(BigDecimal.ZERO);
                    }
                    logger.info("【微信支付回调--保存资金流水记录】 ==============支出=====================：");
                }
            }
            capitalFlow.setPayType(payType);
            capitalFlow.setCreateTime(DateUtils.getNowDate());
            AsDevice device = asDeviceMapper.selectAsDeviceBySn(order.getSn());
            if (device != null) {
                capitalFlow.setModelId(device.getModelId());
            }
            logger.info("【微信支付回调】保存资金流水记录对象 : " + JSON.toJSONString(capitalFlow));
            int i = etCapitalFlowService.insertEtCapitalFlow(capitalFlow);
            if(i==0){
                throw new ServiceException("保存资金流水记录失败");
            }else {
                logger.info("【微信支付回调】保存资金流水记录成功");
            }
        }else{
            logger.info("【合伙人记录资金流水】");
            capitalFlow.setAreaId(order.getAreaId());
            capitalFlow.setOrderNo(order.getOrderNo());
            capitalFlow.setOutTradeNo(order.getOutTradeNo());
            capitalFlow.setType(type);
            capitalFlow.setBusType(busType);
            capitalFlow.setOwnerType(ownerType);
            capitalFlow.setOwnerId(user.getUserId());
            capitalFlow.setOwner(user.getUserName());
            capitalFlow.setAmount(order.getPayFee());
            capitalFlow.setHandlingCharge(BigDecimal.ZERO);//手续费 0
            capitalFlow.setPlatformServiceFee(BigDecimal.ZERO);//平台服务费 0
            if(busType.equals(ServiceConstants.ORDER_TYPE_WITHDRAW)){//提现，需要手续费，不需要平台服务费
                BigDecimal separateAccountFee = order.getPayFee();
                capitalFlow.setPartnerDividend(BigDecimal.ZERO);
                capitalFlow.setOperatorDividend(separateAccountFee.negate());
                capitalFlow.setOperatorBalance(user.getBalance().subtract(separateAccountFee));
                userMapper.changeUserBalance(separateAccountFee.negate(),user.getUserId());
                logger.info("【保存资金流水记录】 ==============支出=====================：");
            }else{
                logger.info("【微信支付回调(合伙人)--保存资金流水记录】 ==============业务类型===骑行收入==================："+busType);
                BigDecimal partnerDividend = order.getPayFee();
                logger.info("【微信支付回调(合伙人)--保存资金流水记录】 ==============合伙人分账partnerDividend=====================："+partnerDividend);
                if(type.equals(ServiceConstants.FLOW_TYPE_INCOME)){
                    capitalFlow.setPartnerDividend(partnerDividend);
                    capitalFlow.setOperatorDividend(order.getPayFee());
                    capitalFlow.setOperatorBalance(user.getBalance().add(partnerDividend));
                    userMapper.changeUserBalance(partnerDividend,user.getUserId());
                    logger.info("【微信支付回调(合伙人)--保存资金流水记录】 ==============收入=====================：");
                }else{
                    capitalFlow.setPartnerDividend(partnerDividend.negate());
                    capitalFlow.setOperatorDividend(order.getPayFee());
                    capitalFlow.setOperatorBalance(user.getBalance().subtract(partnerDividend));
                    userMapper.changeUserBalance(partnerDividend.negate(),user.getUserId());
                    logger.info("【微信支付回调(合伙人)--保存资金流水记录】 ==============支出=====================：");
                }
            }
            capitalFlow.setPayType(payType);
            capitalFlow.setCreateTime(DateUtils.getNowDate());
            AsDevice device = asDeviceMapper.selectAsDeviceBySn(order.getSn());
            if (device != null) {
                capitalFlow.setModelId(device.getModelId());
            }
            logger.info("【微信支付回调】保存资金流水记录对象 : " + JSON.toJSONString(capitalFlow));
            int i = etCapitalFlowService.insertEtCapitalFlow(capitalFlow);
            if(i==0){
                throw new ServiceException("保存资金流水记录失败");
            }else {
                logger.info("【微信支付回调】保存资金流水记录成功");
            }
        }
        return capitalFlow;
    }

    /**
     * 资金流水记录
     * */
    @Override
    public EtCapitalFlow capitalFlowRecordsByRepair(EtOrder order,String type,String busType,String ownerType,SysUser user,String payType) {
        EtCapitalFlow capitalFlow = new EtCapitalFlow();
        if(ownerType.equals(ServiceConstants.OWNER_TYPE_OPERATOR)){//运营商
            SysDept sysDept = wxPayService.getDeptObjByAreaId(order.getAreaId());
            String separateAccount = sysDept.getSeparateAccount();
            capitalFlow.setAreaId(order.getAreaId());
            capitalFlow.setOrderNo(order.getOrderNo());
            capitalFlow.setOutTradeNo(order.getOutTradeNo());
            capitalFlow.setType(type);
            capitalFlow.setBusType(busType);
            capitalFlow.setOwnerType(ownerType);
            capitalFlow.setOwnerId(sysDept.getDeptId());
            capitalFlow.setOwner(sysDept.getDeptName());
            capitalFlow.setAmount(order.getPayFee());
            String handlingCharge1 = sysDept.getHandlingCharge();
            logger.info("【微信支付回调--保存资金流水记录】 获取到配置手续费==============handlingCharge=====================："+handlingCharge1);
            BigDecimal bigDecimal = new BigDecimal(handlingCharge1).divide(new BigDecimal(1000), 6, BigDecimal.ROUND_HALF_UP);
            logger.info("【微信支付回调--保存资金流水记录】 转换后手续费==============bigDecimal=====================："+bigDecimal);
            BigDecimal handlingCharge = bigDecimal.multiply(order.getPayFee()).setScale(2, BigDecimal.ROUND_HALF_UP);
            logger.info("【微信支付回调--保存资金流水记录】 计算出的手续费==============handlingCharge=====================："+handlingCharge);
            BigDecimal serviceFeeScale = new BigDecimal(sysDept.getPlatformServiceFee()).divide(new BigDecimal(100), 6, BigDecimal.ROUND_HALF_UP);
            BigDecimal platformServiceFee = serviceFeeScale.multiply(order.getPayFee());
            logger.info("【微信支付回调--保存资金流水记录】 计算出的平台服务费==============platformServiceFee=====================："+platformServiceFee);
            capitalFlow.setPlatformServiceFee(platformServiceFee);
            capitalFlow.setHandlingCharge(handlingCharge);//手续费
            if(busType.equals(ServiceConstants.ORDER_TYPE_WITHDRAW)){//提现，需要手续费，不需要平台服务费
                BigDecimal separateAccountFee = order.getPayFee();
                capitalFlow.setPartnerDividend(BigDecimal.ZERO);
                capitalFlow.setAmount(order.getPayFee().subtract(handlingCharge));
                capitalFlow.setPlatformServiceFee(BigDecimal.ZERO);
                capitalFlow.setOperatorDividend(separateAccountFee.negate());
                if(StrUtil.isNotBlank(separateAccount) && separateAccount.equals("N")){
                    capitalFlow.setOperatorBalance(sysDept.getBalance().subtract(separateAccountFee));
                    deptService.changeDeptBalance(separateAccountFee.negate(),sysDept.getDeptId());
                }else{
                    capitalFlow.setOperatorBalance(BigDecimal.ZERO);
                }
                logger.info("【微信支付回调--保存资金流水记录】 ==============支出=====================：");
            }else if(busType.equals(ServiceConstants.ORDER_TYPE_VEHICLE_DAMAGE)){
                BigDecimal separateAccountFee = order.getPayFee();
                capitalFlow.setAmount(order.getPayFee());
                capitalFlow.setHandlingCharge(BigDecimal.ZERO);//手续费
                capitalFlow.setPartnerDividend(BigDecimal.ZERO);// 合伙人分账
                capitalFlow.setPlatformServiceFee(BigDecimal.ZERO);//平台服务费
                capitalFlow.setOperatorDividend(separateAccountFee);
                if(StrUtil.isNotBlank(separateAccount) && separateAccount.equals("N")){
                    capitalFlow.setOperatorBalance(sysDept.getBalance().add(separateAccountFee));
                    deptService.changeDeptBalance(separateAccountFee,sysDept.getDeptId());
                }else{
                    capitalFlow.setOperatorBalance(BigDecimal.ZERO);
                }
            }else {
                logger.info("【微信支付回调--保存资金流水记录】 ==============业务类型=====================："+busType);
                BigDecimal partnerDividend = BigDecimal.ZERO;
                BigDecimal separateAccountFee = order.getPayFee().subtract(handlingCharge).subtract(platformServiceFee);
                logger.info("【微信支付回调--保存资金流水记录】 ==============扣掉手续费和服务费之后的金额，这个金额拿来分账=====================："+separateAccountFee);
                BigDecimal operatorDividend = separateAccountFee;
//                if(sysDept.getIsProfitSharing().equals("true")){//需要分账
//                    logger.info("【微信支付回调--保存资金流水记录】 ==============需要分账====================="+sysDept.getIsProfitSharing());
                //获取所有合伙人列表
                SysUser sysUser = new SysUser();
                sysUser.setUserType("03");
                sysUser.setAreaId(order.getAreaId());
                List<SysUser> sysUsers = userMapper.selectUserList(sysUser);
                double totalDividendProportion = sysUsers.stream()
                        .mapToDouble(SysUser::getDividendProportion)
                        .sum();//算出总的分成比例
                BigDecimal decimal = new BigDecimal(totalDividendProportion).divide(new BigDecimal(100), 2, BigDecimal.ROUND_HALF_UP);
                partnerDividend = operatorDividend.multiply(decimal);
                operatorDividend = operatorDividend.subtract(partnerDividend);
//                }
                logger.info("【微信支付回调--保存资金流水记录】 ==============partnerDividend=====================："+partnerDividend);
                logger.info("【微信支付回调--保存资金流水记录】 ==============operatorDividend=====================："+operatorDividend);
                if(type.equals(ServiceConstants.FLOW_TYPE_INCOME)){
                    capitalFlow.setPartnerDividend(partnerDividend);
                    capitalFlow.setOperatorDividend(operatorDividend);
                    if(StrUtil.isNotBlank(separateAccount) && separateAccount.equals("N")){
                        capitalFlow.setOperatorBalance(sysDept.getBalance().add(operatorDividend));
                        deptService.changeDeptBalance(operatorDividend,sysDept.getDeptId());
                    }else{
                        capitalFlow.setOperatorBalance(BigDecimal.ZERO);
                    }
                    logger.info("【微信支付回调--保存资金流水记录】 ==============收入=====================：");
                }else{
                    capitalFlow.setPartnerDividend(partnerDividend.negate());
                    capitalFlow.setOperatorDividend(operatorDividend.negate());
                    if(StrUtil.isNotBlank(separateAccount) && separateAccount.equals("N")){
                        capitalFlow.setOperatorBalance(sysDept.getBalance().subtract(operatorDividend));
                        deptService.changeDeptBalance(operatorDividend.negate(),sysDept.getDeptId());
                    }else{
                        capitalFlow.setOperatorBalance(BigDecimal.ZERO);
                    }
                    logger.info("【微信支付回调--保存资金流水记录】 ==============支出=====================：");
                }
            }
            capitalFlow.setPayType(payType);
            capitalFlow.setCreateTime(DateUtils.getNowDate());
            AsDevice device = asDeviceMapper.selectAsDeviceBySn(order.getSn());
            if (device != null) {
                capitalFlow.setModelId(device.getModelId());
            }
            logger.info("【微信支付回调】保存资金流水记录对象 : " + JSON.toJSONString(capitalFlow));
            int i = etCapitalFlowService.insertEtCapitalFlow(capitalFlow);
            if(i==0){
                throw new ServiceException("保存资金流水记录失败");
            }else {
                logger.info("【微信支付回调】保存资金流水记录成功");
            }
        }else{
            logger.info("【合伙人记录资金流水】");
            capitalFlow.setAreaId(order.getAreaId());
            capitalFlow.setOrderNo(order.getOrderNo());
            capitalFlow.setOutTradeNo(order.getOutTradeNo());
            capitalFlow.setType(type);
            capitalFlow.setBusType(busType);
            capitalFlow.setOwnerType(ownerType);
            capitalFlow.setOwnerId(user.getUserId());
            capitalFlow.setOwner(user.getUserName());
            capitalFlow.setAmount(order.getPayFee());
            capitalFlow.setHandlingCharge(BigDecimal.ZERO);//手续费 0
            capitalFlow.setPlatformServiceFee(BigDecimal.ZERO);//平台服务费 0
            if(busType.equals(ServiceConstants.ORDER_TYPE_WITHDRAW)){//提现，需要手续费，不需要平台服务费
                BigDecimal separateAccountFee = order.getPayFee();
                capitalFlow.setPartnerDividend(BigDecimal.ZERO);
                capitalFlow.setOperatorDividend(separateAccountFee.negate());
                capitalFlow.setOperatorBalance(user.getBalance().subtract(separateAccountFee));
                userMapper.changeUserBalance(separateAccountFee.negate(),user.getUserId());
                logger.info("【保存资金流水记录】 ==============支出=====================：");
            }else{
                logger.info("【微信支付回调(合伙人)--保存资金流水记录】 ==============业务类型===骑行收入==================："+busType);
                BigDecimal partnerDividend = order.getPayFee();
                logger.info("【微信支付回调(合伙人)--保存资金流水记录】 ==============合伙人分账partnerDividend=====================："+partnerDividend);
                if(type.equals(ServiceConstants.FLOW_TYPE_INCOME)){
                    capitalFlow.setPartnerDividend(partnerDividend);
                    capitalFlow.setOperatorDividend(order.getPayFee());
                    capitalFlow.setOperatorBalance(user.getBalance().add(partnerDividend));
                    userMapper.changeUserBalance(partnerDividend,user.getUserId());
                    logger.info("【微信支付回调(合伙人)--保存资金流水记录】 ==============收入=====================：");
                }else{
                    capitalFlow.setPartnerDividend(partnerDividend.negate());
                    capitalFlow.setOperatorDividend(order.getPayFee());
                    capitalFlow.setOperatorBalance(user.getBalance().subtract(partnerDividend));
                    userMapper.changeUserBalance(partnerDividend.negate(),user.getUserId());
                    logger.info("【微信支付回调(合伙人)--保存资金流水记录】 ==============支出=====================：");
                }
            }
            capitalFlow.setPayType(payType);
            capitalFlow.setCreateTime(order.getPayTime());
            AsDevice device = asDeviceMapper.selectAsDeviceBySn(order.getSn());
            if (device != null) {
                capitalFlow.setModelId(device.getModelId());
            }
            logger.info("【微信支付回调】保存资金流水记录对象 : " + JSON.toJSONString(capitalFlow));
            int i = etCapitalFlowService.insertEtCapitalFlow(capitalFlow);
            if(i==0){
                throw new ServiceException("保存资金流水记录失败");
            }else {
                logger.info("【微信支付回调】保存资金流水记录成功");
            }
        }
        return capitalFlow;
    }

    /**
     * 资金流水记录
     * */
    @Override
    public int capitalFlowRecords2(EtOrder order,String type,String busType,String ownerType,SysDept sysDept) {
        EtCapitalFlow capitalFlow = new EtCapitalFlow();
        capitalFlow.setAreaId(order.getAreaId());
        capitalFlow.setOrderNo(order.getOrderNo());
        capitalFlow.setOutTradeNo(order.getOutTradeNo());
        capitalFlow.setType(type);
        capitalFlow.setBusType(busType);
        capitalFlow.setOwnerType(ownerType);
        capitalFlow.setOwnerId(sysDept.getDeptId());
        capitalFlow.setOwner(sysDept.getDeptName());

        String handlingChargeType = sysDept.getHandlingChargeType();
        String handlingCharge1 = sysDept.getWithdrawHandlingCharge();
        BigDecimal handlingCharge;
        logger.info("【保存资金流水记录】 获取到配置手续费==============handlingCharge=====================："+handlingCharge1);
        if(handlingChargeType.equals(ServiceConstants.HANDLING_CHARGE_TYPE_PERCENT)){
            BigDecimal bigDecimal = new BigDecimal(handlingCharge1).divide(new BigDecimal(1000), 6, BigDecimal.ROUND_HALF_UP);
            logger.info("【保存资金流水记录】 按千分比--转换后手续费==============bigDecimal=====================："+bigDecimal);
            handlingCharge = bigDecimal.multiply(order.getPayFee()).setScale(2, BigDecimal.ROUND_HALF_UP);
            logger.info("【保存资金流水记录】 按千分比--计算出的手续费==============handlingCharge=====================："+handlingCharge);
        }else{
            handlingCharge = new BigDecimal(handlingCharge1);
            logger.info("【保存资金流水记录】 按每笔--计算出的手续费==============handlingCharge=====================："+handlingCharge);
        }
        BigDecimal serviceFeeScale = new BigDecimal(sysDept.getPlatformServiceFee()).divide(new BigDecimal(100), 6, BigDecimal.ROUND_HALF_UP);
        BigDecimal platformServiceFee = serviceFeeScale.multiply(order.getPayFee());
        logger.info("【保存资金流水记录】 计算出的平台服务费==============platformServiceFee=====================："+platformServiceFee);
        capitalFlow.setPlatformServiceFee(platformServiceFee);
        capitalFlow.setHandlingCharge(handlingCharge);//手续费
        capitalFlow.setAmount(order.getPayFee().subtract(handlingCharge));
        BigDecimal separateAccountFee = order.getPayFee();
        capitalFlow.setPartnerDividend(BigDecimal.ZERO);
        capitalFlow.setPlatformServiceFee(BigDecimal.ZERO);
        if(type.equals(ServiceConstants.FLOW_TYPE_INCOME)){
            capitalFlow.setStatus(ServiceConstants.FLOW_STATUS_REJECT);
            capitalFlow.setOperatorDividend(separateAccountFee);
            capitalFlow.setOperatorBalance(sysDept.getBalance().add(separateAccountFee));
            deptService.changeDeptBalance(separateAccountFee,sysDept.getDeptId());
            logger.info("【保存资金流水记录】 ==============收入=====================：");
        }else{
            capitalFlow.setStatus(ServiceConstants.FLOW_STATUS_APPLY);
            capitalFlow.setOperatorDividend(separateAccountFee.negate());
            capitalFlow.setOperatorBalance(sysDept.getBalance().subtract(separateAccountFee));
            deptService.changeDeptBalance(separateAccountFee.negate(),sysDept.getDeptId());
            logger.info("【保存资金流水记录】 ==============支出=====================：");
        }
        capitalFlow.setPayType(ServiceConstants.PAY_TYPE_WX);
        capitalFlow.setCreateTime(DateUtils.getNowDate());
        AsDevice device = asDeviceMapper.selectAsDeviceBySn(order.getSn());
        if (device != null) {
            capitalFlow.setModelId(device.getModelId());
        }
        logger.info("【保存资金流水】保存资金流水记录对象 : " + JSON.toJSONString(capitalFlow));
        int i = etCapitalFlowService.insertEtCapitalFlow(capitalFlow);
        return i;
    }

    /**
     * 资金流水记录
     * */
    @Override
    public int capitalFlowRecordsByAppUser(EtOrder order,String type,String busType,String ownerType,SysDept sysDept) {
        EtCapitalFlow capitalFlow = new EtCapitalFlow();
        capitalFlow.setAreaId(order.getAreaId());
        capitalFlow.setOrderNo(order.getOrderNo());
        capitalFlow.setOutTradeNo(order.getOutTradeNo());
        capitalFlow.setType(type);
        capitalFlow.setBusType(busType);
        capitalFlow.setOwnerType(ownerType);
        capitalFlow.setOwnerId(sysDept.getDeptId());
        capitalFlow.setOwner(sysDept.getDeptName());

        String handlingChargeType = sysDept.getHandlingChargeType();
        String handlingCharge1 = sysDept.getWithdrawHandlingCharge();
        BigDecimal handlingCharge;
        logger.info("【保存资金流水记录】 获取到配置手续费==============handlingCharge=====================："+handlingCharge1);
        if(handlingChargeType.equals(ServiceConstants.HANDLING_CHARGE_TYPE_PERCENT)){
            BigDecimal bigDecimal = new BigDecimal(handlingCharge1).divide(new BigDecimal(1000), 6, BigDecimal.ROUND_HALF_UP);
            logger.info("【保存资金流水记录】 按千分比--转换后手续费==============bigDecimal=====================："+bigDecimal);
            handlingCharge = bigDecimal.multiply(order.getPayFee()).setScale(2, BigDecimal.ROUND_HALF_UP);
            logger.info("【保存资金流水记录】 按千分比--计算出的手续费==============handlingCharge=====================："+handlingCharge);
        }else{
            handlingCharge = new BigDecimal(handlingCharge1);
            logger.info("【保存资金流水记录】 按每笔--计算出的手续费==============handlingCharge=====================："+handlingCharge);
        }
        BigDecimal serviceFeeScale = new BigDecimal(sysDept.getPlatformServiceFee()).divide(new BigDecimal(100), 6, BigDecimal.ROUND_HALF_UP);
        BigDecimal platformServiceFee = serviceFeeScale.multiply(order.getPayFee());
        logger.info("【保存资金流水记录】 计算出的平台服务费==============platformServiceFee=====================："+platformServiceFee);
        capitalFlow.setPlatformServiceFee(platformServiceFee);
        capitalFlow.setHandlingCharge(handlingCharge);//手续费
        capitalFlow.setAmount(order.getPayFee().subtract(handlingCharge));
        BigDecimal separateAccountFee = order.getPayFee();
        capitalFlow.setPartnerDividend(BigDecimal.ZERO);
        capitalFlow.setPlatformServiceFee(BigDecimal.ZERO);
        if(type.equals(ServiceConstants.FLOW_TYPE_INCOME)){
            capitalFlow.setStatus(ServiceConstants.FLOW_STATUS_REJECT);
            capitalFlow.setOperatorDividend(separateAccountFee);
            capitalFlow.setOperatorBalance(sysDept.getBalance().add(separateAccountFee));
            deptService.changeDeptBalance(separateAccountFee,sysDept.getDeptId());
            logger.info("【保存资金流水记录】 ==============收入=====================：");
        }else{
            capitalFlow.setStatus(ServiceConstants.FLOW_STATUS_APPLY);
            capitalFlow.setOperatorDividend(separateAccountFee.negate());
            capitalFlow.setOperatorBalance(sysDept.getBalance().subtract(separateAccountFee));
            deptService.changeDeptBalance(separateAccountFee.negate(),sysDept.getDeptId());
            logger.info("【保存资金流水记录】 ==============支出=====================：");
        }
        capitalFlow.setPayType(ServiceConstants.PAY_TYPE_WX);
        capitalFlow.setCreateTime(DateUtils.getNowDate());
        AsDevice device = asDeviceMapper.selectAsDeviceBySn(order.getSn());
        if (device != null) {
            capitalFlow.setModelId(device.getModelId());
        }
        logger.info("【保存资金流水】保存资金流水记录对象 : " + JSON.toJSONString(capitalFlow));
        int i = etCapitalFlowService.insertEtCapitalFlow(capitalFlow);
        return i;
    }

    /**
     * 微信退款回调
     */
    @Override
    @Transactional
    public void weChatRefund(HttpServletRequest request) {
        String body = HttpUtils.getBody(request);
        EtCallbackLog etCallbackLog = new EtCallbackLog();
        etCallbackLog.setBody2(body);
        etCallbackLog.setType("2");
        logger.info("【微信退款回调】接收对象 : " + JSON.toJSONString(body));
        // 解析通知数据
        Notification notification = JSON.parseObject(body, Notification.class);
        logger.info("【微信退款回调】转换成notification: " + JSON.toJSONString(notification));
        // 退款成功通知
        if (NotifyEventType.REFUND_SUCCESS.getValue().equals(notification.getEventType())) {
            // 验签、解密并转换成 RefundNotification
            RefundNotification refundNotification = checkAndParse(request, body, RefundNotification.class);
            logger.info("【微信退款回调】转换成RefundNotification: " + JSON.toJSONString(refundNotification));
            etCallbackLog.setBody(JSON.toJSONString(refundNotification));
            if (Status.SUCCESS.equals(refundNotification.getRefundStatus())) {
                // 退款成功后  改变 etRefundService
                String outRefundNo = refundNotification.getOutRefundNo();
                EtRefund etRefund = new EtRefund();
                etRefund.setRefundNo(outRefundNo);
                etRefund.setRefundResult(Constants.SUCCESS2);
                int i = etRefundService.updateEtRefundByRefundNo(etRefund);
                if(i==0){
                    logger.error("【微信退款回调】更新退款单失败");
                    throw new ServiceException("【微信退款回调】更新退款单失败");
                }
                EtRefund refund = etRefundService.selectEtRefundByRefundNo(outRefundNo);
                EtOrder order = orderService.selectEtOrderByOrderNo(refund.getOrderNo());
                // 新增资金流水记录
//                if(order.getOrderNo().contains("qx")){
//                    capitalFlowRecords(order,ServiceConstants.FLOW_TYPE_DISBURSE,ServiceConstants.ORDER_TYPE_RIDING_REFUND);
//                }
                logger.info("【微信退款回调】全部结束！！！！！");
            }
        }
        //异步保存回调日志
        asynchronousSaveCallbackLog(etCallbackLog);
    }

    /**
     * 验签并解析
     * @param request 请求
     * @param body 请求体
     * @param clazz 返回值类型
     */
    private <T> T checkAndParse(HttpServletRequest request, String body, Class<T> clazz) {
        // 构造 RequestParam
        RequestParam requestParam = new RequestParam.Builder()
                .serialNumber(request.getHeader("Wechatpay-Serial"))
                .nonce(request.getHeader("Wechatpay-Nonce"))
                .signature(request.getHeader("Wechatpay-Signature"))
                .timestamp(request.getHeader("Wechatpay-Timestamp"))
                .body(body)
                .build();
        // 验签
        String wechatpaySerial = request.getHeader("Wechatpay-Serial");
        NotificationParser notificationParser = getNotificationParser(wechatpaySerial);
        return notificationParser.parse(requestParam, clazz);
    }


    public NotificationParser getNotificationParser(String wechatpaySerial) {
        // 根据证书序列号获取运营商
        SysDept sysDept = deptService.selectEtOperatingAreaBySerialNumber(wechatpaySerial);
        if(ObjectUtil.isNull(sysDept)){
            throw new ServiceException("【微信支付回调】根据证书序列号获取运营商失败");
        }else{
            NotificationConfig config = new RSAAutoCertificateConfig.Builder()
                    .merchantId(sysDept.getMerchantId())
                    .privateKeyFromPath(sysDept.getPrivateKeyPath())
                    .merchantSerialNumber(sysDept.getMerchantSerialNumber())
                    .apiV3Key(sysDept.getApiV3Key())
                    .build();
            return new NotificationParser(config);
        }
    }
}
