package com.ruoyi.system.service.impl;

import java.util.List;

import cn.hutool.core.util.ObjectUtil;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.domain.*;
import com.ruoyi.system.domain.vo.AdminOrderVo;
import com.ruoyi.system.mapper.AsDeviceMapper;
import com.ruoyi.system.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.EtAdminOrderMapper;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

/**
 * 系统工单Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-05-05
 */
@Slf4j
@Service
public class EtAdminOrderServiceImpl implements IEtAdminOrderService
{
    @Resource
    private EtAdminOrderMapper etAdminOrderMapper;

    @Resource
    private ISysUserService userService;

    @Autowired
    private IEtFaultService etFaultService;

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private IAsUserService asUserService;

    @Autowired
    private IEtAdminOrderHistoryService etAdminOrderHistoryService;

    @Resource
    private AsDeviceMapper asDeviceMapper;

    /**
     * 查询系统工单
     *
     * @param id 系统工单主键
     * @return 系统工单
     */
    @Override
    public EtAdminOrder selectEtAdminOrderById(Long id)
    {
        EtAdminOrder adminOrder = etAdminOrderMapper.selectEtAdminOrderById(id);
        AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(adminOrder.getSn());
        if(ObjectUtil.isNotNull(asDevice)){
            adminOrder.setDevice(asDevice);
        }
        EtFault etFault = etFaultService.selectEtFaultByFaultId(adminOrder.getFaultId());
        if(ObjectUtil.isNotNull(etFault)){
            AsUser asUser = asUserService.selectUserById(etFault.getUserId());
            if(ObjectUtil.isNotNull(asUser)){
                etFault.setUserName(asUser.getUserName());
            }
            adminOrder.setFault(etFault);
        }
        SysUser sysUser = userService.selectUserById(adminOrder.getAdminId());
        if(ObjectUtil.isNotNull(sysUser)){
            adminOrder.setAdminName(sysUser.getUserName());
        }
        return adminOrder;
    }

    /**
     * 查询系统工单列表
     *
     * @param etAdminOrder 系统工单
     * @return 系统工单
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<EtAdminOrder> selectEtAdminOrderList(EtAdminOrder etAdminOrder)
    {
        List<EtAdminOrder> etAdminOrders = etAdminOrderMapper.selectEtAdminOrderList(etAdminOrder);
        for (EtAdminOrder adminOrder : etAdminOrders){
//            SysUser sysUser = userService.selectUserById(adminOrder.getAdminId());
//            if(ObjectUtil.isNotNull(sysUser)){
//                adminOrder.setAdminName(sysUser.getUserName());
//                adminOrder.setPhone(sysUser.getPhonenumber());
//            }
            EtFault etFault = etFaultService.selectEtFaultByFaultId(adminOrder.getFaultId());
            if(ObjectUtil.isNotNull(etFault)){
                adminOrder.setFaultType(etFault.getType());
                adminOrder.setFaultTypes(etFault.getTypes());
                adminOrder.setFaultTypeStr(etFault.getTypeStr());
            }
            AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(adminOrder.getSn());
            adminOrder.setLatitude(asDevice.getLatitude());
            adminOrder.setLongitude(asDevice.getLongitude());
        }
        return etAdminOrders;
    }

    /**
     * 新增系统工单
     *
     * @param etAdminOrder 系统工单
     * @return 结果
     */
    @Override
    public int insertEtAdminOrder(EtAdminOrder etAdminOrder)
    {
        etAdminOrder.setCreateTime(DateUtils.getNowDate());
        return etAdminOrderMapper.insertEtAdminOrder(etAdminOrder);
    }

    /**
     * 修改系统工单
     *
     * @param etAdminOrder 系统工单
     * @return 结果
     */
    @Override
    public int updateEtAdminOrder(EtAdminOrder etAdminOrder)
    {
        etAdminOrder.setStatus(ServiceConstants.REPLACEMENT_ELECTRICITY_STATUS_COMPLETED);
        return etAdminOrderMapper.updateEtAdminOrder(etAdminOrder);
    }


    /**
     * 维修处理
     *
     * @param etAdminOrder 系统工单
     * @return 结果
     */
    @Transactional
    @Override
    public int repairHandle(EtAdminOrder etAdminOrder) {
        etAdminOrder.setStatus(ServiceConstants.REPLACEMENT_ELECTRICITY_STATUS_COMPLETED);
        int i1 = etAdminOrderMapper.updateEtAdminOrder(etAdminOrder);
        if(i1>0){
            log.info("维修处理成功");
        }
        /**新建工单履历记录*/
        SysUser sysUser = sysUserService.selectUserByUserName("wx");//获取用户名为“wx”的维修人员
        EtAdminOrderHistory etAdminOrderHistory = new EtAdminOrderHistory();
        etAdminOrderHistory.setInfo("审核通过");
        etAdminOrderHistory.setType(ServiceConstants.ADMIN_ORDER_TYPE_REPAIR);
        etAdminOrderHistory.setOperationType(ServiceConstants.OPERATION_TYPE_SUBMIT);
        etAdminOrderHistory.setOrderNo(etAdminOrder.getOrderNo());
        etAdminOrderHistory.setUserId(sysUser.getUserId());
        etAdminOrderHistory.setUserName(sysUser.getUserName());
        etAdminOrderHistory.setStatus(ServiceConstants.REPLACEMENT_ELECTRICITY_STATUS_REPAIR);
        int i = etAdminOrderHistoryService.insertEtAdminOrderHistory(etAdminOrderHistory);
        if(i>0){
            log.info("工单履历记录成功");
        }
        return 1;
    }

    /**
     * 批量删除系统工单
     *
     * @param ids 需要删除的系统工单主键
     * @return 结果
     */
    @Override
    public int deleteEtAdminOrderByIds(Long[] ids)
    {
        return etAdminOrderMapper.deleteEtAdminOrderByIds(ids);
    }

    /**
     * 删除系统工单信息
     *
     * @param id 系统工单主键
     * @return 结果
     */
    @Override
    public int deleteEtAdminOrderById(Long id)
    {
        return etAdminOrderMapper.deleteEtAdminOrderById(id);
    }

    /**
     * 我的工单数
     */
    @Override
    public AdminOrderVo selectEtAdminOrderNum(EtAdminOrder etAdminOrder) {
        AdminOrderVo adminOrderVo = new AdminOrderVo();
        adminOrderVo.setRepairNum(etAdminOrderMapper.selectRepairNum(etAdminOrder));
        adminOrderVo.setPowerReplacementNum(etAdminOrderMapper.selectPowerReplacementNum(etAdminOrder));
        adminOrderVo.setAllNum(adminOrderVo.getRepairNum()+adminOrderVo.getPowerReplacementNum());
        return adminOrderVo;
    }

    /**
     * 该sn是否有换电单
     * @param sn
     * @return
     */
    @Override
    public Boolean checkOrderUnique(String sn) {
        return etAdminOrderMapper.checkOrderUnique(sn)>0;
    }

}
