package com.ruoyi.system.service.impl;

import java.util.List;
import com.ruoyi.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.EtCouponClaimLogMapper;
import com.ruoyi.system.domain.EtCouponUserLog;
import com.ruoyi.system.service.IEtCouponClaimLogService;

/**
 * 优惠券领取记录Service业务层处理
 *
 * @author qzz
 * @date 2024-08-08
 */
@Service
public class EtCouponClaimLogServiceImpl implements IEtCouponClaimLogService
{
    @Autowired
    private EtCouponClaimLogMapper etCouponClaimLogMapper;

    /**
     * 查询优惠券领取记录
     *
     * @param claimId 优惠券领取记录主键
     * @return 优惠券领取记录
     */
    @Override
    public EtCouponUserLog selectEtCouponClaimLogByLogId(Long claimId)
    {
        return etCouponClaimLogMapper.selectEtCouponClaimLogByLogId(claimId);
    }

    /**
     * 查询优惠券领取记录列表
     *
     * @param etCouponUserLog 优惠券领取记录
     * @return 优惠券领取记录
     */
    @Override
    public List<EtCouponUserLog> selectEtCouponClaimLogList(EtCouponUserLog etCouponUserLog)
    {
        return etCouponClaimLogMapper.selectEtCouponClaimLogList(etCouponUserLog);
    }

    /**
     * 新增优惠券领取记录
     *
     * @param etCouponUserLog 优惠券领取记录
     * @return 结果
     */
    @Override
    public int insertEtCouponClaimLog(EtCouponUserLog etCouponUserLog)
    {
        etCouponUserLog.setCreateTime(DateUtils.getNowDate());
        return etCouponClaimLogMapper.insertEtCouponClaimLog(etCouponUserLog);
    }

    /**
     * 修改优惠券领取记录
     *
     * @param etCouponUserLog 优惠券领取记录
     * @return 结果
     */
    @Override
    public int updateEtCouponClaimLog(EtCouponUserLog etCouponUserLog)
    {
        return etCouponClaimLogMapper.updateEtCouponClaimLog(etCouponUserLog);
    }

    /**
     * 批量删除优惠券领取记录
     *
     * @param claimIds 需要删除的优惠券领取记录主键
     * @return 结果
     */
    @Override
    public int deleteEtCouponClaimLogByLogIds(Long[] claimIds)
    {
        return etCouponClaimLogMapper.deleteEtCouponClaimLogByLogIds(claimIds);
    }

    /**
     * 删除优惠券领取记录信息
     *
     * @param claimId 优惠券领取记录主键
     * @return 结果
     */
    @Override
    public int deleteEtCouponClaimLogByLogId(Long claimId)
    {
        return etCouponClaimLogMapper.deleteEtCouponClaimLogByLogId(claimId);
    }

    /**
     * 我的骑行卡(未使用和已过期)
     */
    @Override
    public List<EtCouponUserLog> selectCouponListByUserId(Long userId) {
    	return etCouponClaimLogMapper.selectCouponListByUserId(userId);
    }
}
