package com.ruoyi.system.service.impl;

import java.util.List;

import cn.hutool.core.util.ObjectUtil;
import com.ruoyi.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.EtDividendDetailMapper;
import com.ruoyi.system.domain.EtDividendDetail;
import com.ruoyi.system.service.IEtDividendDetailService;

import javax.annotation.Resource;

/**
 * 分账明细Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-05-27
 */
@Service
public class EtDividendDetailServiceImpl implements IEtDividendDetailService
{
    @Resource
    private EtDividendDetailMapper etDividendDetailMapper;

    /**
     * 查询分账明细
     *
     * @param id 分账明细主键
     * @return 分账明细
     */
    @Override
    public EtDividendDetail selectEtDividendDetailById(Long id)
    {
        return etDividendDetailMapper.selectEtDividendDetailById(id);
    }

    @Override
    public List<EtDividendDetail> selectEtDividendDetailByOrderNo(String orderNo) {
        return etDividendDetailMapper.selectEtDividendDetailByOrderNo(orderNo);
    }

    /**
     * 查询分账明细列表
     *
     * @param etDividendDetail 分账明细
     * @return 分账明细
     */
    @Override
    public List<EtDividendDetail> selectEtDividendDetailList(EtDividendDetail etDividendDetail)
    {
        return etDividendDetailMapper.selectEtDividendDetailList(etDividendDetail);
    }

    /**
     * 新增分账明细
     *
     * @param etDividendDetail 分账明细
     * @return 结果
     */
    @Override
    public int insertEtDividendDetail(EtDividendDetail etDividendDetail)
    {
        etDividendDetail.setCreateTime(DateUtils.getNowDate());
        return etDividendDetailMapper.insertEtDividendDetail(etDividendDetail);
    }

    /**
     * 修改分账明细
     *
     * @param etDividendDetail 分账明细
     * @return 结果
     */
    @Override
    public int updateEtDividendDetail(EtDividendDetail etDividendDetail)
    {
        return etDividendDetailMapper.updateEtDividendDetail(etDividendDetail);
    }

    /**
     * 批量删除分账明细
     *
     * @param ids 需要删除的分账明细主键
     * @return 结果
     */
    @Override
    public int deleteEtDividendDetailByIds(Long[] ids)
    {
        return etDividendDetailMapper.deleteEtDividendDetailByIds(ids);
    }

    /**
     * 删除分账明细信息
     *
     * @param id 分账明细主键
     * @return 结果
     */
    @Override
    public int deleteEtDividendDetailById(Long id)
    {
        return etDividendDetailMapper.deleteEtDividendDetailById(id);
    }

    /**
     * 判断是否已经计算过
     * @param date
     * @return
     */
    @Override
    public boolean isDividendComputed(String date) {
        if(etDividendDetailMapper.isDividendComputed(date) > 0){
            return true;
        }
        return false;
    }

    /**
     * 判断是否已经计算过
     * @param orderNo
     * @return
     */
    @Override
    public boolean isDividendComputedByOrderNo(String orderNo) {
        List<EtDividendDetail> etDividendDetails = etDividendDetailMapper.selectEtDividendDetailByOrderNo(orderNo);
        if(ObjectUtil.isNotNull(etDividendDetails) && etDividendDetails.size() > 0){
            return true;
        }
        return false;
    }


}
