package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.system.domain.*;
import com.ruoyi.system.mapper.AsDeviceMapper;
import com.ruoyi.system.mapper.AsUserMapper;
import com.ruoyi.system.mapper.EtFaultMapper;
import com.ruoyi.system.service.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 故障上报Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-04-06
 */
@Slf4j
@Service
public class EtFaultServiceImpl implements IEtFaultService
{
    @Resource
    private EtFaultMapper etFaultMapper;

    @Resource
    private AsUserMapper asUserMapper;

    @Resource
    private ISysDictDataService sysDictDataService;

    @Resource
    private IAsDeviceService asDeviceService;

    @Autowired
    private IEtAdminOrderService etAdminOrderService;

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private IEtAdminOrderHistoryService etAdminOrderHistoryService;

    @Resource
    private AsDeviceMapper asDeviceMapper;



    /**
     * 查询故障上报
     *
     * @param faultId 故障上报主键
     * @return 故障上报
     */
    @Override
    public EtFault selectEtFaultByFaultId(Long faultId)
    {
        EtFault etFault = etFaultMapper.selectEtFaultByFaultId(faultId);
        if(ObjectUtil.isNotNull(etFault)){
            showType(etFault);
        }
        return etFault;
    }

    @Override
    public void showType(EtFault etFault) {
        String type = etFault.getType();
        if(StringUtils.isNotEmpty(type)){
            String[] split = type.split(",");
            StringBuilder types = new StringBuilder();
            List<String> types2 = new ArrayList<>();
            for (int i = 0; i < split.length; i++) {
                String s = split[i];
                String typeName = sysDictDataService.selectDictLabel("et_fault_type", s);
                types.append(typeName);
                types2.add(s);
                if (i < split.length - 1) {
                    types.append(",");
                }
            }
            etFault.setTypeStr(types.toString());
            etFault.setTypes(types2);
        }else{
            etFault.setTypes(new ArrayList<>());
        }
    }

    /**
     * 查询故障上报列表
     *
     * @param etFault 故障上报
     * @return 故障上报
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<EtFault> selectEtFaultList(EtFault etFault)
    {
        List<EtFault> etFaults = etFaultMapper.selectEtFaultList(etFault);
        for(EtFault etFault1:etFaults){
            AsUser asUser = asUserMapper.selectUserById(etFault1.getUserId());
            if(ObjectUtils.isNotEmpty(asUser)){
                etFault1.setUserName(asUser.getUserName());
            }
            showType(etFault1);
        }
        return etFaults;
    }

    /**
     * 新增故障上报
     *
     * @param etFault 故障上报
     * @return 结果
     */
    @Override
    public int insertEtFault(EtFault etFault)
    {
        AsDevice asDevice = asDeviceService.selectAsDeviceBySn(etFault.getSn());
        if(ObjectUtils.isEmpty(asDevice)){
           throw new ServiceException("该设备不存在");
        }
        //解析地址，将地址保存数据库
//        if(StrUtil.isNotBlank(asDevice.getLongitude()) && StrUtil.isNotBlank(asDevice.getLatitude())){
//            etFault.setAddress(CommonUtil.getAddressByGeo(asDevice.getLongitude() + ","+asDevice.getLatitude()));
//        }
        etFault.setAreaId(asDevice.getAreaId());
        etFault.setCreateTime(DateUtils.getNowDate());
        setType(etFault);
        int i = etFaultMapper.insertEtFault(etFault);
        return i;
    }

    private void setType(EtFault etFault) {
        List<String> types = etFault.getTypes();
        log.info("故障上报获取到的types=:{}", JSON.toJSON(types));
        if(ObjectUtils.isNotEmpty(types)){
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < types.size(); i++) {
                sb.append(types.get(i));
                // 判断是否是最后一个元素，如果不是，则添加逗号
                if (i < types.size() - 1) {
                    sb.append(",");
                }
            }
            etFault.setType(sb.toString());
        }else{
            etFault.setType("");
        }
    }

    /**
     * 修改故障上报
     *
     * @param etFault 故障上报
     * @return 结果
     */
    @Transactional
    @Override
    public int updateEtFault(EtFault etFault)
    {
        //如果工单通过后，创建维修工单并自动分配维修员
        if("2".equals(etFault.getStatus())){
            String orderNo = IdUtils.getOrderNo("wx");
            EtAdminOrder adminOrder = new EtAdminOrder();
            adminOrder.setOrderNo(orderNo);
            adminOrder.setSn(etFault.getSn());
            adminOrder.setType("1");
            adminOrder.setStatus("3");
            adminOrder.setIsEffective("1");
            SysUser sysUser = sysUserService.selectUserByUserName("wx");//获取用户名为“wx”的维修人员
            adminOrder.setAdminId(sysUser.getUserId());
            adminOrder.setAdminName(sysUser.getUserName());
            adminOrder.setFaultId(etFault.getFaultId());
            AsDevice asDevice = asDeviceMapper.selectAsDeviceBySn(etFault.getSn());
            if(ObjectUtil.isNotNull(asDevice)){
                adminOrder.setAreaId(asDevice.getAreaId());
            }
            int adminOrder1 = etAdminOrderService.insertEtAdminOrder(adminOrder);
            if(adminOrder1>0){
                log.info("工单创建成功");
            }

            /**新建工单履历记录*/
            EtAdminOrderHistory etAdminOrderHistory = new EtAdminOrderHistory();
            etAdminOrderHistory.setInfo("审核通过");
            etAdminOrderHistory.setType(ServiceConstants.ADMIN_ORDER_TYPE_REPAIR);
            etAdminOrderHistory.setOperationType(ServiceConstants.OPERATION_TYPE_SUBMIT);
            etAdminOrderHistory.setOrderNo(orderNo);
            etAdminOrderHistory.setUserId(sysUser.getUserId());
            etAdminOrderHistory.setUserName(sysUser.getUserName());
            etAdminOrderHistory.setStatus(ServiceConstants.REPLACEMENT_ELECTRICITY_STATUS_REPAIR);
            int i = etAdminOrderHistoryService.insertEtAdminOrderHistory(etAdminOrderHistory);
            if(i>0){
                log.info("工单履历记录成功");
            }
        }
        return etFaultMapper.updateEtFault(etFault);
    }

    /**
     * 批量删除故障上报
     *
     * @param faultIds 需要删除的故障上报主键
     * @return 结果
     */
    @Override
    public int deleteEtFaultByFaultIds(Long[] faultIds)
    {
        return etFaultMapper.deleteEtFaultByFaultIds(faultIds);
    }

    /**
     * 删除故障上报信息
     *
     * @param faultId 故障上报主键
     * @return 结果
     */
    @Override
    public int deleteEtFaultByFaultId(Long faultId)
    {
        return etFaultMapper.deleteEtFaultByFaultId(faultId);
    }
}
