package com.ruoyi.system.service.impl;

import java.util.List;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.system.domain.EtAreaRule;
import com.ruoyi.system.domain.EtOperatingArea;
import com.ruoyi.system.domain.vo.IntervalRuleVo;
import com.ruoyi.system.domain.vo.StartingRuleVo;
import com.ruoyi.system.mapper.EtAreaRuleMapper;
import com.ruoyi.system.service.IEtOperatingAreaService;
import com.ruoyi.system.service.IWxPayService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.EtFeeRuleMapper;
import com.ruoyi.system.domain.EtFeeRule;
import com.ruoyi.system.service.IEtFeeRuleService;

import javax.annotation.Resource;

/**
 * 收费方式Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-04-08
 */
@Service
public class EtFeeRuleServiceImpl implements IEtFeeRuleService
{
    @Resource
    private EtFeeRuleMapper etFeeRuleMapper;

    @Autowired
    private IEtOperatingAreaService etOperatingAreaService;

    @Autowired
    private IWxPayService wxPayService;

    @Resource
    private EtAreaRuleMapper etAreaRuleMapper;

    /**
     * 查询收费方式
     *
     * @param ruleId 收费方式主键
     * @return 收费方式
     */
    @Override
    public EtFeeRule selectEtFeeRuleByRuleId(Long ruleId)
    {
        EtFeeRule etFeeRule = etFeeRuleMapper.selectEtFeeRuleByRuleId(ruleId);
        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(etFeeRule.getAreaId());
        if(ObjectUtil.isNotNull(area)){
            etFeeRule.setArea(area);
            etFeeRule.setAreaId(area.getAreaId());
        }
        return etFeeRule;
    }

    /**
     * 查询收费方式
     *
     * @param ruleId 收费方式主键
     * @return 收费方式
     */
    @Override
    public EtFeeRule selectEtFeeRuleByRuleIdIncludeDelete(Long ruleId)
    {
        return etFeeRuleMapper.selectEtFeeRuleByRuleIdIncludeDelete(ruleId);
    }

    /**
     * 查询收费方式列表
     *
     * @param etFeeRule 收费方式
     * @return 收费方式
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<EtFeeRule> selectEtFeeRuleList(EtFeeRule etFeeRule)
    {
        if(ObjectUtil.isNull(etFeeRule.getDeptId()) && ObjectUtil.isNotNull(etFeeRule.getAreaId())){
            SysDept sysDept = wxPayService.getDeptObjByAreaId(etFeeRule.getAreaId());
            etFeeRule.setDeptId(sysDept.getDeptId());
        }
        List<EtFeeRule> etFeeRules = etFeeRuleMapper.selectEtFeeRuleList(etFeeRule);
        for (EtFeeRule rule:etFeeRules) {
            StringBuilder sb = new StringBuilder();
            String ridingRuleJson = rule.getRidingRuleJson();
            String unit = "";
            if(rule.getRidingRule().equals("1")){
                StartingRuleVo startingRule = JSONObject.parseObject(ridingRuleJson, StartingRuleVo.class);
                String rentalUnit = rule.getRentalUnit();
                if(rentalUnit.equals(ServiceConstants.RENTAL_UNIT_MINUTES)){
                    unit = "分钟";
                }else if(rentalUnit.equals(ServiceConstants.RENTAL_UNIT_HOURS)){
                    unit = "小时";
                }
                sb.append("起步价："+startingRule.getStartingPrice()+"元(含"+startingRule.getStartingTime()+unit+"),");
                sb.append("超出价："+startingRule.getTimeoutPrice()+"元/"+startingRule.getTimeoutTime()+unit);
            }
            rule.setDescription(sb.toString());
        }
        return etFeeRules;
    }

    /**
     * 根据运营区获取收费方式列表
     *
     * @param areaId 运营区id
     * @return 结果
     */
    @Override
    public List<EtFeeRule> selectEtFeeRuleListByAreaId(Long areaId) {
        return etFeeRuleMapper.selectRuleInfoListByAreaId(areaId);
    }

    /**
     * 新增收费方式
     *
     * @param etFeeRule 收费方式
     * @return 结果
     */
    @Override
    public int insertEtFeeRule(EtFeeRule etFeeRule)
    {
        if(etFeeRule.getDeptId() == null){
            SysUser currentUser = SecurityUtils.getLoginUser().getUser();//获取当前系统用户
            if(!currentUser.isAdmin()){
                etFeeRule.setDeptId(currentUser.getDeptId());
            }
        }
        etFeeRule.setCreateTime(DateUtils.getNowDate());
        etFeeRule.setStatus("0");
        int i = etFeeRuleMapper.insertEtFeeRule(etFeeRule);
        if(ObjectUtil.isNotNull(etFeeRule.getAreaId())){
            etAreaRuleMapper.insert(EtAreaRule.builder().areaId(etFeeRule.getAreaId()).ruleId(etFeeRule.getRuleId()).build());
        }
        return i;
    }

    /**
     * 修改收费方式
     *
     * @param etFeeRule 收费方式
     * @return 结果
     */
    @Override
    public int updateEtFeeRule(EtFeeRule etFeeRule)
    {
        int i = etAreaRuleMapper.deleteAreaRuleByRuleId(etFeeRule.getRuleId());
        if(ObjectUtil.isNotNull(etFeeRule.getAreaId())){
            etAreaRuleMapper.insert(EtAreaRule.builder().areaId(etFeeRule.getAreaId()).ruleId(etFeeRule.getRuleId()).build());
        }
        return etFeeRuleMapper.updateEtFeeRule(etFeeRule);
    }

    /**
     * 批量删除收费方式
     *
     * @param ruleIds 需要删除的收费方式主键
     * @return 结果
     */
    @Override
    public int deleteEtFeeRuleByRuleIds(Long[] ruleIds)
    {
        return etFeeRuleMapper.deleteEtFeeRuleByRuleIds(ruleIds);
    }

    /**
     * 删除收费方式信息
     *
     * @param ruleId 收费方式主键
     * @return 结果
     */
    @Override
    public int deleteEtFeeRuleByRuleId(Long ruleId)
    {
        return etFeeRuleMapper.deleteEtFeeRuleByRuleId(ruleId);
    }

    /**
     * 根据区域ID获取收费方式选择框列表
     *
     * @param areaId 用户ID
     * @return 选中收费方式ID列表
     */
    @Override
    public List<Long> selectRuleListByAreaId(Long areaId) {
        return etFeeRuleMapper.selectRuleListByAreaId(areaId);
    }

    /**
     * 根据区域ID获取收费方式名称选择框列表
     *
     * @param areaId 用户ID
     * @return 选中收费方式名称列表
     */
    @Override
    public List<String> selectRuleNameListByAreaId(Long areaId) {
        return etFeeRuleMapper.selectRuleNameListByAreaId(areaId);
    }
}
