package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.domain.EtHardwareVersion;
import com.ruoyi.system.mapper.AsDeviceMapper;
import com.ruoyi.system.mapper.EtHardwareVersionMapper;
import com.ruoyi.system.service.IEtHardwareVersionService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * 硬件版本Service业务层处理
 *
 * @author qiuzhenzhao
 * @date 2024-07-22
 */
@Service
public class EtHardwareVersionServiceImpl implements IEtHardwareVersionService
{
    @Resource
    private EtHardwareVersionMapper etHardwareVersionMapper;

    @Resource
    private AsDeviceMapper asDeviceMapper;

    /**
     * 查询硬件版本
     *
     * @param id 硬件版本主键
     * @return 硬件版本
     */
    @Override
    public EtHardwareVersion selectEtHardwareVersionById(Long id)
    {
        return etHardwareVersionMapper.selectEtHardwareVersionById(id);
    }

    /**
     * 查询硬件版本列表
     *
     * @param etHardwareVersion 硬件版本
     * @return 硬件版本
     */
    @Override
    public List<EtHardwareVersion> selectEtHardwareVersionList(EtHardwareVersion etHardwareVersion)
    {
        List<EtHardwareVersion> etHardwareVersions = etHardwareVersionMapper.selectEtHardwareVersionList(etHardwareVersion);
        for (EtHardwareVersion etHardwareVersion1:etHardwareVersions) {
            if(etHardwareVersion1.getParentId() != 0){
                int boundNum = asDeviceMapper.countBoundNum(etHardwareVersion1.getId());// 已绑定
                etHardwareVersion1.setBound(boundNum);
                int unBoundNum = asDeviceMapper.countUnBoundNum(etHardwareVersion1.getId());// 未绑定
                etHardwareVersion1.setUnBound(unBoundNum);
                int entered = boundNum+ unBoundNum;// 已录入
                etHardwareVersion1.setEntered(entered);
            }
        }
        // 统计 parentId == 0 的对象的 totalBound, totalUnBound 和 totalEntered
        for (EtHardwareVersion parentVersion : etHardwareVersions) {
            if (parentVersion.getParentId() == 0) {
                int totalBound = 0;
                int totalUnBound = 0;
                int totalEntered = 0;

                for (EtHardwareVersion childVersion : etHardwareVersions) {
                    if (childVersion.getParentId() == parentVersion.getId()) {
                        totalBound += childVersion.getBound();
                        totalUnBound += childVersion.getUnBound();
                        totalEntered += childVersion.getEntered();
                    }
                }
                parentVersion.setBound(totalBound);
                parentVersion.setUnBound(totalUnBound);
                parentVersion.setEntered(totalEntered);
            }
        }
        // 计算丢损数量
        for (EtHardwareVersion parentVersion : etHardwareVersions) {
            if (parentVersion.getParentId() == 0) {
                parentVersion.setLost(parentVersion.getQuantity()-parentVersion.getEntered()-parentVersion.getNotEnteredNum());
            }
        }
        return etHardwareVersions;
    }

    /**
     * 查询硬件版本列表(排除父节点)
     *
     * @param etHardwareVersion 硬件版本
     * @return 硬件版本
     */
    @Override
    public List<EtHardwareVersion> selectEtHardwareVersionListExclude(EtHardwareVersion etHardwareVersion) {
        List<EtHardwareVersion> etHardwareVersions = etHardwareVersionMapper.selectEtHardwareVersionListExclude(etHardwareVersion);
        return etHardwareVersions;
    }

    /**
     * 新增硬件版本
     *
     * @param etHardwareVersion 硬件版本
     * @return 结果
     */
    @Override
    public int insertEtHardwareVersion(EtHardwareVersion etHardwareVersion)
    {
        if(ObjectUtil.isNull(etHardwareVersion.getParentId()) || etHardwareVersion.getParentId() == 0){
            etHardwareVersion.setAncestors("0");
        }else{
            EtHardwareVersion info = etHardwareVersionMapper.selectEtHardwareVersionById(etHardwareVersion.getParentId());
            etHardwareVersion.setAncestors(info.getAncestors() + "," + etHardwareVersion.getParentId());
        }
        etHardwareVersion.setCreateTime(DateUtils.getNowDate());
        return etHardwareVersionMapper.insertEtHardwareVersion(etHardwareVersion);
    }

    /**
     * 修改硬件版本
     *
     * @param etHardwareVersion 硬件版本
     * @return 结果
     */
    @Override
    public int updateEtHardwareVersion(EtHardwareVersion etHardwareVersion)
    {
        return etHardwareVersionMapper.updateEtHardwareVersion(etHardwareVersion);
    }

    /**
     * 批量删除硬件版本
     *
     * @param ids 需要删除的硬件版本主键
     * @return 结果
     */
    @Override
    public int deleteEtHardwareVersionByIds(Long[] ids)
    {
        return etHardwareVersionMapper.deleteEtHardwareVersionByIds(ids);
    }

    /**
     * 删除硬件版本信息
     *
     * @param id 硬件版本主键
     * @return 结果
     */
    @Override
    public int deleteEtHardwareVersionById(Long id)
    {
        return etHardwareVersionMapper.deleteEtHardwareVersionById(id);
    }
}
