package com.ruoyi.system.service.impl;

import java.util.List;
import com.ruoyi.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.EtOnlineLogMapper;
import com.ruoyi.system.domain.EtOnlineLog;
import com.ruoyi.system.service.IEtOnlineLogService;

/**
 * 在线/离线日志Service业务层处理
 * 
 * @author 邱贞招
 * @date 2024-07-13
 */
@Service
public class EtOnlineLogServiceImpl implements IEtOnlineLogService 
{
    @Autowired
    private EtOnlineLogMapper etOnlineLogMapper;

    /**
     * 查询在线/离线日志
     * 
     * @param id 在线/离线日志主键
     * @return 在线/离线日志
     */
    @Override
    public EtOnlineLog selectEtOnlineLogById(Long id)
    {
        return etOnlineLogMapper.selectEtOnlineLogById(id);
    }

    /**
     * 查询在线/离线日志列表
     * 
     * @param etOnlineLog 在线/离线日志
     * @return 在线/离线日志
     */
    @Override
    public List<EtOnlineLog> selectEtOnlineLogList(EtOnlineLog etOnlineLog)
    {
        return etOnlineLogMapper.selectEtOnlineLogList(etOnlineLog);
    }

    /**
     * 新增在线/离线日志
     * 
     * @param etOnlineLog 在线/离线日志
     * @return 结果
     */
    @Override
    public int insertEtOnlineLog(EtOnlineLog etOnlineLog)
    {
        etOnlineLog.setCreateTime(DateUtils.getNowDate());
        return etOnlineLogMapper.insertEtOnlineLog(etOnlineLog);
    }

    /**
     * 修改在线/离线日志
     * 
     * @param etOnlineLog 在线/离线日志
     * @return 结果
     */
    @Override
    public int updateEtOnlineLog(EtOnlineLog etOnlineLog)
    {
        return etOnlineLogMapper.updateEtOnlineLog(etOnlineLog);
    }

    /**
     * 批量删除在线/离线日志
     * 
     * @param ids 需要删除的在线/离线日志主键
     * @return 结果
     */
    @Override
    public int deleteEtOnlineLogByIds(Long[] ids)
    {
        return etOnlineLogMapper.deleteEtOnlineLogByIds(ids);
    }

    /**
     * 删除在线/离线日志信息
     * 
     * @param id 在线/离线日志主键
     * @return 结果
     */
    @Override
    public int deleteEtOnlineLogById(Long id)
    {
        return etOnlineLogMapper.deleteEtOnlineLogById(id);
    }
}
