package com.ruoyi.system.service.impl;

import java.util.List;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.map.GeoUtils;
import com.ruoyi.system.domain.EtOperatingArea;
import com.ruoyi.system.mapper.EtOperatingAreaMapper;
import lombok.extern.slf4j.Slf4j;
import org.locationtech.jts.geom.Geometry;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.EtParkingAreaMapper;
import com.ruoyi.system.domain.EtParkingArea;
import com.ruoyi.system.service.IEtParkingAreaService;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;

import javax.annotation.Resource;

/**
 * 停车区Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-04-08
 */
@Slf4j
@Service
public class EtParkingAreaServiceImpl extends ServiceImpl<EtParkingAreaMapper, EtParkingArea> implements IEtParkingAreaService
{
    @Resource
    private EtParkingAreaMapper etParkingAreaMapper;

    /**
     * 查询停车区
     *
     * @param parkingId 停车区主键
     * @return 停车区
     */
    @Override
    public EtParkingArea selectEtParkingAreaByParkingId(Long parkingId)
    {
        return etParkingAreaMapper.selectEtParkingAreaByParkingId(parkingId);
    }

    /**
     * 查询停车区
     *
     * @param parkingId 停车区主键
     * @return 停车区
     */
    @Override
    public EtParkingArea selectEtParkingAreaByParkingId2(Long parkingId)
    {
        return etParkingAreaMapper.selectEtParkingAreaByParkingId2(parkingId);
    }

    /**
     * 查询停车区列表
     *
     * @param etParkingArea 停车区
     * @return 停车区
     */
    @Override
    public List<EtParkingArea> selectEtParkingAreaList(EtParkingArea etParkingArea)
    {
        return etParkingAreaMapper.selectEtParkingAreaList(etParkingArea);
    }

    /**
     * 新增停车区
     *
     * @param etParkingArea 停车区
     * @return 结果
     */
    @Override
    public int insertEtParkingArea(EtParkingArea etParkingArea)
    {
        etParkingArea.setCreateTime(DateUtils.getNowDate());
        /** 将边界值转Geometry对象*/
        Geometry geometry = GeoUtils.toGeometry(etParkingArea.getBoundaryStr());
        etParkingArea.setBoundary(GeoUtils.wkt(geometry));
        return etParkingAreaMapper.insert(etParkingArea);
    }

    /**
     * 修改停车区
     *
     * @param etParkingArea 停车区
     * @return 结果
     */
    @Override
    public int updateEtParkingArea(EtParkingArea etParkingArea)
    {
        /** 将边界值转Geometry对象*/
        Geometry geometry = GeoUtils.toGeometry(etParkingArea.getBoundaryStr());
        etParkingArea.setBoundary(GeoUtils.wkt(geometry));
        return etParkingAreaMapper.updateById(etParkingArea);
    }

    /**
     * 批量删除停车区
     *
     * @param parkingIds 需要删除的停车区主键
     * @return 结果
     */
    @Override
    public int deleteEtParkingAreaByParkingIds(Long[] parkingIds)
    {
        return etParkingAreaMapper.deleteEtParkingAreaByParkingIds(parkingIds);
    }

    /**
     * 删除停车区信息
     *
     * @param parkingId 停车区主键
     * @return 结果
     */
    @Override
    public int deleteEtParkingAreaByParkingId(Long parkingId)
    {
        return etParkingAreaMapper.deleteEtParkingAreaByParkingId(parkingId);
    }

    /**
     * 校验停车区是否唯一
     *
     * @param etParkingArea 运营区信息
     * @return 结果
     */
    @Override
    public boolean checkParkingNameUnique(EtParkingArea etParkingArea) {
        Long parkingId = StringUtils.isNull(etParkingArea.getParkingId()) ? -1L : etParkingArea.getParkingId();
        EtParkingArea info = etParkingAreaMapper.checkParkingNameUnique(etParkingArea.getParkingName());
        if (StringUtils.isNotNull(info) && info.getParkingId().longValue() != parkingId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 所有停车点
     */
    @Override
    public List<EtParkingArea> allParkingArea(){
        EtParkingArea parkingArea = new EtParkingArea();
        parkingArea.setStatus("0");
        List<EtParkingArea> etParkingAreas = etParkingAreaMapper.selectEtParkingAreaList(parkingArea);
        return etParkingAreas;
    }

    /**
     * 修改停车区状态
     *
     * @param area 停车区信息
     * @return 结果
     */
    @Override
    public int updateParkingStatus(EtParkingArea area) {
        return etParkingAreaMapper.updateEtParkingArea(area);
    }

    /**
     * 根据运营区域id查询运营区域下的停车点数量
     * @param areaId
     * @return
     */
    @Override
    public int selectParkingAreaNumByAreaId(Long areaId,String type){
        return etParkingAreaMapper.selectParkingAreaNumByAreaId(areaId,type);
    }


}
