package com.ruoyi.system.service.impl;

import java.util.List;

import cn.hutool.core.util.ObjectUtil;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.EtPowerReplacementMapper;
import com.ruoyi.system.domain.EtPowerReplacement;
import com.ruoyi.system.service.IEtPowerReplacementService;

import javax.annotation.Resource;

/**
 * 换电工单Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-04-28
 */
@Service
public class EtPowerReplacementServiceImpl implements IEtPowerReplacementService
{
    @Resource
    private EtPowerReplacementMapper etPowerReplacementMapper;

    @Resource
    private ISysUserService userService;

    /**
     * 查询换电工单
     *
     * @param id 换电工单主键
     * @return 换电工单
     */
    @Override
    public EtPowerReplacement selectEtPowerReplacementById(Long id)
    {
        return etPowerReplacementMapper.selectEtPowerReplacementById(id);
    }

    /**
     * 查询换电工单列表
     *
     * @param etPowerReplacement 换电工单
     * @return 换电工单
     */
    @Override
    public List<EtPowerReplacement> selectEtPowerReplacementList(EtPowerReplacement etPowerReplacement)
    {
        List<EtPowerReplacement> etPowerReplacements = etPowerReplacementMapper.selectEtPowerReplacementList(etPowerReplacement);
        for (EtPowerReplacement powerReplacement : etPowerReplacements){
            SysUser sysUser = userService.selectUserById(powerReplacement.getAdminId());
            if(ObjectUtil.isNotNull(sysUser)){
                powerReplacement.setAdminName(sysUser.getUserName());
            }
        }
        return etPowerReplacements;
    }

    /**
     * 新增换电工单
     *
     * @param etPowerReplacement 换电工单
     * @return 结果
     */
    @Override
    public int insertEtPowerReplacement(EtPowerReplacement etPowerReplacement)
    {
        etPowerReplacement.setCreateTime(DateUtils.getNowDate());
        return etPowerReplacementMapper.insertEtPowerReplacement(etPowerReplacement);
    }

    /**
     * 修改换电工单
     *
     * @param etPowerReplacement 换电工单
     * @return 结果
     */
    @Override
    public int updateEtPowerReplacement(EtPowerReplacement etPowerReplacement)
    {
        return etPowerReplacementMapper.updateEtPowerReplacement(etPowerReplacement);
    }

    /**
     * 批量删除换电工单
     *
     * @param ids 需要删除的换电工单主键
     * @return 结果
     */
    @Override
    public int deleteEtPowerReplacementByIds(Long[] ids)
    {
        return etPowerReplacementMapper.deleteEtPowerReplacementByIds(ids);
    }

    /**
     * 删除换电工单信息
     *
     * @param id 换电工单主键
     * @return 结果
     */
    @Override
    public int deleteEtPowerReplacementById(Long id)
    {
        return etPowerReplacementMapper.deleteEtPowerReplacementById(id);
    }

    /**
     * 该mac是否有换电单
     * @param mac
     * @return
     */
    @Override
    public Boolean checkOrderUnique(String mac) {
        Integer integer = etPowerReplacementMapper.checkOrderUnique(mac);
        return integer>0;
    }


}
