package com.ruoyi.system.service.impl;

import java.util.List;
import com.ruoyi.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.EtSoftwareVersionMapper;
import com.ruoyi.system.domain.EtSoftwareVersion;
import com.ruoyi.system.service.IEtSoftwareVersionService;

/**
 * 软件版本Service业务层处理
 * 
 * @author qiuzhenzhao
 * @date 2024-07-22
 */
@Service
public class EtSoftwareVersionServiceImpl implements IEtSoftwareVersionService 
{
    @Autowired
    private EtSoftwareVersionMapper etSoftwareVersionMapper;

    /**
     * 查询软件版本
     * 
     * @param id 软件版本主键
     * @return 软件版本
     */
    @Override
    public EtSoftwareVersion selectEtSoftwareVersionById(Long id)
    {
        return etSoftwareVersionMapper.selectEtSoftwareVersionById(id);
    }

    /**
     * 查询软件版本列表
     * 
     * @param etSoftwareVersion 软件版本
     * @return 软件版本
     */
    @Override
    public List<EtSoftwareVersion> selectEtSoftwareVersionList(EtSoftwareVersion etSoftwareVersion)
    {
        return etSoftwareVersionMapper.selectEtSoftwareVersionList(etSoftwareVersion);
    }

    /**
     * 新增软件版本
     * 
     * @param etSoftwareVersion 软件版本
     * @return 结果
     */
    @Override
    public int insertEtSoftwareVersion(EtSoftwareVersion etSoftwareVersion)
    {
        etSoftwareVersion.setCreateTime(DateUtils.getNowDate());
        return etSoftwareVersionMapper.insertEtSoftwareVersion(etSoftwareVersion);
    }

    /**
     * 修改软件版本
     * 
     * @param etSoftwareVersion 软件版本
     * @return 结果
     */
    @Override
    public int updateEtSoftwareVersion(EtSoftwareVersion etSoftwareVersion)
    {
        etSoftwareVersion.setUpdateTime(DateUtils.getNowDate());
        return etSoftwareVersionMapper.updateEtSoftwareVersion(etSoftwareVersion);
    }

    /**
     * 批量删除软件版本
     * 
     * @param ids 需要删除的软件版本主键
     * @return 结果
     */
    @Override
    public int deleteEtSoftwareVersionByIds(Long[] ids)
    {
        return etSoftwareVersionMapper.deleteEtSoftwareVersionByIds(ids);
    }

    /**
     * 删除软件版本信息
     * 
     * @param id 软件版本主键
     * @return 结果
     */
    @Override
    public int deleteEtSoftwareVersionById(Long id)
    {
        return etSoftwareVersionMapper.deleteEtSoftwareVersionById(id);
    }
}
