package com.ruoyi.system.service.impl;

import java.util.List;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.CommonUtil;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.domain.AsDevice;
import com.ruoyi.system.domain.EtOrder;
import com.ruoyi.system.mapper.AsDeviceMapper;
import com.ruoyi.system.service.IAsDeviceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.EtTripLogMapper;
import com.ruoyi.system.domain.EtTripLog;
import com.ruoyi.system.service.IEtTripLogService;

import javax.annotation.Resource;

/**
 * 行程记录Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-05-13
 */
@Slf4j
@Service
public class EtTripLogServiceImpl implements IEtTripLogService
{
    @Autowired
    private EtTripLogMapper etTripLogMapper;

    @Autowired
    private IAsDeviceService asDeviceService;

    @Resource
    private AsDeviceMapper asDeviceMapper;


    /**
     * 查询行程记录
     *
     * @param tripId 行程记录主键
     * @return 行程记录
     */
    @Override
    public EtTripLog selectEtTripLogByTripId(Long tripId)
    {
        return etTripLogMapper.selectEtTripLogByTripId(tripId);
    }

    /**
     * 查询行程记录列表
     *
     * @param etTripLog 行程记录
     * @return 行程记录
     */
    @Override
    public List<EtTripLog> selectEtTripLogList(EtTripLog etTripLog)
    {
        return etTripLogMapper.selectEtTripLogList(etTripLog);
    }

    /**
     * 新增行程记录
     *
     * @param etTripLog 行程记录
     * @return 结果
     */
    @Override
    public int insertEtTripLog(EtTripLog etTripLog)
    {
        etTripLog.setCreateTime(DateUtils.getNowDate());
        return etTripLogMapper.insertEtTripLog(etTripLog);
    }

    /**
     * 修改行程记录
     *
     * @param etTripLog 行程记录
     * @return 结果
     */
    @Override
    public int updateEtTripLog(EtTripLog etTripLog)
    {
        return etTripLogMapper.updateEtTripLog(etTripLog);
    }

    /**
     * 批量删除行程记录
     *
     * @param tripIds 需要删除的行程记录主键
     * @return 结果
     */
    @Override
    public int deleteEtTripLogByTripIds(Long[] tripIds)
    {
        return etTripLogMapper.deleteEtTripLogByTripIds(tripIds);
    }

    /**
     * 删除行程记录信息
     *
     * @param tripId 行程记录主键
     * @return 结果
     */
    @Override
    public int deleteEtTripLogByTripId(Long tripId)
    {
        return etTripLogMapper.deleteEtTripLogByTripId(tripId);
    }


    public EtTripLog createTripLog(String orderNo,String sn,String type) {
        EtTripLog etTripLog = new EtTripLog();
        etTripLog.setType(type);
        etTripLog.setOrderNo(orderNo);
        AsDevice asDevice = null;
        if(StrUtil.isNotBlank(sn)){
            asDevice = asDeviceMapper.selectAsDeviceBySn(sn);
            etTripLog.setLatitude(asDevice.getLatitude());
            etTripLog.setLongitude(asDevice.getLongitude());
            String location = asDevice.getLongitude() + ","+asDevice.getLatitude();
//            etTripLog.setAddress(CommonUtil.getAddressByGeo(location));
            etTripLog.setSn(sn);
        }
        log.info("创建行程记录:{}", JSON.toJSON(etTripLog));
        return etTripLog;
    }

    /**
     * 创建行程记录并保存
     * @param orderNo
     * @return
     */
    public int tripLog(String orderNo,String sn,String type){
        EtTripLog tripLog = createTripLog(orderNo,sn,type);
        return insertEtTripLog(tripLog);
    }
}
