package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.aliyuncs.dysmsapi.model.v20170525.SendSmsResponse;
import com.aliyuncs.exceptions.ClientException;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.domain.TreeSelect;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysRole;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.*;
import com.ruoyi.common.utils.spring.SpringUtils;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.system.domain.*;
import com.ruoyi.system.mapper.*;
import com.ruoyi.system.service.*;
import com.wechat.pay.java.service.profitsharing.model.AddReceiverResponse;
import com.wechat.pay.java.service.profitsharing.model.DeleteReceiverResponse;
import com.wechat.pay.java.service.transferbatch.model.TransferDetailInput;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 部门管理 服务实现
 *
 * @author ruoyi
 */
@Slf4j
@Service
public class SysDeptServiceImpl implements ISysDeptService
{
    @Resource
    private SysDeptMapper deptMapper;

    @Resource
    private SysRoleMapper roleMapper;

    @Resource
    private EtAreaDeptMapper etAreaDeptMapper;

    @Resource
    private SysUserMapper userMapper;

    @Autowired
    private ISysConfigService configService;

    @Resource
    private SysUserRoleMapper userRoleMapper;

    @Autowired
    private IWxPayService wxPayService;

    @Autowired
    private IAsUserService asUserService;

    @Resource
    private IEtOperatingAreaService etOperatingAreaService;

    @Autowired
    private CallbackService callbackService;

    @Resource
    private EtCapitalFlowMapper etCapitalFlowMapper;

    @Resource
    private AsUserMapper asUserMapper;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Value("${aliyun.accessKeyId}")
    private String accessKeyId;

    @Value("${aliyun.accessKeySecret}")
    private String accessKeySecret;

    @Value("${aliyun.signName}")
    private String signName;

    @Value("${aliyun.templateCode3}")
    private String templateCode3;

    @Value("${aliyun.phone}")
    private String phone;

    @Resource
    private EtMsgLogMapper etMsgLogMapper;

    /**
     * 查询部门管理数据
     *
     * @param dept 部门信息
     * @return 部门信息集合
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<SysDept> selectDeptList(SysDept dept)
    {
        return deptMapper.selectDeptList(dept);
    }

    /**
     * 查询部门树结构信息
     *
     * @param dept 部门信息
     * @return 部门树信息集合
     */
    @Override
    public List<TreeSelect> selectDeptTreeList(SysDept dept)
    {
        List<SysDept> depts = SpringUtils.getAopProxy(this).selectDeptList(dept);
        return buildDeptTreeSelect(depts);
    }

    /**
     * 构建前端所需要树结构
     *
     * @param depts 部门列表
     * @return 树结构列表
     */
    @Override
    public List<SysDept> buildDeptTree(List<SysDept> depts)
    {
        List<SysDept> returnList = new ArrayList<SysDept>();
        List<Long> tempList = depts.stream().map(SysDept::getDeptId).collect(Collectors.toList());
        for (SysDept dept : depts)
        {
            // 如果是顶级节点, 遍历该父节点的所有子节点
            if (!tempList.contains(dept.getParentId()))
            {
                recursionFn(depts, dept);
                returnList.add(dept);
            }
        }
        if (returnList.isEmpty())
        {
            returnList = depts;
        }
        return returnList;
    }

    /**
     * 构建前端所需要下拉树结构
     *
     * @param depts 部门列表
     * @return 下拉树结构列表
     */
    @Override
    public List<TreeSelect> buildDeptTreeSelect(List<SysDept> depts)
    {
        List<SysDept> deptTrees = buildDeptTree(depts);
        return deptTrees.stream().map(TreeSelect::new).collect(Collectors.toList());
    }

    /**
     * 根据角色ID查询部门树信息
     *
     * @param roleId 角色ID
     * @return 选中部门列表
     */
    @Override
    public List<Long> selectDeptListByRoleId(Long roleId)
    {
        SysRole role = roleMapper.selectRoleById(roleId);
        return deptMapper.selectDeptListByRoleId(roleId, role.isDeptCheckStrictly());
    }

    /**
     * 根据部门ID查询信息
     *
     * @param deptId 部门ID
     * @return 部门信息
     */
    @Override
    public SysDept selectDeptById(Long deptId)
    {
        SysDept sysDept = deptMapper.selectDeptById(deptId);
        if(ObjectUtil.isNotNull(sysDept.getAppUserId())){
            AsUser asUser = asUserService.selectUserById(sysDept.getAppUserId());
            sysDept.setUserName(asUser.getUserName());
        }
        return sysDept;
    }

    /**
     * 根据app用户id查询信息
     *
     * @param appUserId app用户id
     * @return 部门信息
     */
    @Override
    public SysDept selectDeptByAppUserId(Long appUserId)
    {
        return deptMapper.selectDeptByAppUserId(appUserId);
    }

    /**
     * 根据ID查询所有子部门（正常状态）
     *
     * @param deptId 部门ID
     * @return 子部门数
     */
    @Override
    public int selectNormalChildrenDeptById(Long deptId)
    {
        return deptMapper.selectNormalChildrenDeptById(deptId);
    }

    /**
     * 是否存在子节点
     *
     * @param deptId 部门ID
     * @return 结果
     */
    @Override
    public boolean hasChildByDeptId(Long deptId)
    {
        int result = deptMapper.hasChildByDeptId(deptId);
        return result > 0;
    }

    /**
     * 查询部门是否存在用户
     *
     * @param deptId 部门ID
     * @return 结果 true 存在 false 不存在
     */
    @Override
    public boolean checkDeptExistUser(Long deptId)
    {
        int result = deptMapper.checkDeptExistUser(deptId);
        return result > 0;
    }

    /**
     * 查询部门是否存在运营区
     *
     * @param deptId 部门ID
     * @return 结果 true 存在 false 不存在
     */
    @Override
    public boolean checkDeptExistArea(Long deptId)
    {
        int result = deptMapper.checkDeptExistArea(deptId);
        return result > 0;
    }

    /**
     * 查询部门是否存在收费方式
     *
     * @param deptId 部门ID
     * @return 结果 true 存在 false 不存在
     */
    @Override
    public boolean checkDeptExistFeeRule(Long deptId)
    {
        int result = deptMapper.checkDeptExistFeeRule(deptId);
        return result > 0;
    }


    /**
     * 校验部门名称是否唯一
     *
     * @param dept 部门信息
     * @return 结果
     */
    @Override
    public boolean checkDeptNameUnique(SysDept dept)
    {
        Long deptId = StringUtils.isNull(dept.getDeptId()) ? -1L : dept.getDeptId();
        SysDept info = deptMapper.checkDeptNameUnique(dept.getDeptName(), dept.getParentId());
        if (StringUtils.isNotNull(info) && info.getDeptId().longValue() != deptId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验部门是否有数据权限
     *
     * @param deptId 部门id
     */
    @Override
    public void checkDeptDataScope(Long deptId)
    {
        if (!SysUser.isAdmin(SecurityUtils.getUserId()))
        {
            SysDept dept = new SysDept();
            dept.setDeptId(deptId);
            List<SysDept> depts = SpringUtils.getAopProxy(this).selectDeptList(dept);
            if (StringUtils.isEmpty(depts))
            {
                throw new ServiceException("没有权限访问部门数据！");
            }
        }
    }

    /**
     * 新增保存部门信息
     *  分账  partner_dividend   对账  reconciliation  dividend amount 金额  分成  Divide amount  总金额
     * @param dept 部门信息
     * @return 结果
     */
    @Override
    @Transactional
    public int insertDept(SysDept dept)
    {
        SysDept info = deptMapper.selectDeptById(dept.getParentId());
        // 如果父节点不为正常状态,则不允许新增子节点
        if (!UserConstants.DEPT_NORMAL.equals(info.getStatus()))
        {
            throw new ServiceException("部门停用，不允许新增");
        }
        dept.setAncestors(info.getAncestors() + "," + dept.getParentId());
        if (StrUtil.isNotBlank(dept.getIsUsePlatformApp()) && dept.getIsUsePlatformApp().equals("true")) {
            SysDept platform = deptMapper.selectDeptById(100L);
            dept.setAppid(platform.getAppid());
            dept.setAppSecret(platform.getAppSecret());
            dept.setAppName(platform.getAppName());
            dept.setDomain(platform.getDomain());
        }
        if (dept.getSeparateAccount().equals("N")) {
            SysDept platform = deptMapper.selectDeptById(100L);
            dept.setMerchantId(platform.getMerchantId());
            dept.setApiV3Key(platform.getApiV3Key());
            dept.setNotifyUrl(platform.getNotifyUrl());
            dept.setPrivateKeyPath(platform.getPrivateKeyPath());
            dept.setMerchantSerialNumber(platform.getMerchantSerialNumber());
            dept.setRefundNotifyUrl(platform.getRefundNotifyUrl());
        }
        int i = deptMapper.insertDept(dept);
        Long[] areaIds = dept.getAreaIds();
        if (StringUtils.isNotNull(areaIds)){
            for (Long areaId:areaIds){
                etAreaDeptMapper.insert(EtAreaDept.builder().areaId(areaId).deptId(dept.getDeptId()).build());
            }
        }
        // 添加运营商账号并添加运营商角色
        createOperator(dept);
        // 是否分账
//        if("true".equals(dept.getIsProfitSharing()) && StringUtils.isNotEmpty(dept.getMerchantId()) && dept.getParentId() != 0L){
//            AddReceiverResponse addReceiverResponse = wxPayService.addReceiver(dept.getMerchantId(),dept.getDeptId(), ServiceConstants.PROFITSHARING_TYPE_PLATFORM);
//            log.info("添加分账接收方响应:【{}】", JSON.toJSON(addReceiverResponse));
//        }
        return i;
    }

    /**
     * 修改保存部门信息
     *
     * @param dept 部门信息
     * @return 结果
     */
    @Override
    @Transactional
    public int updateDept(SysDept dept)
    {
        SysDept newParentDept = deptMapper.selectDeptById(dept.getParentId());
        SysDept oldDept = deptMapper.selectDeptById(dept.getDeptId());
        if (StringUtils.isNotNull(newParentDept) && StringUtils.isNotNull(oldDept))
        {
            String newAncestors = newParentDept.getAncestors() + "," + newParentDept.getDeptId();
            String oldAncestors = oldDept.getAncestors();
            dept.setAncestors(newAncestors);
            updateDeptChildren(dept.getDeptId(), newAncestors, oldAncestors);
        }
        if (StrUtil.isNotBlank(dept.getIsUsePlatformApp()) && dept.getIsUsePlatformApp().equals("true")) {
            SysDept platform = deptMapper.selectDeptById(100L);
            dept.setAppid(platform.getAppid());
            dept.setAppSecret(platform.getAppSecret());
        }
        int result = deptMapper.updateDept(dept);
        if (UserConstants.DEPT_NORMAL.equals(dept.getStatus()) && StringUtils.isNotEmpty(dept.getAncestors())
                && !StringUtils.equals("0", dept.getAncestors()))
        {
            // 如果该部门是启用状态，则启用该部门的所有上级部门
            updateParentDeptStatusNormal(dept);
        }
        etAreaDeptMapper.deleteAreaDeptByDeptId(dept.getDeptId());
        Long[] areaIds = dept.getAreaIds();
        if (StringUtils.isNotNull(areaIds)){
            for (Long areaId:areaIds){
                etAreaDeptMapper.insert(EtAreaDept.builder().areaId(areaId).deptId(dept.getDeptId()).build());
            }
        }
        // 添加运营商账号并添加运营商角色
//        createOperator(dept);
//        if("true".equals(dept.getIsProfitSharing()) && StringUtils.isNotEmpty(dept.getMerchantId()) && dept.getParentId() != 0L && !dept.getMerchantId().equals("1656437344")){
//            // 删除分账接收方
//            DeleteReceiverResponse deleteReceiverResponse = wxPayService.deleteReceiver(dept.getMerchantId(),dept.getDeptId(),ServiceConstants.PROFITSHARING_TYPE_PLATFORM);
//            log.info("删除分账接收方响应:【{}】", JSON.toJSON(deleteReceiverResponse));
//
//            AddReceiverResponse addReceiverResponse = wxPayService.addReceiver(dept.getMerchantId(),dept.getDeptId(), ServiceConstants.PROFITSHARING_TYPE_PLATFORM);
//            log.info("添加分账接收方响应:【{}】", JSON.toJSON(addReceiverResponse));
//        }
        return result;
    }

    /**
     * 添加运营商账号
     * 创建
     * @param dept 当前部门
     */
    private void createOperator(SysDept dept) {
        // 创建运营商账号并添加运营商角色
        SysUser sysUser = new SysUser();
        sysUser.setDeptId(dept.getDeptId());
        sysUser.setUserName(dept.getPhone());
        sysUser.setNickName(dept.getPhone());
        sysUser.setPhonenumber(dept.getPhone());
        sysUser.setUserType("00");
        String password = configService.selectConfigByKey("sys.user.initPassword");
        sysUser.setPassword(SecurityUtils.encryptPassword(password));
        sysUser.setEmail(dept.getEmail());
        sysUser.setStatus("0");
        sysUser.setDelFlag("0");
        sysUser.setCreateBy(SecurityUtils.getUsername());
        sysUser.setCreateTime(DateUtils.getNowDate());
        int rows = userMapper.insertUser(sysUser);

        // 添加运营商角色
        if (rows > 0) {
            log.info("创建运营商账号成功");
            Long[] roleIds = new Long[]{3L};
            // 新增用户与角色管理
            List<SysUserRole> list = new ArrayList<>(roleIds.length);
            for (Long roleId : roleIds)
            {
                SysUserRole ur = new SysUserRole();
                ur.setUserId(sysUser.getUserId());
                ur.setRoleId(roleId);
                list.add(ur);
            }
            userRoleMapper.batchUserRole(list);
        }
    }

    /**
     * 修改该部门的父级部门状态
     *
     * @param dept 当前部门
     */
    private void updateParentDeptStatusNormal(SysDept dept)
    {
        String ancestors = dept.getAncestors();
        Long[] deptIds = Convert.toLongArray(ancestors);
        deptMapper.updateDeptStatusNormal(deptIds);
    }

    /**
     * 修改子元素关系
     *
     * @param deptId 被修改的部门ID
     * @param newAncestors 新的父ID集合
     * @param oldAncestors 旧的父ID集合
     */
    public void updateDeptChildren(Long deptId, String newAncestors, String oldAncestors)
    {
        List<SysDept> children = deptMapper.selectChildrenDeptById(deptId);
        for (SysDept child : children)
        {
            child.setAncestors(child.getAncestors().replaceFirst(oldAncestors, newAncestors));
        }
        if (children.size() > 0)
        {
            deptMapper.updateDeptChildren(children);
        }
    }

    /**
     * 删除部门管理信息
     *
     * @param deptId 部门ID
     * @return 结果
     */
    @Override
    public int deleteDeptById(Long deptId)
    {
        return deptMapper.deleteDeptById(deptId);
    }

    /**
     * 根据证书序列号获取运营商
     *
     * @param wechatpaySerial 证书序列号
     * @return 结果
     */
    @Override
    public SysDept selectEtOperatingAreaBySerialNumber(String wechatpaySerial) {
        SysDept sysDept = deptMapper.selectEtOperatingAreaBySerialNumber(wechatpaySerial);
        return sysDept;
    }

    /**
     * 获取运营商余额
     *
     * @return 结果
     */
    @Override
    public void changeDeptBalance(BigDecimal amount,Long deptId) {
        deptMapper.changeDeptBalance(amount,deptId);
    }

    /**
     * 管理员提现
     *  1. 获取当前用户
     *  2. 获取最新用户
     *  3. 获取部门对象
     *  4. 发起提现
     *  -- 5. 更新部门余额（默认通过，直接扣余额）
     *  6. 记录资金流水
     */
    @Override
    public int adminWithdraw(BigDecimal amount,SysDept sysDept,String method) {
        String orderNo = IdUtils.getOrderNo("tx");
        if(method.equals(ServiceConstants.WITHDRAW_METHOD_WX)){
            AsUser asUser1 = asUserService.selectUserById(sysDept.getAppUserId());
            log.info("【管理员提现】获取到运营商信息：【{}】", JSON.toJSON(sysDept));
            if(ObjectUtil.isNull(sysDept)){
                throw new RuntimeException("运营商未绑定app用户,请联系管理员添加【绑定微信用户】");
            }
            String batchName= sysDept.getDeptName()+"发起提现";
            List<TransferDetailInput> transferDetailInputs = new ArrayList<>();
            TransferDetailInput transferDetailInput = new TransferDetailInput();
            transferDetailInput.setOutDetailNo(IdUtils.getOrderNo("txd"));//明细订单号
            transferDetailInput.setTransferAmount(amount.multiply(new BigDecimal(100)).longValue());
            transferDetailInput.setTransferRemark(sysDept.getDeptName()+"共享电动车提现");
            transferDetailInput.setOpenid(asUser1.getWxopenid());
            transferDetailInputs.add(transferDetailInput);
            log.info("【审核通过】发起转账到零钱:{}",JSON.toJSONString(transferDetailInputs));
            // todo 暂时先不发起真实提现  真正的提现是
//        wxPayService.transfer(sysDept,orderNo,batchName,batchName,amount,1,transferDetailInputs);
        }
        //扣余额并记录资金流水
        EtOrder order = new EtOrder();
        order.setOrderNo(orderNo);
        order.setPayFee(amount);
        order.setTotalFee(amount);
        List<Long> longs = etOperatingAreaService.selectAreaListByDeptId(sysDept.getDeptId());
        if(longs.size() == 0){
            throw new ServiceException("【提现】提现失败，该运营商id:"+sysDept.getDeptId()+"下未找到该运营区");
        }
        order.setAreaId(longs.get(0));
        // 发短信
        asynchronousMsg(sysDept.getDeptName());
        //记录资金流水
        return callbackService.capitalFlowRecords2(order,ServiceConstants.FLOW_TYPE_DISBURSE,ServiceConstants.ORDER_TYPE_WITHDRAW,ServiceConstants.OWNER_TYPE_OPERATOR,sysDept);
    }

    private void asynchronousMsg(String deptName) {
        scheduledExecutorService.schedule(() -> {
            /** 提现申请发送一个短信给丁建帮*/
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("name", deptName);

            SendSmsVo sendSmsVo = new SendSmsVo();
            sendSmsVo.setMobile(phone);
            sendSmsVo.setTemplateCode(templateCode3);
            sendSmsVo.setParam(jsonObject.toJSONString());
            sendSmsVo.setSignName(signName);
            SendSmsResponse response = null;
            log.info("【提现申请】向阿里云发送短信，请求,----------【{}】", JSON.toJSONString(sendSmsVo));
            try {
                response = SendAliSmsUtil.sendVerifyCode(accessKeyId,accessKeySecret,sendSmsVo);
            } catch (ClientException e) {
                e.printStackTrace();
            }
            log.info("【提现申请】发送阿里云短信成功,返回----------【{}】",JSON.toJSONString(response));

            // 短信日志
            EtMsgLog etMsgLog = new EtMsgLog();
            etMsgLog.setPhone(phone);
            etMsgLog.setContent("用户"+deptName+"，发起押金提现申请，请尽快审核！");
            etMsgLog.setType("2");
            etMsgLog.setSignName(signName);
            etMsgLog.setTemplateCode(templateCode3);
            int i = etMsgLogMapper.insertEtMsgLog(etMsgLog);
            if(i>0){
                log.info("【提现申请】短信日志记录成功");
            }else{
                log.info("【提现申请】短信日志记录失败");
            }
        }, 0 , TimeUnit.HOURS);
    }

    /**
     * 管理员提现失败
     *  1. 生成一条相同的金额
     *  2.
     */
    @Override
    public int rejectWithdraw(Long flowId) {
        EtCapitalFlow etCapitalFlow = etCapitalFlowMapper.selectEtCapitalFlowByFlowId(flowId);
        SysDept sysDept = deptMapper.selectDeptById(etCapitalFlow.getOwnerId());
        log.info("【提现失败】获取到提现流水信息：【{}】", JSON.toJSON(etCapitalFlow));
        EtOrder order = new EtOrder();
        order.setOrderNo(etCapitalFlow.getOrderNo());
        order.setPayFee(etCapitalFlow.getAmount().add(etCapitalFlow.getHandlingCharge()));
        order.setTotalFee(etCapitalFlow.getAmount());
        List<Long> longs = etOperatingAreaService.selectAreaListByDeptId(etCapitalFlow.getOwnerId());
        if(longs.size() == 0){
            throw new ServiceException("【管理员提现失败】提现失败，未找到该运营区，运营商id:"+etCapitalFlow.getOwnerId());
        }
        order.setAreaId(longs.get(0));
        //记录资金流水
        int i = callbackService.capitalFlowRecords2(order, ServiceConstants.FLOW_TYPE_INCOME, ServiceConstants.ORDER_TYPE_WITHDRAW_FAIL, ServiceConstants.OWNER_TYPE_OPERATOR, sysDept);
        if(i>0){
            etCapitalFlow.setStatus(ServiceConstants.FLOW_STATUS_COMPLETE);
            if(etCapitalFlowMapper.updateEtCapitalFlow(etCapitalFlow) ==0){
                throw new ServiceException("【管理员提现失败】提现失败，更新资金流水表失败");
            }
        }else{
            throw new ServiceException("【管理员提现失败】提现失败，插入资金流水失败");
        }
        return i;
    }

    @Override
    public int selectAllCount() {
        return deptMapper.selectAllCount();
    }

    @Override
    public int getAppCount() {
        return deptMapper.getAppCount();
    }

    @Override
    public BigDecimal selectAllBalance() {
        return deptMapper.selectAllBalance();
    }

    /**
     * 递归列表
     */
    private void recursionFn(List<SysDept> list, SysDept t)
    {
        // 得到子节点列表
        List<SysDept> childList = getChildList(list, t);
        t.setChildren(childList);
        for (SysDept tChild : childList)
        {
            if (hasChild(list, tChild))
            {
                recursionFn(list, tChild);
            }
        }
    }

    /**
     * 得到子节点列表
     */
    private List<SysDept> getChildList(List<SysDept> list, SysDept t)
    {
        List<SysDept> tlist = new ArrayList<SysDept>();
        Iterator<SysDept> it = list.iterator();
        while (it.hasNext())
        {
            SysDept n = (SysDept) it.next();
            if (StringUtils.isNotNull(n.getParentId()) && n.getParentId().longValue() == t.getDeptId().longValue())
            {
                tlist.add(n);
            }
        }
        return tlist;
    }

    /**
     * 判断是否有子节点
     */
    private boolean hasChild(List<SysDept> list, SysDept t)
    {
        return getChildList(list, t).size() > 0;
    }

    /**
     * 绑定app用户
     */
    @Override
    public int bandAppUser(SysDept dept) {
        return  deptMapper.updateDept(dept);
    }
}
